/*
 *  Formula-Utiliteis.h
 *  Version 1.3.5 / 970409
 *  by Satoshi Takahashi
 */


#ifndef FORMULA_UTILITIES_H  /*pragma once*/
#define FORMULA_UTILITIES_H

/******************************* Utilities ***********************************/

/*------ Input ------*/

void SkipSpacesAndReadCont(void);
void ReadContIfNeed(void);
void SFStrInc(void);

int IsNameStart(char c);

/*------ Multiply ------*/

void MultiplyBases(s_f_Base * base1, s_f_Base * base2);
void MultiplyBases3(s_f_Base * base1, s_f_Base * base2, s_f_Base * base3);

#define MultiplyBlocks MultiplyPlusBlocks
s_f_FB * MultiplyPlusBlocks(s_f_FB **block1,
			s_f_FB **block2, s_f_FB ** lastFB);
s_f_FB * MultiplyMinusBlocks(s_f_FormulaBlock **block1,
			s_f_FB **block2, s_f_FB ** lastFB);
s_f_FB * MultiplySignedBlocks(s_f_FB **block1, s_f_FB **block2,
		 s_f_FB ** lastBlock,int sign);

#define MultiplyBlockAndBase(block,base,lastBlock) \
MultiplySignedBlockAndBase(block,base,lastBlock,0)
#define MultiplyPlusBlockAndBase(block,base,lastBlock) \
MultiplySignedBlockAndBase(block,base,lastBlock,PlusSign)
#define MultiplyMinusBlockAndBase(block,base,lastBlock) \
MultiplySignedBlockAndBase(block,base,lastBlock,MinusSign)
s_f_FB* MultiplySignedBlockAndBase(s_f_FB** block,
							s_f_Base** base,s_f_FB** lastBlock,int sign);

/*------ Expand and Sort ------*/

void NextFB(s_f_FB** fbp);
void CopyOneBlock(s_f_FB* curFB, s_f_FB** newTopFBP);
void AppendOneBlock(s_f_FB* curFB, s_f_FB** newLastFBP);
void AddBlocksWithLast(s_f_FB** newLastFBP,s_f_FB* curBlockFB,
										s_f_FB* curBlockLastFB);
s_f_FB* ComposeFrac(s_f_FB* numFBP,s_f_FB* denomFBP);

void InvertSigns(s_f_FB **block);
void FBSignOp(s_f_FB **retFBP);

int CompareBases(int *power1, int *power2);

s_f_FB* FindLastBlock(s_f_FB* fb);


/*------ Copy nad Make -------*/

s_f_FB* CopyFB(s_f_FB* orgBlock);
s_f_FB * CopyOneFB(s_f_FB *curFB);
s_f_Base *CopyBase(s_f_Base *orgBase);

s_f_FB* NewFormulaBlock(void);
s_f_Base* NewFormulaBase(void);
void SetBlockType(s_f_FB* fbp);
void SetBaseType(s_f_FB* fbp);

s_f_FB *NewBaseFB(void);
s_f_FB* MakeNextBaseFB(s_f_FB* fbp);

s_f_FB* MakeNextBlock(s_f_FB* fbp);
#define MakeNextFB MakeNextBlock

s_f_FB *MakeFB(double coef);

/*------ Delete and Free -------*/

void DeleteOneFB(s_f_FB **delFBP);

void FreeOneBaseFB(s_f_FB **freeFBP);
void FreeBaseFBs(s_f_FB* fb);
void FreeBaseFBsWithLast(s_f_FB *topFB, s_f_FB *lastFB);

void FreeOneFB(s_f_FB **freeFBP);

/*-------------------------- Get Parameters ---------------------------*/

int GetParamsFNum(char *inputline, int *formulaNumP);
int GetParamsFNumSNum(char *inputline, int *formulaNumP,
			int *symbolNumP);
int GetParamsFNumSNumInt(char *inputline, int *formulaNumP,
			int *symbolNumP, int *powerP);
#define GetSelectParams GetParamsFNumSNumInt

int GetParamsFNumSFNum(char *inputline, int *formulaNumP,
			int *symbolNumP, int *symbolFormulaNumP);


/*------ Other -------*/

int FormulaNumber(char *fname);
int SymbolNumber(char *sname);

int DecomposeFrac(s_f_FB **fracFB, s_f_FB **numFB, s_f_FB **denomFB);
#if 0
int GetFormulaAndSymbolNum(char *procName, char *formulaString, char *symbolString,
							int *formulaNumP, int *symbolNumP);
#endif
s_f_FB *AppendNextOneFB(s_f_FB *curFB, s_f_FB *appendFB);
s_f_FB *ComposeTwoFB(s_f_FB *firstFB, s_f_FB *secondFB, int operand);
s_f_FB *ComposeCopyTwoFB(s_f_FB *firstFB, s_f_FB *secondFB, int operand);


void InsertFB(s_f_FB *prevFB, s_f_FB **inFBP, s_f_FB *nextFB, s_f_FB **topFBP);
void ConcatFB(s_f_FB *fb1, s_f_FB *fb2);


int InSet(int n, int sum, int *array);
int OrderCompareBases(int *power1, int *power2);


#endif /*pragma once*/
