/*
 *  Formula.c
 *  Version 1.8 / 000707
 *  by Satoshi Takahashi
 */

/* to do list
divide polynomial
*/

#ifndef FORMULA_MAIN_C /*pragma once*/
#define FORMULA_MAIN_C


/*********************************** Switches ***********************************/

#include <Formula-Switches.h>


/******************************** Include Headers *******************************/

#include <string.h>
#include <stdarg.h>
#include <stdlib.h>
#include <s_Common.h>
#include <ctype.h>
#include <stdio.h>

#include <StartSCL.h>
#include <DialogFOpen.h>
#include <s_Print.h>
#include <s_String.h>
#include <s_RedrawFunc.h>
#include <DragAndDrop.h>

#include <Formula.h>
#include <Formula-Common.h>

#if 0
#include <s_undef.c>
#endif

#if PROFILE_FORMULA
#define S_PROFILE 1
#include <s_Profile.c>
#else
#define S_PROFILE 0
#include <s_Profile.h>
#endif

/******************************** Include Variables and SubFiles **********************/

#include <Formula-v.c>


/************************************* Debug Ruotine ***************************/

#if DEBUG_FORMULA
void ShowOneFB(s_f_FB* fb,char* str)
{
	ScrollFormula(2);
	MoveTo(0,s_f_fy);
	s_printf("%s\n",str);
	if (fb)
		PrintFormulaBlock(fb,0);
	else
		DrawStyleText("NULL");
	Stop();
}

void DebugCheckOrder(s_f_FB *fb)
{
	if (fb)
	{
		if (fb->next && (fb->next->prev != fb))
			df(fb,"different");
	}
}

void DebugCheckOrderFBs(s_f_FB *fb)
{
	
	if (fb)
	{
		if (fb->prev)
			df(fb->prev,"illegal prev");
	}
	
	while(fb->next)
	{
		if (fb->next && (fb->next->prev != fb))
			df(fb,"different next");
		fb = fb->next;
	}
}

void DebugCheckLast(s_f_FB *inFB, s_f_FB *lastFB)
{
s_f_FB *curFB;

	curFB = inFB;

	while(curFB->next)
		curFB = curFB->next;
	
	if (curFB != lastFB)
		s_printf("Dif last!\n");
}


#endif /* df etc has been moved to Formula.h */

/******************************* FormulaShell and Help *************************/

void FormulaShell(void)
{
//char com[S_F_MAX_INPUT_LENGTH+1];
char commandWord[256];
voidFunc *commandFunc;

	SetUpFormula();	

#if !S_UNIX
//s_printf("DDCount = %d\n", DDCount());
	if (DDCount())
	{
		SetFormulaInputFile();
	}
#endif /*!S_UNIX*/

	while(1)
	{
		if	(!ReadOneLine())
		{
			s_printf("End of File at: %s\n",s_f_inputLine);
			continue;
		}

		if (!SScanString(&s_f_str,commandWord))
			continue;

#if TEST_SC		
		if (s_f_showCommandFlag)
		{
			ScrollFormula(2);
			MoveTo(0, s_f_fy);
			s_DirectDprintf(s_f_inputLine);
			ScrollFormula(0);
		}
#endif /*TEST_SC*/
		
		commandFunc = SelectCommand(commandWord);
		(*commandFunc)();
	}
}

void WaitReturn(void)
{
int i;
char *message = "Press return to continue";

	s_printf(message);
#if !S_UNIX
	Stop();	
#else
	getchar();
#endif
	for (i= 0; i< strlen("Press return to continue"); i++)
	{
#if !S_UNIX
		TEKey('\b', s_textH);
#else
		printf("\b");
#endif
	}
}

void FormulaHelp(void)
{
	s_printf("- Help and Quit -\n");
	s_printf(" help\n");
	s_printf(" ?: help\n");
	s_printf(" quit\n");

	s_printf("- Set and Display -\n");
	s_printf(" set f = ...\n");
	s_printf(" show f\n");
	s_printf(" clear\n");

	WaitReturn();

	s_printf("\n- Calculate Formula -\n");
	s_printf(" calc f: calculate\n");
	s_printf(" substitute f x\n");
	s_printf(" copy f g\n");
	s_printf(" coef f x n: coeficient of x^n\n");
	s_printf(" select f x n: select x^n\n");
#if TEST_SOL
	s_printf(" collect f x\n");
#endif
	s_printf(" trunc f x n: truncate > x^n\n");
	s_printf(" st f x ... z n: truncate power sum > n\n");
	s_printf(" reduce f x n (f c)\n");
	s_printf(" ed f x n: expand denominator\n");
	s_printf(" denom f: denominator\n");
	s_printf(" numerator f\n");
#if TEST_SOL
	s_printf(" solve f x\n");
#endif
#if TEST_DIV
	s_printf(" divide f ...\n");
#endif
#if TEST_DIV
	s_printf(" factor f ...\n");
#endif
	s_printf(" complexclean f: calc complex formula\n");
	s_printf(" realdenom f: make denominator real\n");

	WaitReturn();

	s_printf("\n- Input Output -\n");
	s_printf(" if: input file\n");
	s_printf(" ic: input console\n");
	s_printf(" bprint: begin printing\n");
	s_printf(" eprint: end printing\n");
	s_printf(" bsave: begin saving\n");
	s_printf(" esave: end saving\n");

	WaitReturn();

	s_printf("\n- Environment Setting -\n");
	s_printf(" divlevel n: set vertical fraction level\n");
	s_printf(" decimal n: set decimal digit\n");
	s_printf(" order x ... z: set compare order\n");
	s_printf(" fmax n: formula max number\n");
	s_printf(" smax n: symbol max number\n");
#if TEST_SC
	s_printf(" command on (off): display command on-off\n");
#endif
	s_printf(" ishow on (off): display input of set formula on-off\n");
	s_printf(" display on (off): display result on-off\n");
	s_printf(" time on (off): print time on-off\n");
	s_printf(" gc off (on): garbage collection of memory\n");
	s_printf(" rm n: set right margin (obsolete)\n");

	WaitReturn();

	s_printf("\n- Information -\n");	
	s_printf(" memory\n");

	s_printf("\n- Other -\n");	
	s_printf(" comment begin (end)\n");
	s_printf(" #: comment\n");
	s_printf(" %%: comment\n");




#if 0 /* detailed help will be made in the future */

	s_printf("coefficient: coef f x n --\
 extract coefficient of n-th power of x in (numerator of) f\n");
	s_printf("comment begin (end): comment begin  or  comment end\n");
	s_printf("divlevel n: set division level to print vertically\n");
	s_printf("gc: gc 1 -- garbage collection on (use memory economically), gc 0 -- garbage collection off\n");
	s_printf("reduce: reduce f x n (or f c) --\
 reduce numerator and denominator of f by n-th power of x (or by coefficient c)\n");
	s_printf("select: select f x n --\
 select n-th power of x in (numerator of) f\n");
	s_printf("st: sum truncate.  st f x ... z n --\
 truncate more than n-th power of total of x ... z in f\n");
	s_printf("truncate: truncate f x n --\
 truncate more than n-th power of x in f\n");

#endif /*0*/
}



/********************************* Other **************************************/

void SetDecimalDigit(void)
{
	s_f_coefDecimalNum = atoi(s_f_str);
}

void SetRightMargin(void)
{
	s_f_rightMargin = atoi(s_f_str);
}

void SetDivLevel(void)
{
	s_f_vDivLevel = atoi(s_f_str);
}

void FormulaComment(void) /* should be changed */
{
int commentFlag;

	commentFlag = CheckComment(s_f_str);

	while (commentFlag)
	{
		
		if (s_fgets(s_f_inputLine,S_F_MAX_INPUT_LENGTH,s_f_input))
		{
			if (s_f_inputLine[strlen(s_f_inputLine)-1] == '\n' )
			{
				s_f_inputLine[strlen(s_f_inputLine)-1] = 0;
			}
			/* s_f_inputLine does not contain \n at last line */
		}
		else /* already reached eof */
		{
			SetFormulaInputConsole();
			s_printf("End of File at: %s\n",s_f_inputLine);
			commentFlag = 0;
		}
		
		if (!strncmp(s_f_inputLine, "comment", 2))
		{ 
			commentFlag = CheckComment(strchr(s_f_inputLine,' '));
		}
	}	
}

int CheckComment(char *str)
{
	SkipSpaces(&str);
	
	switch (str[0])
	{
	case 0:
	case 'b':	/* begin */
	case '1':
		return 1;
	case 'e':	/* end */
	case '0':
		return 0;
	case 'o':
		goto l_o;
	default:
		return 1;
	}

l_o:
	switch (str[0])
	{
	case 0:
	case 'n':	/* on */
		return 1;
	case 'f':	/* off */
		return 0;
	default:
		return 1;
	}	
}

void OneLineComment(void)
{
	while (s_f_contLineFlag) ReadOneLine();
}

void SetFlag(int *flagP)
{
	SkipSpaces(&s_f_str);
	
	switch (s_f_str[0])
	{
	case '1':
		*flagP = 1; 
		return;
	case '0':
		*flagP = 0; 
		return;
	case 'o':
		goto l_o;
	default:
		s_printf("Error in SetFlag%s\n",s_f_inputLine);
		return;
	}
l_o:
	switch (s_f_str[1])
	{
	case 'n':
		*flagP = 1; 
		return;
	case 'f':
		*flagP = 0; 
		return;
	default:
		s_printf("Error in SetFlag%s\n",s_f_inputLine);
		return;
	}
}

void	SetUpFormula(void)	
{

#if !S_UNIX
	KeyScript(smRoman);
#endif /*!S_UNIX*/
	SetFont("Times");

SetUpProfile(1);
SetProfileName(0, "Calc");
}

void ErrorIlegalCommand(void)
{
	s_printf("Illegal command: %s\n",s_f_inputLine);
}

void ErrorAmbiguousCommand(void)
{
	s_printf("Ambiguous command: %s\n",s_f_inputLine);
}

voidFunc *SelectCommand(char *commandString)
{
int i, length, maxLength = 0, maxNum = 0, maxIndex;
	
	for (i = 0; commandTable[i].name; i++)
	{
		length = StringCoincideLength(commandTable[i].name, commandString);
		
		if (length > maxLength)
		{
			maxLength = length;
			maxNum = 1;
			maxIndex = i;
		}
		else if (length == maxLength)
			maxNum++;
	}
	
	if (maxLength == 0)
		return &ErrorIlegalCommand;
	
	if (maxNum > 1)
		return &ErrorAmbiguousCommand;
	
	return commandTable[maxIndex].func;
}

int StringCoincideLength(char *string1, char *string2)
{
int i;
	
	for (i = 0; string1[i]; i++)
	{
		if (string1[i] != string2[i])
			break;
	}
	i += !(string1[i]||string2[i]);
	
	return i;
}

void SetDisplayFlag(void)
{
	SetFlag(&s_f_showFlag);
}

void SetGCFlag(void)
{
	SetFlag(&s_f_GCFlag);
}

void SetTimeFlag(void)
{
	SetFlag(&s_f_printTimeFlag);
}

#if TEST_SC
void SetShowCommandFlag(void)
{
	SetFlag(&s_f_showCommandFlag);
}
#endif /*TEST_SC*/


void ShowMemory(void)
{
#if !S_UNIX
	s_printf("%ld bytes free\n", FreeMem());
#else
	s_printf("Not implemented on UNIX\n");
#endif
}

void SelectFormula(void)
{
	SelectCoefFormula(0);
}

void CoefFormula(void)
{
	SelectCoefFormula(1);
}

void QuitFormula(void)
{
	ExitToShell();
}


void SetSymbolMax(void)
{
	if (s_f_setFlag)
		s_printf("Can't change symbol max after set formula\n");
	else
		SScanInt(&s_f_str,&s_f_symbolMax);
}

void SetFormulaMax(void)
{
	if (s_f_setFlag)
		s_printf("Can't change formula max after set formula\n");
	else
		SScanInt(&s_f_str,&s_f_formulaMax);
}



#endif /*pragma once*/
