/*
 *  Formula-v.c -- global variables for Formula.c
 *  Version 2.0.2 / 020215
 *  by Satoshi Takahashi and Jiro Kikuchi
 */


#ifndef FORMULA_V_C /*pragma once*/
#define FORMULA_V_C



/**************************** Includes ***************************/

#include <Formula-v.h>
#include <Formula-Utilities.h>
#include <Formula-IO.h>
#include <Formula-Calc.h>
#include <Formula.h>

/**************************** Symbols and Formulas  ***************************/

int s_f_symbolMax = S_F_SYMBOL_MAX_INIT;
int s_f_formulaMax = S_F_FORMULA_MAX_INIT;
int s_f_sNum = 0;
char *(*s_f_sName);

s_f_symbolTableStruct *s_f_symbolTable;

int s_f_fNum = 0;
char *(*s_f_fName);

s_f_Base *s_f_currBase = 0;

s_f_Formula *s_f_formulas;

int s_f_curFNum = -1;


/**************************** Control Sequence Table  ***************************/

struct	s_f_ControlStruct;

s_f_ControlStruct controlTable[] =
	{
		{"a", symbolType, "$Symbol$a $Times$", 0},
		{"b", symbolType, "$Symbol$b  $Times$", 0},
		{"d", symbolType, "$Symbol$d $Times$", 0},
		{"e", symbolType, "$Symbol$e $Times$", 0},
		{"g", symbolType, "$Symbol$g $Times$", 0},
		{"h", symbolType, "$Symbol$h $Times$", 0},
		{"i", symbolType, "$&Pi$&I", 0},
		{"p", symbolType, "$Symbol$p $Times$", 0},
		{"s", symbolType, "$Symbol$s $Times$", 0},
		{"t", symbolType, "$Symbol$t $Times$", 0},
		{"w", symbolType, "$Symbol$w $Times$", 0},
#if TEST_CO
		{"f", operatorType, "", fracOp},
		{"frac", operatorType, "", fracOp},
#endif
		{0, 0, 0},
	};

#if 0
int s_f_totalControlNum = 4;
#endif



/********************************** Input ***********************************/

char *s_f_str; /* current position in line */
FILE *s_f_input = stdin;
char s_f_inputLine[S_F_MAX_INPUT_LENGTH+1];
int s_f_contLineFlag = 0;
char *s_f_strEnd; 
int s_f_setFlag = 0;


/********************************** OutPut ***********************************/

short s_f_fx = 0, s_f_fy = 0;
short s_f_fbx = 0, s_f_fby = 0;
short s_f_fontSize = 12;
char s_f_defaultFont[256] = "Times";
int s_f_divLevel = 0;

int s_f_vDivLevel = 1; /* max of vertical fraction to print */
int s_f_divVMove = 8;
int s_f_coefDecimalNum = 0;
int s_f_rightMargin = 70;
int s_f_contLeft = 20;
int s_f_bottomMargin = 15;
int s_f_multiScroll = 8;

int s_f_showBlock = 0;	/* used in PrintformulaBlock for debugging */


/**************************** Print Formula ***********************************/

int s_f_printFlag = 0;
int s_f_printContFlag = 0;


/**************************** Save Formula ***********************************/

FILE *s_f_formulaOutFile;
int  s_f_saveFlag = 0;

/**************************** Input Show ***********************************/

int s_f_inputShowFlag = 0;

/********************************* Free ****************************************/

s_f_FB *s_f_freeFBHead, *s_f_freeFBTail;

/********************************** Other ***********************************/

s_f_FB *s_f_dummyFB = 0;
s_f_Base *s_f_tmpBase;

int s_f_GCFlag = 1; /*garbage collection*/

int (*s_f_compareBasesP)(int*, int*) = &CompareBases;
int *s_f_compareOrder;

#if TEST_SC
int s_f_showCommandFlag = 0;
#endif


#if 0 /*just to see it*/
typedef struct
{
	char* name;
	voidFunc *func;
} commandTableStruct;
#endif /*0*/

commandTableStruct commandTable[] = 
{
	{"",			OneLineComment},
	{"#",			OneLineComment},
	{"%", 			OneLineComment},
	{"bprint",		BeginFormulaPrinting },				 /* in Print.c */
	{"bsave",		BeginFormulaSaving },
	{"calc",		SortFracFormula },
	{"clear",		ClearFormulaDisp },						
	{"coefficient",	CoefFormula },	 /*SelectCoefFormula(...,1)*/
#if TEST_SOL
	{"collect",		CollectFormula	},
#endif
#if TEST_SC
	{"command", 	SetShowCommandFlag },
#endif
	{"comment", 	FormulaComment },
	{"complexclean",		CleanComplexFormula },						
#if TEST_CO
	{"copy", 		CopyFormula },
#endif
	{"decimal", 	SetDecimalDigit },
	{"denominator",	GetDenominator },
#if TEST_DIFF
	{"diff", 		DifferentiateFormula },
#endif
	{"display",		SetDisplayFlag },	
#if TEST_DIV
	{"divide",		DivideFormula },	
#endif
	{"divlevel",	SetDivLevel },	
	{"dl",			SetDivLevel },	
	{"ed",			ExpandDenominator },
	{"eprint",		EndFormulaPrinting },/* in Print.c */
	{"esave",		EndFormulaSaving },					
#if TEST_DIV
	{"factor",		FactorFormula },	
#endif
	{"fmax", 		SetFormulaMax	},
	{"gc", 			SetGCFlag },	
	{"help", 		FormulaHelp }, 							
	{"ic",			SetFormulaInputConsole }, 				
	{"if", 			SetFormulaInputFile },
	{"ishow",		SetInputShowFlag},
	{"inputshow",	SetInputShowFlag},
	{"memory",		ShowMemory },
	{"numerator",	GetNumerator },
	{"order",		SetCompareOrder },
#if TEST_PRED
	{"preduce",		PolynomialReduceFormula},
#endif
	{"quit",		QuitFormula},
	{"realdenom",	CompMakeDenomReal},
	{"reduce",		ReduceFormula },
	{"rm", 			SetRightMargin },
	{"select",		SelectFormula }, 	/*SelectCoefFormula(...,0)*/
	{"set", 		SetFormulaByString },
	{"show", 		PrintFormula	},
	{"smax", 		SetSymbolMax	},
#if TEST_SOL
	{"solve",		SolveFormula	},
#endif
	{"st",			SumTruncateFormula },
	{"substitute", 	SubstituteFormula },
	{"sumtruncate",	SumTruncateFormula },
	{"time",		SetTimeFlag },
	{"truncate",	TruncateFormula },
	{"?", 			FormulaHelp }, 							
	{0,				0},
};


#endif /*pragma once*/
