/*
 *  Formula-v.h
 *  Version 1.8.1 / 000830
 *  by Satoshi Takahashi and Jiro Kikuchi
 */

#ifndef FORMULA_V_H /*pragma once*/
#define FORMULA_V_H

#include <Formula-Switches.h>
#include <Formula-Consts.h>
#include <stdio.h>

/******************************** Types **********************************/

typedef struct
{
	double coef;
	int *power;
}	s_f_Base;

typedef struct s_f_FormulaBlock /* need name, because it contain a pointer to itself */
{
	int type;
	int operand; /* AddOp, ..., DivOp operand of this term or block */
	int power;  /* use only if type is BlockType */
	s_f_Base *base;
	struct s_f_FormulaBlock *block; /* ( ) */
	struct s_f_FormulaBlock *next;
	struct s_f_FormulaBlock *prev;
} s_f_FormulaBlock;

#define s_f_FB s_f_FormulaBlock

typedef struct
{
	s_f_FormulaBlock *fb;
	int divDepth;
	int expanded; 			/* flag : formula is already expanded */
	int sorted; 			/* flag : formula is already sorted */
	int frac; 				/* formula is fractional (rational) */
	char *printName;
	int controlFlag;
	int cleanComplexFlag;
} s_f_Formula;



/**************************** Symbols and Formulas  ***************************/

extern int s_f_symbolMax;
extern int s_f_formulaMax;
extern int s_f_sNum;
extern char *(*s_f_sName);

typedef struct
{
	char *name;
	char *printName;
	int controlFlag;
	int controlNum;
}	s_f_symbolTableStruct;

extern s_f_symbolTableStruct *s_f_symbolTable;

extern int s_f_fNum;
extern char *(*s_f_fName);

extern s_f_Base * s_f_currBase;

extern s_f_Formula *s_f_formulas;

extern int s_f_curFNum;


/**************************** Control Sequence Table  ***************************/

typedef struct
{
	char* controlName;
	int type;
	char* printName;
	int operatorNum;
}	s_f_ControlStruct;

enum { symbolType = 1, operatorType };

extern s_f_ControlStruct controlTable[];

#if TEST_CO

typedef struct
{
	int usedFlag;
	int num;
	int power;
}	s_f_FormulaPB;

enum operatorNum { fracOp } ;

typedef struct
{
	int usedFlag;
	int num;
	int type;
	int operatorFlag;
	int operatorNum;
}	s_f_ControlPB;

#endif /*TEST_CO*/

/********************************** Input ***********************************/

extern char *s_f_str; /* current position in line */
extern FILE *s_f_input;
extern char s_f_inputLine[S_F_MAX_INPUT_LENGTH+1];
extern int s_f_contLineFlag;
extern char *s_f_strEnd; 
extern int s_f_setFlag;


/********************************** OutPut ***********************************/

extern short s_f_fx, s_f_fy;
extern short s_f_fbx, s_f_fby;
extern short s_f_fontSize;
extern char s_f_defaultFont[256];
extern int s_f_divLevel;

extern int s_f_vDivLevel; /* max of vertical fraction to print */
extern int s_f_divVMove;
extern int s_f_coefDecimalNum;
extern int s_f_rightMargin;
extern int s_f_contLeft;
extern int s_f_bottomMargin;
extern int s_f_multiScroll;

extern int s_f_showBlock;	/* used in PrintformulaBlock for debugging */


/**************************** Print Formula ***********************************/

extern int s_f_printFlag;
extern int s_f_printContFlag;


/**************************** Save Formula ***********************************/

extern FILE *s_f_formulaOutFile;
extern int  s_f_saveFlag;


/**************************** Input Show ***********************************/

extern int s_f_inputShowFlag;

/********************************* Free ****************************************/

extern s_f_FB *s_f_freeFBHead, *s_f_freeFBTail;
extern s_f_Base s_f_nullBase;


/********************************** Other ***********************************/

extern s_f_FB *s_f_dummyFB;
extern s_f_Base *s_f_tmpBase;

extern int s_f_GCFlag;

extern int (*s_f_compareBasesP)(int*, int*);
extern int *s_f_compareOrder;

#if TEST_SC
extern int s_f_showCommandFlag;
#endif

typedef void (voidFunc)(void);

typedef struct
{
	char* name;
	voidFunc *func;
} commandTableStruct;

extern commandTableStruct commandTable[];


#endif /*pragma once*/