/*
 *  Graph1D.c
 *  Version 1.9.1 / 010219
 *  by Satoshi Takahashi, Hidekazu Koike, Tomohiro Mizuno and Yumiko Takahashi
 */

#ifndef GRAPH1D_C /*pragma once*/
#define GRAPH1D_C
#include <s_Config.h>

#include <string.h>
#include <math.h>

#include <Plot.h>
#include <HugeArray.h>
#include <ParametersDialog.h>
#include <s_QDGlobal.h>
#include <DialogFOpen.h>
#include <s_Drawing.h>
#include <StartSCL.h>
#include <s_Print.h>
#include <Save.h>
#if !S_UNIX
#include <PDProcs.h>
#endif
#include <s_Common.h>
#include <s_RedrawFunc.h>
#include <DragAndDrop.h>
#include <EPS.h>

extern double s_d_scale;

#include <Graph-Switches.h>
#include <GraphCommon.h>
#include <GraphSpreadSheet.h>

#include <Graph1D.h>

#include <VarBody.h>
#include <Graph1D-v.c>

/******************** Switches ********************/

#ifndef S_G_PAINT
#define S_G_PAINT 1
#endif

#ifndef GXYM /*Graph xy-plot multiple series */
#define GXYM 1
#endif


/********** Common Variables **********/



/********** Function Graph Variables **********/

/*moved to Graph1D-v.c */


/********** SpreadSheet Variables **********/
/******************** Function Definitions ********************/

void FunctionGraph(void)
{
	s_g1.type = fType;
	DrawGraph();
}

void XYFunctionGraph(void)
{
	s_g1.type = xyType;
	DrawGraph();
}

void DrawGraph(void)
{
#if !GXYM
double x;
#endif

double xMin, xMax, yMin, yMax;

#if S_G_PAINT
PenState savePenState;
#endif

	s_g.dim = 1;

#if S_G_PAINT
	GetPenState(&savePenState);
	s_PenMode(patOr);
#endif

	if (!s_g.PDSetUpFlag)
	{
		SetUpParametersDialogP(&s_g.dialogP);
		s_g.PDSetUpFlag = 1;
	}
	s_sg.realDataNumber = 0;

	if (! s_g.addParamFlag )
	{
		GraphAddParameters();
	}
	ShowDrawing();
	

	s_TextSize(s_g.labelSize/s_d_scale);
	SetFont("Times");
	SetPlot2D2W(300/s_d_scale,300/s_d_scale);
	s_PenSize(1/s_d_scale,1/s_d_scale);
	ParametersDialogP(s_g.dialogP);
	while(!s_g.exitFlag)
	{
		CheckPause();

		if ((s_g1.type == fType) && (s_fg.barFlag))
		{
			xMin = GraphConvertX(s_g.limXMin - 0.5*s_fg.dx);
			xMax = GraphConvertX(s_g.limXMax + 0.5*s_fg.dx);
		}
		else
		{
			xMin = GraphConvertX(s_g.limXMin);
			xMax = GraphConvertX(s_g.limXMax);
		}
		yMin = GraphConvertY(s_g.limYMin);
		yMax = GraphConvertY(s_g.limYMax);

		SetPlot2D2B(s_g.left/s_d_scale,s_g.bottom/s_d_scale);
		SetPlot2D2O(xMin,yMin);
		SetPlot2D2W(s_g.width/s_d_scale/(xMax-xMin),
								s_g.height/s_d_scale/(yMax-yMin));

		if ((s_g1.type == fType) && (s_fg.barFlag))
		{
			xMin = GraphConvertX(s_g.limXMin);
			xMax = GraphConvertX(s_g.limXMax);
		}

		EraseDrawing();
		if (s_g.printFlag) BeginPrinting();
		if (s_g.saveFlag) BeginSaving();
		if (s_g.epsSaveFlag) BeginEPSSaving();

		GraphFrameAndLabel();
		if (s_g1.userProcFlag) (*s_g1.userProc)();

		switch(s_g1.type)
		{
		case spreadSheetType:
			GraphPlotSpreadSheet();
			break;
		case xyType:
#if !GXYM
			PenSize(s_g1.pen1/s_d_scale,s_g1.pen1/s_d_scale);
			SetPlot2D2I(s_fg.fx[0](s_fg.xMin),s_fg.fy[0](s_fg.xMin));
			for( x = s_fg.xMin; x < s_fg.xMax + s_fg.dx; x += s_fg.dx)
			{
				Plot2D2(s_fg.fx[0](x),s_fg.fy[0](x));
			}
			break;
#endif
		case fType:
		default:
			GraphPlotFunction();
			break;
		}
		if (s_g.printFlag) EndPrinting();
		if (s_g.saveFlag) EndSaving();
		if (s_g.epsSaveFlag) EndEPSSaving();
		Stop();
		ParametersDialogP(s_g.dialogP);
	}
#if S_G_PAINT
	s_PenMode(savePenState.pnMode);
#endif
}


void GraphAddParameters(void)
{
int i;
char dialogString[256];


/* Save and Load */

	GraphAddParameters1();

	if ( s_g1.type == spreadSheetType)
			AddParameterP(&SGInit,  doType, "Load New Data", s_g.dialogP);			


	if (s_g.dim == 1)
		AddParameterP(&s_g.colorFlag, flagType, "Color", s_g.dialogP);
		

/* Frame and Region */
	switch(s_g1.type)
	{
	case fType:
		AddParameterP(&s_fg.xMin, doubleType, "Min Draw X", s_g.dialogP);
		AddParameterP(&s_fg.xMax, doubleType, "Max Draw X", s_g.dialogP);
		s_g1.pen = MakeArrayDouble1(s_fg.funcNum);
		for (i=0; i < s_fg.funcNum; i++)
		{
			s_g1.pen[i] = 1.0;
			sprintf(dialogString, "Pen %d Width", i+1);
			AddParameterP(&s_g1.pen[i], doubleType, dialogString, s_g.dialogP);
		}
		break;
	case xyType:
		GraphAddparametersXYFunc();
		break;
	}
	
	GraphFrameAddParameters();

	switch(s_g1.type)
	{
	case spreadSheetType:
		SGInit();
		SGSS1DAddParameters();
		break;
	case fType:
		AddParameterP(&s_fg.barFlag, flagType,  "Bar Graph", s_g.dialogP);
		AddParameterP(&s_fg.dx, doubleType, "dx", s_g.dialogP);
		break;
	}

	for ( i = 0; i < s_fg.funcNum; i++)
	{
		sprintf(dialogString, "paint f%d", i+1);
		AddParameterP(&s_fg.paintFlag[i], flagType, dialogString, s_g.dialogP);
	}
	for ( i = 0; i < s_fg.funcNum; i++)
	{
		sprintf(dialogString, "f%d paint pattern", i+1);
		AddParameterP(&s_fg.pnumber[i], intType, dialogString, s_g.dialogP);
	}

#if 0
	AddParameterP(&MagnifyPlot2D, doType, "Magnify", s_g.dialogP);
	AddParameterP(&ShrinkPlot2DHalf, doType, "Shrink", s_g.dialogP);
#endif
}

void GraphAddparametersXYFunc(void)
{
int i;
char dialogString[256];

	for ( i = 0; i < s_fg.funcNum; i++)
	{
		sprintf(dialogString, "min t%d", i+1);
		AddParameterP(&s_fg.xMinA[i],doubleType,dialogString,s_g.dialogP);
		sprintf(dialogString, "max t%d", i+1);
		AddParameterP(&s_fg.xMaxA[i],doubleType,dialogString,s_g.dialogP);
	}
	AddParameterP(&s_fg.dx,doubleType,"dt",s_g.dialogP);
}

void GraphPlotFunction(void)
{
int i;
double x = 0;

#if S_G_PAINT
PolyHandle paintPH = 0;
Pattern pat;

	SetPattern(pat, 23); /*ltGray*/
#endif


				for ( i=0; i<s_fg.funcNum; i++)
				{
#if S_G_PAINT
					if ((s_g1.type == fType) && s_fg.paintFlag[i] && !s_fg.barFlag)
					//if (s_fg.paintFlag[i] && !s_fg.barFlag)
					{
						paintPH = s_OpenPoly();
						s_ShowPen();
					}
#endif
					PenSize(s_g1.pen[i]/s_d_scale, s_g1.pen[i]/s_d_scale);
#if GXYM
					switch (s_g1.type)
					{
					case fType:
						x = GraphPlotFuctionOneSeries(i);
						break;
					case xyType:
						x = GraphPlotXYFuctionOneSeries(i);
						break;
					}
					
#else /*GXYM*/
					SetPlot2D2I(s_fg.xMin,s_fg.func[i](s_fg.xMin));
					for( x = s_fg.xMin; x < s_fg.xMax + s_fg.dx*0.001; x += s_fg.dx)
					{
						Plot2D2(x,s_fg.func[i](x));
					}
#endif /*else GXYM*/
#if S_G_PAINT

					if ((s_g1.type == fType) && s_fg.paintFlag[i] && !s_fg.barFlag)
					//if (s_fg.paintFlag[i] && !s_fg.barFlag)
					{
						s_HidePen();
						Plot2D2(x,s_g.limYMin);
						Plot2D2(s_fg.xMin,s_g.limYMin);
						Plot2D2(s_fg.xMin,s_fg.func[i](s_fg.xMin));
						ClosePoly();
						SetPattern(pat, s_fg.pnumber[i]);
						s_FillPoly(paintPH,pat);
						KillPoly(paintPH);
					}
#endif /*S_G_PAINT*/
				}
}


#if GXYM
double GraphPlotFuctionOneSeries(int i)
{
double x;

	if (s_fg.barFlag)
	{
		GraphPlotFuctionOneBar(s_fg.xMin, i);
	}
	else
	{
		SetPlot2D2I(s_fg.xMin, s_fg.func[i](s_fg.xMin));
	}
	for( x = s_fg.xMin; x < s_fg.xMax + s_fg.dx*0.001; x += s_fg.dx)
	{
		if (s_fg.barFlag)
		{
			GraphPlotFuctionOneBar(x, i);
		}
		else
		{
			Plot2D2(x, s_fg.func[i](x));
		}
	}
	
	return x; /*used for paint*/
}

void GraphPlotFuctionOneBar(double x, int i)
{
Rect barRect;
Pattern pat;

	SetRect(&barRect, Plot2DTransX(x - 0.5*s_fg.dx),
					Plot2DTransY(s_fg.func[i](x)),
					Plot2DTransX(x + 0.5*s_fg.dx),
					Plot2DTransY(s_g.limYMin));
	FrameRect(&barRect);
	if (s_fg.paintFlag[i])
	{
		SetPattern(pat, s_fg.pnumber[i]);
		FillRect(&barRect, pat);
	}	
}

double GraphPlotXYFuctionOneSeries(int i)
{
double x;

	SetPlot2D2I(s_fg.fx[i](s_fg.xMinA[i]),s_fg.fy[i](s_fg.xMinA[i]));
	for( x = s_fg.xMinA[i]; x < s_fg.xMaxA[i] + s_fg.dx*0.001; x += s_fg.dx)
	{
		Plot2D2(s_fg.fx[i](x),s_fg.fy[i](x));
	}

	return x; /*used for paint*/
}
#endif /*GXYM*/

void SetGraphMinDrawX(double d)
{
	s_fg.xMin = d;
}

void SetGraphMaxDrawX(double d)
{
	s_fg.xMax = d;
}

void SetGraphFunction(double (*fp)(double))
{
	s_fg.func[0] = fp;
	s_g1.type = fType;
	s_fg.funcNum = 1;
}

void AddGraphFunction(double (*fp)(double))
{
	if ( s_fg.funcNum < S_FG_MAXFUNCNUM )
	{
		s_fg.func[s_fg.funcNum] = fp;
		s_g1.type = fType;
		s_fg.funcNum++;
	}
	else
	{
		s_printf("Error in AddGraphFunction:\nToo many functions\n");
	}
}

void SetGraphXYFunction(double (*fx)(double),double (*fy)(double))
{
	s_fg.fx[0] = fx;
	s_fg.fy[0] = fy;
	s_g1.type = xyType;
	s_fg.funcNum = 1;
}

#if GXYM
void AddGraphXYFunction(double (*fx)(double),double (*fy)(double))
{
	if ( s_fg.funcNum < S_FG_MAXFUNCNUM )
	{
		s_fg.fx[s_fg.funcNum] = fx;
		s_fg.fy[s_fg.funcNum] = fy;
		s_g1.type = xyType;
		s_fg.funcNum++;
	}
	else
	{
		s_printf("Error in AddGraphFunction:\nToo many functions\n");
	}
}
#endif /*GXYM*/

void SetGraphUserProc(void (*userProc)(void))
{
	s_g1.userProc = userProc;
	s_g1.userProcFlag = 1;
}


void in_Graph1DAddParameter(Ptr p,int type,char* com)
{

#if APPEND_LAST1D
	if (! s_g1.addParamFlag )
	{
		GraphAddParameters();
	}
	
#else
	if (!s_g.PDSetUpFlag)
	{
		SetUpParametersDialogP(&s_g.dialogP);
		s_g.PDSetUpFlag = 1;
	}
#endif

	AddParameterP(p, type, com, s_g.dialogP);
}

#endif /*pragma once*/
