/*
 *  Graph2D.c
 *  Version 2.0.3 / 020726
 *  by Tomohiro Mizuno, Satoshi Takahashi, Yumiko Takahashi and Jiro Kikuchi
 */

#ifndef GRAPH2D_C /*pragma once*/
#define GRAPH2D_C

/****************************** Switches ******************************/
#include <Graph2D-Switches.h>

/****************************** Include Headers **********************/
#include <math.h>
#include <stdlib.h>

#include <Plot.h>
#include <StartSCL.h>
#include <s_QDGlobal.h>
#include <Plot.h>
#include <s_Common.h>
#include <PDProcs.h>
#include <s_RedrawFunc.h>
#include <s_Print.h>
#include <Save.h>
#include <HugeArray.h>
#include <s_undef.c>
#include <s_Math.h>

#include <Graph1D.h>
#include <GraphCommon.h>
#include <GraphSpreadSheet.h>
#include <Graph2D.h>


#define G2D_MAX_FUNC_NUM 10



/****************************** Variables ******************************/

#include <VarBody.h>
#include <Graph2D-v.c>

/****************************** Plot.c Variables ******************************/

extern double s_wx, s_wy;
extern long s_bx, s_by; /* to add dialog */


/****************************** Routines ******************************/

void Graph2DFunction(void)
{
double x;
double y;
double x0, y0, x1, y1, x2, y2, fmin, fav, fmax;
PenState savePenState;
#if MOVEPOLY
PolyHandle tri1, tri2;
#endif
int i;

	s_g.dim = 2;
	
	if (!s_g2d.funcNum)
	{
		PrintError("Error in Graph2DF: No function");
	}
	
	SetPlot2D2O(0,0);
	
	GetPenState(&savePenState);
	s_PenMode(patOr);

	if (! s_g.addParamFlag )
		Graph2DFAddParameters();

#if MOVEPOLY
	DefineTriangle(0, 0, s_g2d.dx, 0, 0, s_g2d.dy, &tri1);
	DefineTriangle(s_g2d.dx, 0, 0, s_g2d.dy, s_g2d.dx, s_g2d.dy, &tri2);
#endif

	ParametersDialogP(s_g.dialogP);
	while(!s_g.exitFlag)
	{
		if (s_g.spreadSheet)
		{
			if (s_g.limXMin < s_sg.min[s_sg.seriesX[0]-1])
				s_g.limXMin = s_sg.min[s_sg.seriesX[0]-1];
			if (s_g.limXMax > s_sg.max[s_sg.seriesX[0]-1])
				s_g.limXMax = s_sg.max[s_sg.seriesX[0]-1];
			if (s_g.limYMin < s_sg.min[s_sg.seriesY[0]-1])
				s_g.limYMin = s_sg.min[s_sg.seriesY[0]-1];
			if (s_g.limYMax > s_sg.max[s_sg.seriesY[0]-1])
				s_g.limYMax = s_sg.max[s_sg.seriesY[0]-1];
			SG2DArray();
		}
#if CL
		SG2DResetArray();
#endif
		
		EraseDrawing();
		
		if (s_g.printFlag) BeginPrinting();
		if (s_g.saveFlag) BeginSaving();
		if (s_g.epsSaveFlag) BeginEPSSaving();

		SetPlot2D2B(s_g.left/s_d_scale,s_g.bottom/s_d_scale);
		SetPlot2D2O(s_g.limXMin,s_g.limYMin);
		SetPlot2D2W(s_g.width/s_d_scale/(s_g.limXMax-s_g.limXMin),
								s_g.height/s_d_scale/(s_g.limYMax-s_g.limYMin));

		GraphFrameAndLabel();
		
		for (i = 0; i < s_g2d.funcNum; i++)
		{
			if (!s_g2d.selectPlotArray[i])
				continue;
#if CL
			if (s_g2d.contourLabelFlag && !s_g2d.onlyZero)
				Graph2DInitArray();
#endif
			SetPattern(s_g2d.pat, s_g2d.pnumber[i]);
			for ( x = s_g.limXMin; x < s_g.limXMax-0.0000001; x += s_g2d.dx, CheckPause())
			{
				for ( y = s_g.limYMin; y < s_g.limYMax-0.0000001; y += s_g2d.dy )
				{
					x0 = x;
					y0 = y;
					x1 = x + s_g2d.dx;
					y1 = y;
					x2 = x;
					y2 = y + s_g2d.dy;
					Graph2DOrderPoints(&x0, &y0, &x1, &y1, &x2, &y2, &fmin, &fav, &fmax, i);
					LevelLineTriangle( x0, y0, x1, y1, x2, y2, fmin, fav, fmax);
					
#if MOVEPOLY
					OffsetPoly(tri1, x*s_wx, - y*s_wy);
					PaintNegativeTriangle( x0, y0, x1, y1, x2, y2, fmin, fav, fmax, tri1);
					OffsetPoly(tri1, -x*s_wx, y*s_wy);
#else
					PaintNegativeTriangle( x0, y0, x1, y1, x2, y2, fmin, fav, fmax);
#endif
					
					x0 = x + s_g2d.dx;
					y0 = y;
					x1 = x;
					y1 = y + s_g2d.dy;
					x2 = x + s_g2d.dx;
					y2 = y + s_g2d.dy;
					Graph2DOrderPoints(&x0, &y0, &x1, &y1, &x2, &y2, &fmin, &fav, &fmax, i);
					LevelLineTriangle( x0, y0, x1, y1, x2, y2, fmin, fav, fmax);
				
#if MOVEPOLY
					OffsetPoly(tri2, x*s_wx, - y*s_wy);
					PaintNegativeTriangle( x0, y0, x1, y1, x2, y2, fmin, fav, fmax, tri2);
					OffsetPoly(tri2, -x*s_wx, y*s_wy);
#else
					PaintNegativeTriangle( x0, y0, x1, y1, x2, y2, fmin, fav, fmax);
#endif
				

				}/* for y */
			}/* for x */
		} /*for i*/
		
		for (i = 0; i < s_g2d.extraCLCount; i++)
			ExtraHeightDraw(i);
		
		if (s_g.printFlag) EndPrinting();
		if (s_g.saveFlag) EndSaving();
		if (s_g.epsSaveFlag) EndEPSSaving();
		Stop();
		ParametersDialogP(s_g.dialogP);
	}
	
	s_PenMode(savePenState.pnMode);
}


void Graph2DFAddParameters(void)
{
int i;
char funcName[256];

	s_g.addParamFlag = 1;

	if (!s_g.PDSetUpFlag)
	{
		SetUpParametersDialogP(&s_g.dialogP);
		s_g.PDSetUpFlag = 1;
	}

	GraphAddParameters1();

/* Frame and Region */
	if (!s_g.spreadSheet)
	{
		AddParameterP(&(s_g2d.dx), doubleType, "dx", s_g.dialogP);
		AddParameterP(&(s_g2d.dy), doubleType, "dy", s_g.dialogP);
	}
	AddParameterP(&(s_g2d.df), doubleType, "df", s_g.dialogP);
	AddParameterP(&(s_g2d.fmaxBound), doubleType, "Func Max Bound", s_g.dialogP);
	AddParameterP(&(s_g2d.fminBound), doubleType, "Func Min Bound", s_g.dialogP);
	
	GraphFrameAddParameters();

	if (s_g.spreadSheet)
		SG2DInit();

	AddParameterP(&s_g2d.contourLabelFlag, flagType, "Contour Labels", s_g.dialogP);
	AddParameterP(&s_g2d.contourLabelSize, intType, "Label Size", s_g.dialogP);
	AddParameterP(&s_g2d.contourLabelDigit, intType, "Label Digit", s_g.dialogP);
	AddParameterP(&s_g2d.contourFreq, doubleType, "Labels Interval", s_g.dialogP);
	AddParameterP(&ExtraHeightNew, doType, "New Label", s_g.dialogP);
	AddParameterP(&ResetExtraHeight, doType, "Reset Label", s_g.dialogP);
	AddParameterP(&s_g2d.paintNegativeFlag, flagType, "Paint Negative Region", s_g.dialogP);
	AddParameterP(&s_g2d.onlyZero, flagType, "Only Zero Level", s_g.dialogP);
	
	for ( i = 0; i < s_g2d.funcNum; i++)
	{
		sprintf(funcName, "Plot f%d", i+1);
		s_g2d.selectPlotArray[i] = 1;
		AddParameterP(&(s_g2d.selectPlotArray[i]), flagType, funcName, s_g.dialogP);
	}
	
}

#if CL
void Graph2DMaxAndMin(void)
{
double fmax, fmin, fret, x, y;
int i;
int maxFlag = 0, minFlag = 0;
int valNotSet = 1;


    
    //fmax = s_g2d.fminBound;
    //fmin = s_g2d.fmaxBound;
	
	for (i = 0; i < s_g2d.funcNum; i++)
	{
		for ( x = s_g.limXMin; x < s_g.limXMax + 0.5*s_g2d.dx; x += s_g2d.dx, CheckPause() )
			for ( y = s_g.limYMin; y < s_g.limYMax + 0.5*s_g2d.dy; y += s_g2d.dy )
			{
				fret = s_g2d.f[i](x, y);
				
                if (NotADouble(fret))
                    continue;
                if (valNotSet)
                {
                    valNotSet = 0;
                    fmax = fret;
                    fmin = fmax;
                }
				
				if (fret > fmax)
				{
					fmax = fret;
					if (fmax > s_g2d.fmaxBound)
					{
                        fmax = s_g2d.fmaxBound;
						if (minFlag)
							goto endFunc;
						maxFlag =1;
					}
				}
				else if (fret < fmin)
				{
					fmin = fret;
					if (fmin < s_g2d.fminBound)
					{
                        fmin = s_g2d.fminBound;
						if (maxFlag)
							goto endFunc;
						minFlag =1;
					}
				}
			}
	}

endFunc:	
	s_g2d.fmax = fmax;
	s_g2d.fmin = fmin;
}

int *s_g2dArray = 0;

void Graph2DMakeArray(void)
{
long size;

	if (s_g2dArray)
		free(s_g2dArray);
		
	size = (s_g2d.fmax - s_g2d.fmin)/s_g2d.df;
	if ( size > s_g2d.maxArraySize )
	{
		s_g2d.fmax = min (s_g2d.df*s_g2d.fmaxBound, s_g2d.fmax);
		s_g2d.fmin = max (s_g2d.df*s_g2d.fminBound, s_g2d.fmin);
		size = (s_g2d.fmax - s_g2d.fmin)/s_g2d.df;
	}

	if ( !(s_g2dArray = calloc( size+1, sizeof(int) ) ) )
		s_OOMError();
	
	s_g2d.arraySize = size;
}

void Graph2DInitArray(void)
{
int i;

	for ( i = 0; i < s_g2d.arraySize+1; i++)
	{
		s_g2dArray[i] = s_g2d.contourFreq*( (8*i) % 21) + 1;
	}
}

void ContourLabel(double labelValue)
{
int index;
short txSize;

	if (s_g2d.onlyZero)
		return;
	
	if (labelValue < s_g2d.fmin || labelValue > s_g2d.fmax)
		return;

	index = (labelValue - s_g2d.fmin)/s_g2d.df;
	if (index > s_g2d.arraySize) /*fmax may have been changed */
		return;
	if ( s_g2dArray[index] > 0 )
	{
		s_g2dArray[index]--;
		if (!s_g2dArray[index])
		{
			s_Move(-13,4);
			txSize = GetPortTextSize(s_thePort);
			s_TextSize(s_g2d.contourLabelSize);
			s_PenMode(patCopy);
			s_DirectDprintf("%.*f", s_g2d.contourLabelDigit, labelValue + 0.0000000000000001);
			s_PenMode(patOr);
			s_TextSize(txSize);
		}
	}
}
#endif /*CL*/

#if MOVEPOLY
void DefineTriangle(double x0, double y0 , double x1, double y1, double x2, double y2,
						PolyHandle *tri)
{
	*tri = s_OpenPoly();
	Plot2DMoveTo(x0, y0);
	Plot2DLineTo(x1, y1);
	Plot2DLineTo(x2, y2);
	Plot2DLineTo(x0, y0);
	s_ClosePoly();
}
#endif

void LevelLineTriangle(double x0, double y0 , double x1, double y1, double x2, double y2,
							double fmin, double fav, double fmax)
{
double cmin, cav, cmax;
double c;
double x, y;
double r;

	if (s_g2d.onlyZero)
	{
		if (fmin > 0 || fmax < 0)
			return;
		cmin = 0;
		cav = 0;
		cmax = 0;
	}
	else
	{
		cmin = (ceil(fmin/s_g2d.df))*s_g2d.df;
		cav = (ceil(fav/s_g2d.df))*s_g2d.df;
		cmax = (ceil(fmax/s_g2d.df))*s_g2d.df;
	}

	for (c = cmin; c <= fav; c+=s_g2d.df)
	{
		if (fav - fmin)
		{
			r = (c - fmin)/(fav - fmin);
			
			x = (1-r)*x0 + r*x1;
			y = (1-r)*y0 + r*y1;
			Plot2DMoveTo(x, y);

			r = (c - fmin)/(fmax - fmin);
			
			x = (1-r)*x0 + r*x2;
			y = (1-r)*y0 + r*y2;
			Plot2DLineTo(x, y);
#if CL
			if (s_g2d.contourLabelFlag)
				ContourLabel(c);
#endif		
		}
		if (s_g2d.onlyZero)
			break;
	}

	for (c = cav; c <= fmax; c+=s_g2d.df)
	{
		if (s_g2d.onlyZero && c < fav)
			break;
		if (fmax - fav)
		{
			r = (c - fav)/(fmax - fav);
			
			x = (1-r)*x1 + r*x2;
			y = (1-r)*y1 + r*y2;
			Plot2DMoveTo(x, y);

			r = (c - fmin)/(fmax - fmin);
			
			x = (1-r)*x0 + r*x2;
			y = (1-r)*y0 + r*y2;
			Plot2DLineTo(x, y);
#if CL
			if (s_g2d.contourLabelFlag)
				ContourLabel(c);
#endif		
		}
		if (s_g2d.onlyZero)
			break;
	}	
}

void Graph2DOrderPoints(double *x0p, double *y0p, double *x1p, double *y1p,
					double *x2p, double *y2p, double *fminp, double *favp, double *fmaxp,
					int funci)
{
	*fminp = s_g2d.f[funci](*x0p, *y0p);
	*fminp = min(*fminp, s_g2d.fmaxBound);
	*fminp = max(*fminp, s_g2d.fminBound);
	
	*favp = s_g2d.f[funci](*x1p, *y1p);
	*favp = min(*favp, s_g2d.fmaxBound);
	*favp = max(*favp, s_g2d.fminBound);

	*fmaxp = s_g2d.f[funci](*x2p, *y2p);
	*fmaxp = min(*fmaxp, s_g2d.fmaxBound);
	*fmaxp = max(*fmaxp, s_g2d.fminBound);


	if (*fminp > *favp)
	{
		ExchangeDouble(fminp, favp);
		ExchangeDouble(x0p, x1p);
		ExchangeDouble(y0p, y1p);
	}
	if (*favp > *fmaxp)
	{
		ExchangeDouble(favp, fmaxp);
		ExchangeDouble(x1p, x2p);
		ExchangeDouble(y1p, y2p);
	}
	if (*fminp > *favp)
	{
		ExchangeDouble(fminp, favp);
		ExchangeDouble(x0p, x1p);
		ExchangeDouble(y0p, y1p);
	}
}

void ExchangeDouble(double *r1p, double *r2p)
{
double inter;

	inter = *r1p;
	*r1p = *r2p;
	*r2p = inter;
}

#if MOVEPOLY
void PaintNegativeTriangle(double x0, double y0 , double x1, double y1, double x2, double y2,
							double fmin, double fav, double fmax, PolyHandle tri)
{
#else
void PaintNegativeTriangle(double x0, double y0 , double x1, double y1, double x2, double y2,
							double fmin, double fav, double fmax)
{
#endif

double r;
PolyHandle paintPH;
double xs1, ys1, xs2, ys2;

	if (!s_g2d.paintNegativeFlag)
		return;

	if (fmin > 0)
		return;

#if MOVEPOLY
	if (fmax < 0)
	{
		s_FillPoly(tri,s_g2d.pat);
		return;
	}
#endif
	paintPH = s_OpenPoly();
	
#if MOVEPOLY
	if (0)
		;
#else
	if (fmax < 0)
	{
		Plot2DMoveTo(x0, y0);
		Plot2DLineTo(x1, y1);
		Plot2DLineTo(x2, y2);
		Plot2DLineTo(x0, y0);
	}
#endif
	else if (fav > 0)
	{
		r = (0.0 - fmin)/(fmax - fmin);
		xs1 = (1-r)*x0 + r*x2;
		ys1 = (1-r)*y0 + r*y2;

		r = (0.0 - fmin)/(fav - fmin);
		xs2 = (1-r)*x0 + r*x1;
		ys2 = (1-r)*y0 + r*y1;

		Plot2DMoveTo(x0, y0);
		Plot2DLineTo(xs1, ys1);
		Plot2DLineTo(xs2, ys2);
		Plot2DLineTo(x0, y0);
	}
	else
	{
		r = (0.0 - fmin)/(fmax - fmin);
		xs1 = (1-r)*x0 + r*x2;
		ys1 = (1-r)*y0 + r*y2;

		r = (0.0 - fav)/(fmax - fav);
		xs2 = (1-r)*x1 + r*x2;
		ys2 = (1-r)*y1 + r*y2;

		Plot2DMoveTo(x0, y0);
		Plot2DLineTo(xs1, ys1);
		Plot2DLineTo(xs2, ys2);
		Plot2DLineTo(x1, y1);
		Plot2DLineTo(x0, y0);
	}

	s_ClosePoly();
	s_FillPoly(paintPH,s_g2d.pat);
	KillPoly(paintPH);
	
	return;

#if MOVEPOLY
}
#else
}
#endif

#if 0
void Graph2DPlotFrame()
{
	SetFrame2DXMinDigit(s_g2d.xMinLabelDigit);
	SetFrame2DXMaxDigit(s_g2d.xMaxLabelDigit);
	SetFrame2DYMinDigit(s_g2d.yMinLabelDigit);
	SetFrame2DYMaxDigit(s_g2d.yMaxLabelDigit);
	s_PenSize(s_g2d.frameWidth/s_d_scale,s_g2d.frameWidth/s_d_scale);
						/*s_d_scale is in StartSCL.c*/
	SetFont(s_g2d.defaultFont);
	s_TextSize(s_g2d.fontSize);
	PlotFrame2D(s_g2d.xmin,s_g2d.xmax,s_g2d.ymin,s_g2d.ymax);
	Plot2DMoveTo(s_g2d.xmax,s_g2d.ymin);
	s_Move(0.05*300/s_d_scale,0.13*300/s_d_scale);
	DrawStyleText(s_g2d.xLabel);
	Plot2DMoveTo(s_g2d.xmin,s_g2d.ymax);
	s_Move(-0.15*300/s_d_scale,-0.08*300/s_d_scale);
	DrawStyleText(s_g2d.yLabel);
}
#endif

void SetGraph2DFunction( double (*fp)(double, double) )
{
	s_g2d.funcNum = 0;
	AddGraph2DFunc(fp);
}

void AddGraph2DFunction( double (*fp)(double, double) )
{
	if (s_g2d.funcNum < G2D_MAX_FUNC_NUM)
	{
		s_g2d.f[s_g2d.funcNum] = fp;
		s_g2d.funcNum++;
	}
	else
	{
		s_printf("Error in AddGraph2DFunc:\n Too many functions\n");
	}
}

void SetGraph2DDXYF(double dx, double dy, double df)
{
	s_g2d.dx = dx;
	s_g2d.dy = dy;
	s_g2d.df = df;
}

void SetGraph2DContourLabelFlag(int flag) 
{
	s_g2d.contourLabelFlag = flag;
}

void SetGraph2DPaintNegativeFlag(int flag)
{
	s_g2d.paintNegativeFlag = flag;
}

void in_Graph2DAddParameter(Ptr p,int type,char* com)
{

#if APPEND_LAST2D
	if (! s_g.addParamFlag )
	{
		Graph2DFAddParameters();
	}
	
#else
	if (!s_g.PDSetUpFlag)
	{
		SetUpParametersDialogP(&s_g.dialogP);
		s_g.PDSetUpFlag = 1;
	}
#endif

	AddParameterP(p, type, com, s_g.dialogP);
}


void SG2DResetArray(void)
{
static struct
{
int notYet;
int contourLabelFlag;
int onlyZero;
double xMin;
double xMax;
double yMin;
double yMax;
double df;
int sF;
} old = {1, 0, 0, 0.0, 0.0, 0.0, 0.0, 0.0, 0};

	if (s_g2d.contourLabelFlag)
	{
		if (old.notYet
			|| s_g2d.contourLabelFlag > old.contourLabelFlag
			|| s_g2d.onlyZero < old.onlyZero
			|| s_g.limXMin < old.xMin
			|| s_g.limXMax > old.xMax
			|| s_g.limYMin < old.yMin
			|| s_g.limYMax > old.yMax
			|| s_g2d.df < old.df
			|| (s_sg.seriesF &&(s_sg.seriesF[0] != old.sF))
        )
		{
			old.notYet = 0;
			old.contourLabelFlag = s_g2d.contourLabelFlag;
			old.onlyZero = s_g2d.onlyZero;
			old.xMin = s_g.limXMin;
			old.xMax = s_g.limXMax;
			old.yMin = s_g.limYMin;
			old.yMax = s_g.limYMax;
			old.df = s_g2d.df;
            if (s_sg.seriesF)
                old.sF = s_sg.seriesF[0];
			Graph2DMaxAndMin();
			Graph2DMakeArray();
		}
	}
}


void ExtraHeightNew(void)
{
Point pen;

	if (!s_g2d.extraCLX)
	{
		s_g2d.extraCLX = MakeArrayInt1(s_g2d.extraCLMax);
		s_g2d.extraCLY = MakeArrayInt1(s_g2d.extraCLMax);
		s_g2d.extraCLValue = MakeArrayDouble1(s_g2d.extraCLMax);
	}
	
	if (s_g2d.extraCLCount >= s_g2d.extraCLMax)
	{
		AlertMessage("Too many extra labels!", 1);
		return;
	}
	
	while (!MousePressed());
	GetMouse(&pen);
	s_g2d.extraCLX[s_g2d.extraCLCount] = pen.h;
	s_g2d.extraCLY[s_g2d.extraCLCount] = pen.v;
	s_g2d.extraCLValue[s_g2d.extraCLCount]
		= Graph2DHeight(pen.h, pen.v);
	
	ExtraHeightDraw(s_g2d.extraCLCount);
	Stop();
	
	s_g2d.extraCLCount++;
}


double Graph2DHeight(int penX, int penY)
{
double x, y, rx, ry, x0, y0, value;
int nx, ny;

	x = Plot2DReverseX(penX);
	y = Plot2DReverseY(penY);
	if (!s_g.spreadSheet)
	{
		value = s_g2d.f[0](x, y);
	}
	else
	{
		
		nx = floor(x/s_g2d.dx);
		rx = x/s_g2d.dx - nx;
		ny = floor(y/s_g2d.dy);
		ry = y/s_g2d.dy - ny;
		
		x0 = nx * s_g2d.dx;
		y0 = ny * s_g2d.dy;
		
		if (rx + ry <= 1.0)
		{
			value = (1-rx-ry) * s_g2d.f[0](x0, y0)
					+ rx * s_g2d.f[0](x0+s_g2d.dx, y0)
					+ ry * s_g2d.f[0](x0, y0+s_g2d.dy);
		}
		else
		{
			value = (rx+ry-1) * s_g2d.f[0](x0 + s_g2d.dx, y0 + s_g2d.dy)
					+ (1-ry) * s_g2d.f[0](x0 + s_g2d.dx, y0)
					+ (1-rx) * s_g2d.f[0](x0, y0 + s_g2d.dy);
		}
	}
	
	return floor(value/s_g2d.df + 0.5)*s_g2d.df;
}

void ExtraHeightDraw(int count)
{
short txSize;

	txSize = GetPortTextSize(s_thePort);
	s_TextSize(s_g2d.contourLabelSize);
	s_Dprintf(s_g2d.extraCLX[count]-8,
			s_g2d.extraCLY[count]+4,
			"%.*f", s_g2d.contourLabelDigit, 
			s_g2d.extraCLValue[count]);
	s_TextSize(txSize);
}

void ResetExtraHeight(void)
{
	s_g2d.extraCLCount = 0;
}


void SetGraph2DOnlyZero(int onlyZero)
{
    s_g2d.onlyZero = onlyZero;
}

#endif /*pragma once */

