/*
 *  Graph3D.c
 *  Version 1.9 / 010207
 *  by Yukie Mimura and Satoshi Takahashi
 */

#ifndef GRAPH3D_C /*pragma once*/
#define GRAPH3D_C

#include <math.h>
#include <stdlib.h>
#include <string.h>

#include <s_Math.h>
#include <s_Drawing.h>
#include <s_Print.h>
#include <Save.h>
#include <s_RedrawFunc.h>
#include <HugeArray.h>
#include <s_Common.h>

#include <GraphCommon.h>
#include <GraphSpreadSheet.h>
#include <Graph3D.h>

#include <VarBody.h>
#include <Graph3D-v.c>

double SG3DIncidence(double x1, double y1, double z1, double z2, double z3, double z4);


void Graph3DFunction(void)
{
double  potential1;
double  potential2;
double  potential3;
double  potential4;
double  angle;
double  x;
double  y;
PolyHandle  thePoly;
g3dfType g3d;
double oldf0, oldf1;
int index;

	s_g.dim = 3;

	Graph3DFAddParameters();       
	
	if (s_g3d.useResolution)
	{
		s_g3d.xIncrement = s_g3d.xResolution/s_g3d.xScale;
		s_g3d.yIncrement = s_g3d.yResolution/s_g3d.yScale;
	}

	g3d.oldFArray = 0;
	
	while(1){
		ParametersDialogP(s_g.dialogP);
		if (s_g.exitFlag)
			break;
		if (s_g.spreadSheet)
		{
			SG2DArray();
			if (s_g3d.xMin < s_sg.min[s_sg.seriesX[0]-1])
				s_g3d.xMin = s_sg.min[s_sg.seriesX[0]-1];
			if (s_g3d.xMax > s_sg.max[s_sg.seriesX[0]-1])
				s_g3d.xMax = s_sg.max[s_sg.seriesX[0]-1];
			if (s_g3d.yMin < s_sg.min[s_sg.seriesY[0]-1])
				s_g3d.yMin = s_sg.min[s_sg.seriesY[0]-1];
			if (s_g3d.yMax > s_sg.max[s_sg.seriesY[0]-1])
				s_g3d.yMax = s_sg.max[s_sg.seriesY[0]-1];
		}
		
		SG3DSetting(&g3d);
		
		SG3DMakeArray(&g3d);
		
		EraseDrawing();

		if (s_g.printFlag)  BeginPrinting(); 
		else if (s_g.saveFlag)  BeginSaving();
		if (s_g.epsSaveFlag) BeginEPSSaving();
		
		for (x = g3d.xStart, index = 0; x*g3d.xSign < g3d.xEnd*g3d.xSign;
				x += g3d.xInc, index++)
		{
			g3d.oldFArray[index] = (*s_g3d.f)(x + (g3d.xSign > 0)*s_g3d.xIncrement,
							g3d.yStart + (g3d.ySign < 0)*s_g3d.yIncrement);
		}	
		for (y = g3d.yStart; y*g3d.ySign < g3d.yEnd*g3d.ySign; y += g3d.yInc)
		{
			oldf0 = s_g3d.f(g3d.xStart + (g3d.xSign < 0)*s_g3d.xIncrement, y);
			oldf1 = s_g3d.f(g3d.xStart + (g3d.xSign < 0)*s_g3d.xIncrement,
							y + s_g3d.yIncrement);
//s_printf("%d\n", (g3d.xSign > 0));
			for (x = g3d.xStart, index = 0; x*g3d.xSign < g3d.xEnd*g3d.xSign;
					x += g3d.xInc, index++)
			{
				thePoly = s_OpenPoly();
				if (g3d.xSign > 0)
				{
					potential1 = oldf0;
					potential2 = oldf1;
					if (g3d.ySign > 0)
					{
						oldf0 = potential4 = g3d.oldFArray[index];
						g3d.oldFArray[index] = oldf1 = potential3 = (*s_g3d.f)(x + s_g3d.xIncrement, y + s_g3d.yIncrement);
					}
					else
					{
						oldf1 = potential3 = g3d.oldFArray[index];
						g3d.oldFArray[index] = oldf0 = potential4 = (*s_g3d.f)(x + s_g3d.xIncrement, y);
					}
				}
				else
				{
					potential4 = oldf0;
					potential3 = oldf1;
					if (g3d.ySign > 0)
					{
						oldf0 = potential1 = g3d.oldFArray[index];
						g3d.oldFArray[index] = oldf1 = potential2 = (*s_g3d.f)(x, y+s_g3d.yIncrement);
					}
					else
					{
						oldf1 = potential2 = g3d.oldFArray[index];
						g3d.oldFArray[index] = oldf0 = potential1 = (*s_g3d.f)(x, y);
					}
				}

				if (s_g3d.color)
				{
					if (s_g3d.colorF && s_g3d.colorF(potential1, potential2, potential3, potential4))
					{
						/* something is done in s_g3d.colorF */
					}
					else
					{
						angle=SG3DIncidence(x, y, potential1, potential2, potential3, potential4);
						SetRGBColor(s_g3d.redBase + angle*(s_g3d.redTop - s_g3d.redBase),
							s_g3d.greenBase + angle*(s_g3d.greenTop - s_g3d.greenBase),
							s_g3d.blueBase + angle*(s_g3d.blueTop - s_g3d.blueBase));
					}
				}

				s_MoveTo(SG3DX(x, y, &g3d), SG3DY(x, y, potential1, &g3d));
				LineTo(SG3DX(x, y + s_g3d.yIncrement, &g3d),
						SG3DY(x, y + s_g3d.yIncrement, potential2, &g3d));
				LineTo(SG3DX(x + s_g3d.xIncrement, y + s_g3d.yIncrement, &g3d),
						SG3DY(x + s_g3d.xIncrement, y + s_g3d.yIncrement, potential3, &g3d));
				LineTo(SG3DX(x + s_g3d.xIncrement, y, &g3d),
						SG3DY(x + s_g3d.xIncrement, y, potential4, &g3d));
				LineTo(SG3DX(x, y, &g3d), SG3DY(x, y, potential1, &g3d));
				s_ClosePoly();
				
				if (s_g3d.color)
				{
					PaintPoly(thePoly);
					if (s_g3d.glid)
					{
						s_ForeColor(blackColor);
						FramePoly(thePoly);
					}
				}
				else
				{
					ErasePoly(thePoly);
					FramePoly(thePoly);
				}
				KillPoly(thePoly);  
			}
		} 

		if (s_g3d.showAxis)
		{
			s_ForeColor(blackColor);
			SG3DFrame(&g3d);
		}

		s_ForeColor(blackColor);
		if (s_g.printFlag)  EndPrinting(); 
		else if (s_g.saveFlag)  EndSaving(); 
		if (s_g.epsSaveFlag) EndEPSSaving();

    	Stop();   
	}
}


void SG3DSetting(g3dfType *g3dfP)
{
	s_g3d.xScale = s_g3d.xWidth/(s_g3d.xMax - s_g3d.xMin);
	if (s_g3d.equalScale)
		s_g3d.yScale = s_g3d.xScale;
	else
		s_g3d.yScale = s_g3d.yWidth/(s_g3d.yMax - s_g3d.yMin);
	//if (s_g3d.equalScale)
		//s_g3d.yScale = s_g3d.xScale;
	if (s_g3d.equalIncrement)
		s_g3d.yIncrement = s_g3d.xIncrement;
	if (s_g3d.equalResolution)
		s_g3d.yResolution = s_g3d.xResolution;
	if (s_g3d.useResolution)
	{
		s_g3d.xIncrement = s_g3d.xResolution/s_g3d.xScale;
		s_g3d.yIncrement = s_g3d.yResolution/s_g3d.yScale;
	}
	g3dfP->zView = cos(Pi*s_g3d.zAngle/180.0);
	g3dfP->zfView = sin(Pi*s_g3d.zAngle/180.0);
	g3dfP->xh = cos(Pi*s_g3d.xyAngle/180.0);
	g3dfP->xv = sin(Pi*s_g3d.xyAngle/180.0);
	g3dfP->yh = - sin(Pi*s_g3d.xyAngle/180.0);
	g3dfP->yv = cos(Pi*s_g3d.xyAngle/180.0);

	g3dfP->real_center = (s_g3d.xMax + s_g3d.xMin) / 2.0;
	g3dfP->imag_center = (s_g3d.yMax + s_g3d.yMin) / 2.0;
	
	if (g3dfP->xv > 0)
	{
		g3dfP->xStart = s_g3d.xMax - s_g3d.xIncrement;
		g3dfP->xEnd = s_g3d.xMin - 0.5*s_g3d.xIncrement;
		g3dfP->xInc = - s_g3d.xIncrement;
		g3dfP->xSign = -1;
	}
	else
	{
		g3dfP->xStart = s_g3d.xMin;
		g3dfP->xEnd = s_g3d.xMax - 0.5*s_g3d.xIncrement;
		g3dfP->xInc = s_g3d.xIncrement;
		g3dfP->xSign = 1;
	}

	if (g3dfP->yv > 0)
	{
		g3dfP->yStart = s_g3d.yMax - s_g3d.yIncrement;
		g3dfP->yEnd = s_g3d.yMin - 0.5*s_g3d.yIncrement;
		g3dfP->yInc = - s_g3d.yIncrement;
		g3dfP->ySign = -1;
	}
	else
	{
		g3dfP->yStart = s_g3d.yMin;
		g3dfP->yEnd = s_g3d.yMax - 0.5*s_g3d.yIncrement;
		g3dfP->yInc = s_g3d.yIncrement;
		g3dfP->ySign = 1;
	}
	s_g3d.lx = - sin(s_g3d.lightZAngle*Pi/180.0) * cos(s_g3d.lightXYAngle*Pi/180.0);
	s_g3d.ly = - sin(s_g3d.lightZAngle*Pi/180.0) * sin(s_g3d.lightXYAngle*Pi/180.0);
	s_g3d.lz = - cos(s_g3d.lightZAngle*Pi/180.0);
}

double SG3DX(double x, double y, g3dfType *g3dfP)
{
	return s_g3d.horizontal + s_g3d.xScale * g3dfP->xh *(x - g3dfP->real_center) 
		+ s_g3d.yScale * g3dfP->yh *(y - g3dfP->imag_center);
}


double SG3DY(double x, double y, double z, g3dfType *g3dfP)
{
	return s_g3d.vertical
			- g3dfP->zView * (s_g3d.xScale * g3dfP->xv *(x - g3dfP->real_center)
				+ s_g3d.yScale * g3dfP->yv * (y - g3dfP->imag_center))
			- g3dfP->zfView * s_g3d.mag * z;
}

double SG3DIncidence(double x1, double y1, double z1, double z2, double z3, double z4)
{
double x2, y2,x3, y3, x4, y4;
double a1, b1, c1, a2, b2, c2, a3, b3, c3;
double v_root, l_root;
double angle;

    x2 = x1;
	y2 = y1+s_g3d.yIncrement;
	x3 = x1+s_g3d.xIncrement;
	y3 = y1+s_g3d.yIncrement;
	x4 = x1+s_g3d.xIncrement;
    y4 = y1;
	a1 = s_g3d.xScale*(x3-x1);
	b1 = s_g3d.yScale*(y3-y1);
	c1 = s_g3d.mag*(z3-z1);
	a2 = s_g3d.xScale*(x4-x2);
	b2 = s_g3d.yScale*(y4-y2);
	c2 = s_g3d.mag*(z4-z2);
	a3 = b1*c2-b2*c1;
	b3 = c1*a2-c2*a1;
	c3 = a1*b2-a2*b1;
    v_root = sqrt(a3*a3+b3*b3+c3*c3);
    l_root = sqrt(s_g3d.lx*s_g3d.lx+s_g3d.ly*s_g3d.ly+s_g3d.lz*s_g3d.lz);
	angle =  a3/v_root*s_g3d.lx/l_root+b3/v_root*s_g3d.ly/l_root+c3/v_root*s_g3d.lz/l_root;
	
	if (angle<0)
		return 0;
	else
		return angle;
}



void SetGraph3DFunction( double (*fp)(double, double) )
{
	s_g3d.f = fp;
}




#ifndef APPEND_LAST3D
#define APPEND_LAST3D 0
#endif
void in_Graph3DAddParameter(Ptr p,int type,char* com)
{

#if APPEND_LAST3D
	if (! s_g.addParamFlag )
	{
		Graph3DFAddParameters();
	}
	
#else
	if (!s_g.PDSetUpFlag)
	{
		SetUpParametersDialogP(&s_g.dialogP);
		s_g.PDSetUpFlag = 1;
	}
#endif

	AddParameterP(p, type, com, s_g.dialogP);
}

void SetGraph3DColorFunction( int (*fp)(double, double, double, double) )
{
	s_g3d.colorF = fp;
}


void Graph3DFAddParameters(void)
{
	if (!s_g.PDSetUpFlag)
	{
		SetUpParametersDialogP(&s_g.dialogP);
		s_g.PDSetUpFlag = 1;
	}
	
	GraphAddParameters1();

    AddParameterP(&Stop,doType, "Show", s_g.dialogP);
    AddParameterP(&s_g3d.color,flagType, "Color", s_g.dialogP);
    AddParameterP(&s_g3d.glid,flagType, "Glid", s_g.dialogP);
    AddParameterP(&s_g3d.xMin,doubleType, "X Min", s_g.dialogP);
    AddParameterP(&s_g3d.xMax,doubleType, "X Max", s_g.dialogP);
    AddParameterP(&s_g3d.yMin,doubleType, "Y Min", s_g.dialogP);
    AddParameterP(&s_g3d.yMax,doubleType, "Y Max", s_g.dialogP);
    AddParameterP(&s_g3d.xWidth, intType, "X Width", s_g.dialogP);
    AddParameterP(&s_g3d.yWidth, intType, "Y Width", s_g.dialogP);
    AddParameterP(&s_g3d.equalScale, flagType, "Same X Y Scale", s_g.dialogP);
    AddParameterP(&s_g3d.mag,doubleType, "Func Scale", s_g.dialogP);
	if (s_g.spreadSheet)
	{
		SG3DInit();
	}
	else
	{
	    AddParameterP(&s_g3d.xIncrement,doubleType, "X Increment", s_g.dialogP);
	    AddParameterP(&s_g3d.yIncrement,doubleType, "Y Increment", s_g.dialogP);
	    AddParameterP(&s_g3d.equalIncrement, flagType, "Same X Y Inc.", s_g.dialogP);
	    AddParameterP(&s_g3d.useResolution, flagType, "Use Resolution", s_g.dialogP);
	    AddParameterP(&s_g3d.xResolution,doubleType, "X Resolustion", s_g.dialogP);
	    AddParameterP(&s_g3d.yResolution,doubleType, "Y Resolustion", s_g.dialogP);
	    AddParameterP(&s_g3d.equalResolution, flagType, "Same X Y Res.", s_g.dialogP);
    }
    AddParameterP(&s_g3d.horizontal, doubleType, "Horizontal Pos.", s_g.dialogP);
    AddParameterP(&s_g3d.vertical, doubleType, "Vertical Pos.", s_g.dialogP);
    AddParameterP(&s_g3d.xyAngle, doubleType, "Angle XY", s_g.dialogP);
    AddParameterP(&s_g3d.zAngle, doubleType, "Angle Z", s_g.dialogP);
    AddParameterP(&s_g3d.lightXYAngle, doubleType, "Light XY Angle", s_g.dialogP);
    AddParameterP(&s_g3d.lightZAngle, doubleType, "Light Z Angle", s_g.dialogP);
    AddParameterP(&s_g3d.redTop, doubleType, "Light Red", s_g.dialogP);
    AddParameterP(&s_g3d.greenTop ,doubleType, "Light Green", s_g.dialogP);
    AddParameterP(&s_g3d.blueTop, doubleType, "Light Blue", s_g.dialogP);
    AddParameterP(&s_g3d.redBase, doubleType, "Shadow Red", s_g.dialogP);
    AddParameterP(&s_g3d.greenBase, doubleType, "Shadow Green", s_g.dialogP);
    AddParameterP(&s_g3d.blueBase, doubleType, "Shadow Blue", s_g.dialogP);
    AddParameterP(&s_g3d.showAxis, flagType, "Show Axis", s_g.dialogP);
    AddParameterP(&s_g3d.arrow, flagType, "Arrow", s_g.dialogP);
    AddParameterP(&s_g3d.showXLabel, flagType, "Show X Label", s_g.dialogP);
    AddParameterP(&s_g3d.showYLabel, flagType, "Show Y Label", s_g.dialogP);
    AddParameterP(&s_g3d.showFLabel, flagType, "Show Func Label", s_g.dialogP);
    AddParameterP(&s_g3d.xLabel, stringType, "X Label", s_g.dialogP);
    AddParameterP(&s_g3d.yLabel, stringType, "Y Label", s_g.dialogP);
    AddParameterP(&s_g3d.fLabel, stringType, "Func Label", s_g.dialogP);
    AddParameterP(&s_g3d.labelSize, intType, "Label Size", s_g.dialogP);
    AddParameterP(&s_g3d.fMin, doubleType, "Func Min", s_g.dialogP);
    AddParameterP(&s_g3d.fMax, doubleType, "Func Max", s_g.dialogP);
    AddParameterP(&s_g3d.showXMin, flagType, "Show X Min", s_g.dialogP);
    AddParameterP(&s_g3d.showXMax, flagType, "Show X Max", s_g.dialogP);
    AddParameterP(&s_g3d.showYMin, flagType, "Show Y Min", s_g.dialogP);
    AddParameterP(&s_g3d.showYMax, flagType, "Show Y Max", s_g.dialogP);
    AddParameterP(&s_g3d.showFMin, flagType, "Show Func Min", s_g.dialogP);
    AddParameterP(&s_g3d.showFMax, flagType, "Show Func Max", s_g.dialogP);
    AddParameterP(&s_g3d.xMinDigit, intType, "X Min Digit", s_g.dialogP);
    AddParameterP(&s_g3d.xMaxDigit, intType, "X Max Digit", s_g.dialogP);
    AddParameterP(&s_g3d.yMinDigit, intType, "Y Min Digit", s_g.dialogP);
    AddParameterP(&s_g3d.yMaxDigit, intType, "Y Max Digit", s_g.dialogP);
    AddParameterP(&s_g3d.fMinDigit, intType, "Func Min Digit", s_g.dialogP);
    AddParameterP(&s_g3d.fMaxDigit, intType, "Func Max Digit", s_g.dialogP);
    AddParameterP(&s_g3d.xLabelX, intType, "X Label X Pos.", s_g.dialogP);
    AddParameterP(&s_g3d.xLabelY, intType, "X Label Y Pos.", s_g.dialogP);
    AddParameterP(&s_g3d.yLabelX, intType, "Y Label X Pos.", s_g.dialogP);
    AddParameterP(&s_g3d.yLabelY, intType, "Y Label Y Pos.", s_g.dialogP);
    AddParameterP(&s_g3d.fLabelX, intType, "Func. Label X Pos.", s_g.dialogP);
    AddParameterP(&s_g3d.fLabelY, intType, "Func. Label Y Pos.", s_g.dialogP);
    AddParameterP(&s_g3d.xDisplaceX, intType, "X Num X Pos.", s_g.dialogP);
    AddParameterP(&s_g3d.xDisplaceY, intType, "X Num Y Pos.", s_g.dialogP);
    AddParameterP(&s_g3d.yDisplaceX, intType, "Y Num X Pos.", s_g.dialogP);
    AddParameterP(&s_g3d.yDisplaceY, intType, "Y Num Y Pos.", s_g.dialogP);
    AddParameterP(&s_g3d.fDisplaceX, intType, "Func. Num X Pos.", s_g.dialogP);
    AddParameterP(&s_g3d.fDisplaceY, intType, "Func. Num Y Pos.", s_g.dialogP);
}


void SG3DFrame(g3dfType *g3dfP)
{
	s_MoveTo(SG3DX(s_g3d.xMin, s_g3d.yMin, g3dfP),
			SG3DY(s_g3d.xMin, s_g3d.yMin, s_g3d.fMin, g3dfP));
	if (s_g3d.showXMin)
		DrawValue(s_g3d.xDisplaceX, s_g3d.xDisplaceY, s_g3d.xMinDigit, s_g3d.xMin);

	LineTo(SG3DX(s_g3d.xMax, s_g3d.yMin, g3dfP),
			SG3DY(s_g3d.xMax, s_g3d.yMin, s_g3d.fMin, g3dfP));
	if (s_g3d.showXMax)
		DrawValue(s_g3d.xDisplaceX, s_g3d.xDisplaceY, s_g3d.xMaxDigit, s_g3d.xMax);
	if (s_g3d.showXLabel)
		DrawLabel(s_g3d.xLabelX, s_g3d.xLabelY, s_g3d.xLabel);
	if (s_g3d.arrow)
		G3DDrawArrow(s_g3d.xMin, s_g3d.yMin, s_g3d.fMin,
					s_g3d.xMax, s_g3d.yMin, s_g3d.fMin, g3dfP);

	s_MoveTo(SG3DX(s_g3d.xMin, s_g3d.yMin, g3dfP),
			SG3DY(s_g3d.xMin, s_g3d.yMin, s_g3d.fMin, g3dfP));
	if (s_g3d.showYMin)
		DrawValue(s_g3d.yDisplaceX, s_g3d.yDisplaceY, s_g3d.yMinDigit, s_g3d.yMin);

	LineTo(SG3DX(s_g3d.xMin, s_g3d.yMax, g3dfP),
			SG3DY(s_g3d.xMin, s_g3d.yMax, s_g3d.fMin, g3dfP));
	if (s_g3d.showYMax)
		DrawValue(s_g3d.yDisplaceX, s_g3d.yDisplaceY, s_g3d.yMaxDigit, s_g3d.yMax);
	if (s_g3d.showYLabel)
		DrawLabel(s_g3d.yLabelX, s_g3d.yLabelY, s_g3d.yLabel);
	if (s_g3d.arrow)
		G3DDrawArrow(s_g3d.xMin, s_g3d.yMin, s_g3d.fMin,
					s_g3d.xMin, s_g3d.yMax, s_g3d.fMin, g3dfP);
	
	s_MoveTo(SG3DX(s_g3d.xMin, s_g3d.yMin, g3dfP),
			SG3DY(s_g3d.xMin, s_g3d.yMin, s_g3d.fMin, g3dfP));
	if (s_g3d.showFMin)
		DrawValue(s_g3d.fDisplaceX, s_g3d.fDisplaceY, s_g3d.fMinDigit, s_g3d.fMin);

	LineTo(SG3DX(s_g3d.xMin, s_g3d.yMin, g3dfP),
			SG3DY(s_g3d.xMin, s_g3d.yMin, s_g3d.fMax, g3dfP));
	if (s_g3d.showFMax)
		DrawValue(s_g3d.fDisplaceX, s_g3d.fDisplaceY, s_g3d.fMaxDigit, s_g3d.fMax);
	if (s_g3d.showFLabel)
		DrawLabel(s_g3d.fLabelX, s_g3d.fLabelY, s_g3d.fLabel);
	if (s_g3d.arrow)
		G3DDrawArrow(s_g3d.xMin, s_g3d.yMin, s_g3d.fMin,
					s_g3d.xMin, s_g3d.yMin, s_g3d.fMax, g3dfP);

	TextSize(12);
}


void DrawValue(short x, short y, int digit, double value)
{
char scale_num[256];
Point penPos;
short txSize;

	txSize = GetPortTextSize(s_thePort);
	GetPen(&penPos);
	
	s_TextSize(s_g3d.labelSize);
	s_Move(x, y);
	sprintf(scale_num,"%.*f", digit, value);
	DrawText(scale_num,0,strlen(scale_num));
	s_MoveTo(penPos.h, penPos.v);
	s_TextSize(txSize);
}


void DrawLabel(short x, short y, char *label)
{
Point penPos;
short txSize;

	txSize = GetPortTextSize(s_thePort);
	GetPen(&penPos);

	s_TextSize(s_g3d.labelSize);
	s_Move(x, y);
	DrawStyleText(label);
	s_MoveTo(penPos.h, penPos.v);
	s_TextSize(txSize);
}


void SG3DMakeArray(g3dfType *g3dP)
{
	if (g3dP->oldFArray)
		free(g3dP->oldFArray);
	
	g3dP->oldFArray = MakeArrayDouble1((s_g3d.xMax - s_g3d.xMin)/s_g3d.xIncrement+1.5);
	if (!g3dP->oldFArray)
		s_OOMError();
}


void G3DDrawArrow(double x1, double y1, double f1,
					double x2, double y2, double f2, g3dfType *g3dfP)
{
short ax1, ay1, ax2, ay2;

	ax1 = SG3DX(x1, y1, g3dfP);
	ay1 = SG3DY(x1, y1, f1, g3dfP);
	ax2 = SG3DX(x2, y2, g3dfP);
	ay2 = SG3DY(x2, y2, f2, g3dfP);
	AxisArrow(ax1, ay1, ax2, ay2);
}



#endif /*pragma once*/