/*
 *  GraphSpreadSheet.c
 *  Version 1.9.1 / 010220
 *  by Satoshi Takahashi
 */

#ifndef GRAPHSPREADSHEET_C /*pragma once*/
#define GRAPHSPREADSHEET_C

#include <math.h>
#include <string.h>

#include <HugeArray.h>
#include <DragAndDrop.h>
#include <DialogFOpen.h>
#include <StartSCL.h>
#include <Plot.h>
#include <s_RedrawFunc.h>
#include <s_String.h>
#include <s_Math.h>
#include <DelayedSetParams.h>
#include <s_Common.h>

#include <GraphCommon.h>
#include <Graph1D.h>
#include <Graph2D.h>
#include <Graph3D.h>

#include <GraphSpreadSheet.h>

#include <VarBody.h>
#include <GraphSpreadSheet-v.c>

void SpreadSheetGraph(void)
{
	s_g1.type = spreadSheetType;
	DrawGraph();
}

void SGInit()
{
int i=0, sn = 0;
long j = 0;
char *oneLineP;
char dataString[8];
static int repeat = 0;
int readFlag = 1;

#if !S_UNIX
	if (DDCount())
	{
		s_sg.inFile = DDFOpen(s_sg.fName,"r");
	}
	else
#endif /*!S_UNIX*/
		if(s_sg.fNameFlag)
	{
		s_sg.inFile = DialogFOpen(s_sg.fName,"r");
	}
	else
	{
		s_sg.inFile = fopen(s_sg.fName,"r");
	}
	
	if (!s_sg.inFile)
		return;
	
	fgets(s_sg.oneLine,1024,s_sg.inFile);
	i = 0;
    s_sg.seriesNum = 1;
	for (oneLineP = s_sg.oneLine; oneLineP[0]; oneLineP++)
    {
        s_sg.seriesNum += (oneLineP[0] == '\t');
    }
    //if (s_sg.seriesNum > s_sg.maxUseSeriesNum)
        //s_sg.seriesNum = s_sg.maxUseSeriesNum; /*bug!!*/
    
    s_sg.seriesTitle = calloc(s_sg.seriesNum, 128);
    
    oneLineP = s_sg.oneLine;
	for (i = 0; i < s_sg.seriesNum; i++)
    {
        sscanf(oneLineP,"%s%n",s_sg.seriesTitle[i], &sn);
        oneLineP += sn +1;
	}
	if (!repeat)
	{
		AddParameterP(&s_sg.seriesNum, staticIntType, s_sg.serNumString, s_g.dialogP);
		for (i = 0; i < s_sg.seriesNum; i++)
		{
			sprintf(dataString,"Data %d",i+1);
			AddParameterP(&s_sg.seriesTitle[i], stringType,dataString, s_g.dialogP);
		}
	}
	if (!s_sg.data)
	{
		s_sg.data = MakeArrayDouble2(s_sg.seriesNum, s_sg.data_number);
		s_sg.isNumber = MakeArrayInt2(s_sg.seriesNum, s_sg.data_number);

		if (!s_sg.useSeriesNum)
			s_sg.useSeriesNum = min(s_sg.seriesNum, s_sg.maxUseSeriesNum);

		s_sg.seriesX = MakeArrayInt1(s_sg.useSeriesNum);
		s_sg.seriesY = MakeArrayInt1(s_sg.useSeriesNum);
		s_sg.seriesF = MakeArrayInt1(s_sg.useSeriesNum);
		s_sg.seriesSD = MakeArrayInt1(s_sg.useSeriesNum);
		s_sg.plotSeriesMin = MakeArrayDouble1(s_sg.useSeriesNum);
		s_sg.plotSeriesMax = MakeArrayDouble1(s_sg.useSeriesNum);
		s_sg.pen = MakeArrayDouble1(s_sg.useSeriesNum);
		s_sg.noLineFlag = MakeArrayInt1(s_sg.useSeriesNum);
		s_sg.radius = MakeArrayInt1(s_sg.useSeriesNum);
		s_sg.circleFlag = MakeArrayInt1(s_sg.useSeriesNum);
		
		s_sg.min = MakeArrayDouble1(s_sg.seriesNum);
		s_sg.max = MakeArrayDouble1(s_sg.seriesNum);
		s_sg.step = MakeArrayDouble1(s_sg.seriesNum);

		s_sg.color = MakeArrayInt1(s_sg.useSeriesNum);
		
		s_sg.seriesX[0] = 1;
		s_sg.seriesY[0] = 2;
		s_sg.seriesF[0] = 3;
		for (i = 0; i < s_sg.useSeriesNum; i++)
		{
			s_sg.pen[i] = i + 1;
			s_sg.radius[i] = i + 1;
			s_sg.color[i] = i + 1;
		}
	}
	
	readAgain:
	j = 0;
	do
	{
		fgets(s_sg.oneLine,1024,s_sg.inFile);
		if (strlen(s_sg.oneLine) > 1 /*=strlen("\n")*/) /*skip empty line*/
		{
			if (readFlag)
			{
				oneLineP = s_sg.oneLine;
				for (i = 0; i < s_sg.seriesNum; i++)
				{ 
					s_sg.isNumber[i][j] = (SScanDoubleTabSeparated(&oneLineP, &(s_sg.data[i][j]), 1) > 0);
//s_printf("%d ", s_sg.isNumber[i][j]);
				}
//s_printf("\n");
				j++;
				if (j >= s_sg.data_number)
				{
					//PrintError("Data is too many! Set the data number by SetGraphDataNumber(n);\n");
							/*exit*/
					FreeArrayDouble2(s_sg.data, s_sg.seriesNum, s_sg.data_number);
					readFlag = 0;
				}
			}
			else
				j++;
		}
	} while(!feof(s_sg.inFile));
	
	if (!readFlag)
	{
		readFlag = 1;
		s_sg.data_number = j+1;
		s_sg.data = MakeArrayDouble2(s_sg.seriesNum, s_sg.data_number);
		rewind(s_sg.inFile);
		clearerr(s_sg.inFile);
		fgets(s_sg.oneLine, 1024, s_sg.inFile);
		goto readAgain;
	}

	s_sg.realDataNumber = j;
	
	fclose(s_sg.inFile);
	repeat=1;
	
	SGSeriesRange();

	if (s_g.dim == 1)
	{
		s_g.limXMin = s_sg.min[s_sg.seriesX[0]-1];
		s_g.limXMax = s_sg.max[s_sg.seriesX[0]-1];
		if (s_g.limXMax == s_g.limXMin)
			s_g.limXMax = s_g.limXMin + 1;
		s_g.limYMin = floor(s_sg.min[s_sg.seriesY[0]-1]);
		s_g.limYMax = ceil(s_sg.max[s_sg.seriesY[0]-1]);
		if (s_g.limYMax == s_g.limYMin)
			s_g.limYMax = s_g.limYMin + 1;
		for (i = 0; i < s_sg.useSeriesNum; i++)
		{
			s_sg.plotSeriesMin[i] = s_g.limXMin;
			s_sg.plotSeriesMax[i] = s_g.limXMax;
		}
	}

	DelaySetParamsIntArray(s_sg.useSeriesNum, 0);
	DelaySetParamsDoubleArray(s_sg.useSeriesNum, 0);
}


void GraphPlotSpreadSheet(void)
{
int i;

	for (i = 0; i < s_sg.useSeriesNum; i++)
	{
		if (!s_sg.seriesX[i])
			continue;
		if (s_g.colorFlag)
		{
			switch (s_sg.color[i] % 6)
			{
			case 1:
				s_ForeColor(s_p.color1);
				break;
			case 2:
				s_ForeColor(s_p.color2);
				break;
			case 3:
				s_ForeColor(s_p.color3);
				break;
			case 4:
				s_ForeColor(s_p.color4);
				break;
			case 5:
				s_ForeColor(s_p.color5);
				break;
			case 0:
				s_ForeColor(s_p.color6);
				break;
			}
		}
		PlotSpreadSheetOneSeries(s_sg.seriesX[i]-1, s_sg.seriesY[i]-1, s_sg.seriesSD[i]-1,
						s_sg.pen[i], s_sg.radius[i], s_sg.noLineFlag[i], s_sg.circleFlag[i],
						s_sg.plotSeriesMin[i],s_sg.plotSeriesMax[i]);
	}
	
	if (s_sg.regLineFlag)
	{
		if (s_g.colorFlag)
			s_ForeColor(blueColor);
		Graph1DRegressionLine(s_sg.seriesX[0]-1, s_sg.seriesY[0]-1);
	}

	if (s_g.colorFlag)
		s_ForeColor(blackColor);
}

void PlotSpreadSheetOneSeries(int xSeries, int ySeries, int stdDevSeries, 
		double pen, double radius, int noLineFlag, int circleFlag,
		double plotSeriesMin, double plotSeriesMax)
{
long i;
int firstIn = 0;
double x, y;

	s_PenSize(pen/s_d_scale + 0.5, pen/s_d_scale + 0.5);
	
	x = GraphConvertX(s_sg.data[xSeries][0]);
	y = GraphConvertY(s_sg.data[ySeries][0]);
		
	SetPlot2D2I(x, y);
	
	for( i = 0; i < s_sg.realDataNumber; i++)
	{
//s_printf("%d, %d\n", s_sg.isNumber[xSeries][i], s_sg.isNumber[ySeries][i]);
        if (!s_sg.isNumber[xSeries][i]
                || !s_sg.isNumber[ySeries][i])
            continue;
		x = GraphConvertX(s_sg.data[xSeries][i]);
		y = GraphConvertY(s_sg.data[ySeries][i]);

		if (s_sg.printDataFlag)
		{
			s_printf("%f, %f\n", x, y);
		}

		if ( s_sg.data[xSeries][i] < s_g.limXMin
				|| s_sg.data[xSeries][i] < plotSeriesMin)
		{
			firstIn = 0;
		}
		else if ( s_sg.data[xSeries][i] > s_g.limXMax
					|| s_sg.data[xSeries][i] > plotSeriesMax)
		{
			firstIn = 0;
		}
		else
		{
			if (!firstIn)
			{
				firstIn = 1;
			}
			else
			{					
				if (!noLineFlag)
					Plot2D2(x,y);
			}
			if (stdDevSeries >= 0)
				StdDevBar( xSeries, ySeries, stdDevSeries, i);

			SetPlot2D2I(x,y);

			switch (circleFlag)
			{
			case 1:
				PlotPaintCircle(radius);
				break;
			case 2:
				PlotCircle(radius);
				break;
			case 3:
				PlotPaintRect(radius);
				break;
			case 4:
				PlotRect(radius);
				break;
			case 5:
				PlotCross(radius);
				break;
			case 6:
				PlotPaintTriangle(radius);
				break;
			case 7:
				PlotTriangle(radius);
				break;
			}
		}
	}
}

void StdDevBar(int xSeries, int ySeries, int stdDevSeries, int dataNumber)
{
		Plot2DMoveTo(s_sg.data[xSeries][dataNumber],
			s_sg.data[ySeries][dataNumber]-s_sg.data[stdDevSeries][dataNumber]);
		s_Move(-5,0);
		Line(10,0);
		s_Move(-5,0);
		Plot2DLineTo(s_sg.data[xSeries][dataNumber],
			s_sg.data[ySeries][dataNumber]+s_sg.data[stdDevSeries][dataNumber]);
		s_Move(-5,0);
		Line(10,0);
}

void SGSS1DAddParameters(void)
{
int i;
char PDString[32];

	//AddParameterP(&s_g.colorFlag, flagType, "Color", s_g.dialogP);
	AddParameterP(&s_sg.realDataNumber, longType, "Data Number", s_g.dialogP);
	for (i = 0; i < s_sg.useSeriesNum; i++)
	{
		sprintf(PDString, "Plot %d X", i+1);
		AddParameterP(&s_sg.seriesX[i], intType, PDString, s_g.dialogP);
		sprintf(PDString, "Plot %d Y", i+1);
		AddParameterP(&s_sg.seriesY[i], intType, PDString, s_g.dialogP);
		sprintf(PDString, "Plot %d SD", i+1);
		AddParameterP(&s_sg.seriesSD[i], intType, PDString, s_g.dialogP);
	}
	for (i = 0; i < s_sg.useSeriesNum; i++)
	{
		sprintf(PDString, "Plot %d Min", i+1);
		AddParameterP(&s_sg.plotSeriesMin[i], doubleType, PDString, s_g.dialogP);
		sprintf(PDString, "Plot %d Max", i+1);
		AddParameterP(&s_sg.plotSeriesMax[i], doubleType, PDString, s_g.dialogP);
	}
	
	for (i = 0; i < s_sg.useSeriesNum; i++)
	{
		sprintf(PDString, "Pen %d Width", i+1);
		AddParameterP(&s_sg.pen[i], doubleType, PDString, s_g.dialogP);
	}
	for (i = 0; i < s_sg.useSeriesNum; i++)
	{
		sprintf(PDString, "No Line %d", i+1);
		AddParameterP(&s_sg.noLineFlag[i], flagType, PDString, s_g.dialogP);
	}
	for (i = 0; i < s_sg.useSeriesNum; i++)
	{
		sprintf(PDString, "Data Mark %d", i+1);
		AddParameterP(&(s_sg.circleFlag[i]), intType, PDString, s_g.dialogP);
	}
	for (i = 0; i < s_sg.useSeriesNum; i++)
	{
		sprintf(PDString, "Mark %d Radius", i+1);
		AddParameterP(&s_sg.radius[i], intType, PDString, s_g.dialogP);
	}
	for (i = 0; i < s_sg.useSeriesNum; i++)
	{
		sprintf(PDString, "Data %d Color", i+1);
		AddParameterP(&s_sg.color[i], intType, PDString, s_g.dialogP);
	}
	AddParameterP(&s_sg.regLineFlag, flagType, "Regression Line", s_g.dialogP);
	AddParameterP(&s_sg.showGradientFlag, flagType, "Show Gradient", s_g.dialogP);
	AddParameterP(&s_sg.gradientSize, intType, "Grad. FontSize", s_g.dialogP);
	AddParameterP(&s_sg.regressionLineRangeFlag, flagType, "Reg. Line Range", s_g.dialogP);
	AddParameterP(&s_sg.regressionLineMin, doubleType, "Reg. Line Min", s_g.dialogP);
	AddParameterP(&s_sg.regressionLineMax, doubleType, "Reg. Line Max", s_g.dialogP);
	AddParameterP(&s_sg.log_x_flag, flagType, "log(x)", s_g.dialogP);
	AddParameterP(&s_sg.log_y_flag, flagType, "log(y)", s_g.dialogP);			
	AddParameterP(&s_sg.printDataFlag, flagType, "Show Data", s_g.dialogP);			
}
/******************** Setting Parameters ********************/

void SetGraphDataNumber(long data_number)
{
	s_sg.data_number = data_number;	
}

void SetGraphMaxSeriesNumber(int seriesNumber)
{
	s_sg.maxUseSeriesNum = seriesNumber;	
}

void SetGraphRegressionLineFlag(int flag)
{
	s_sg.regLineFlag = flag;
}

void SetGraphShowGradientFlag(int flag)
{
	s_sg.showGradientFlag = flag;
}

void SetGraphGradientFontSize(int fontsize)
{
	s_sg.gradientSize = fontsize;
}
void SetGraphFileName(const char* fName)
{
	strcpy(s_sg.fName,fName);
	s_sg.fNameFlag = 0;
}

void SetGraphLogXFlag(int flag)
{
	s_sg.log_x_flag = flag;
}

void SetGraphLogYFlag(int flag)
{
	s_sg.log_y_flag = flag;
}

void SetGraphShowDataFlag(int flag)
{
	s_sg.printDataFlag = flag;
}

void SetGraphSeriesX(int i, int series)
{
	SetParamsIntArray(&s_sg.seriesX, i - 1, series, 0);
}

void SetGraphSeriesY(int i, int series)
{
	SetParamsIntArray(&s_sg.seriesY, i - 1, series, 0);
}

void SetGraphSeriesF(int i, int series)
{
	SetParamsIntArray(&s_sg.seriesF, i - 1, series, 0);
}

void SetGraphSeriesStdDev(int i, int series)
{
	SetParamsIntArray(&s_sg.seriesSD, i - 1, series, 0);
}

void SetGraphPenWidth(int i, double pen)
{
	SetParamsDoubleArray(&(s_sg.pen), i - 1, pen, 0);
}

void SetGraphNoLine(int i, int noLine)
{
	SetParamsIntArray(&s_sg.noLineFlag, i - 1, noLine, 0);
}

void SetGraphDataMark(int i, int dataMark)
{
	SetParamsIntArray(&s_sg.circleFlag, i - 1, dataMark, 0);
}

void SetGraphRadius(int i, int radius)
{
	SetParamsIntArray(&s_sg.radius, i - 1, radius, 0);
}

/******************** Regression Line ********************/

void Graph1DRegressionLine(int xSeries, int ySeries)
{
long i;
double sum_X_Y = 0;
double sum_X_2 = 0;
double average_X=0;
double average_Y=0;
double gradient;
double Y_segment;
char gradientText[256];
double x, y, xMin, xMax;
long rangeNumber=0;
	
	
	for(i=0; i < s_sg.realDataNumber; i++)
	{
		if( (!s_sg.regressionLineRangeFlag) ||
			s_sg.regressionLineMin<s_sg.data[xSeries][i]&&
			s_sg.regressionLineMax>s_sg.data[xSeries][i])
		{
			x = GraphConvertX(s_sg.data[xSeries][i]);
			average_X += x;
			
			y = GraphConvertY(s_sg.data[ySeries][i]);
			average_Y += y;
			
			rangeNumber++;

		}
	}

	average_X /= rangeNumber;
	average_Y /= rangeNumber;
	
	for(i=0; i < s_sg.realDataNumber; i++)
	{
		if( (!s_sg.regressionLineRangeFlag) ||
			s_sg.regressionLineMin<s_sg.data[xSeries][i]&&
			s_sg.regressionLineMax>s_sg.data[xSeries][i])
		{
			x = GraphConvertX(s_sg.data[xSeries][i]);			
			sum_X_2 += (x-average_X)*(x-average_X);
			y = GraphConvertY(s_sg.data[ySeries][i]);			
			sum_X_Y +=(x-average_X)*(y-average_Y);
		}
	}
	
	gradient=sum_X_Y/sum_X_2;
		
	Y_segment=average_Y-gradient*average_X;

	xMin = GraphConvertX(s_g.limXMin);
	xMax = GraphConvertX(s_g.limXMax);
	
	Plot2DMoveTo(xMin, gradient*xMin + Y_segment);
	Plot2DLineTo(xMax, gradient*xMax + Y_segment);
	
	if (s_sg.showGradientFlag)
	{
		Move(-40, -8);
		s_TextSize(s_sg.gradientSize);
		sprintf(gradientText, "a = %f", gradient);
		DrawStyleText(gradientText);
		s_TextSize(s_g.labelSize);
	}
	
	if(s_sg.regressionLineRangeFlag)
	{
		xMin = GraphConvertX(s_sg.regressionLineMin);
		xMax = GraphConvertX(s_sg.regressionLineMax);
	
		Plot2DMoveTo(xMin,gradient*xMin + Y_segment);
		Move(0,5);
		Line(0,-10);
		Plot2DMoveTo(xMax,gradient*xMax + Y_segment);
		Move(0,5);
		Line(0,-10);
	}
}

double GraphConvertX(double x)
{
	if (s_sg.log_x_flag)
		return SafeLog(x);
	else
		return x;
}	

double GraphConvertY(double y)
{
	if (s_sg.log_y_flag)
		return SafeLog(y);
	else
		return y;
}	

/********** 2D **********/

void Graph2DSS(void)
{
	s_g2d.contourFreq = 0.1;
	s_g.spreadSheet = 1;
	SetGraph2DFunc(&Graph2DSSFunc);
	Graph2DF();
}

void SG2DInit(void)
{
int i;
char PDString[32];

	s_sg.maxUseSeriesNum = 1;
	SGInit();
	AddParameterP(&s_sg.realDataNumber, longType, "Data Number", s_g.dialogP);
	for (i = 0; i < s_sg.useSeriesNum; i++)
	{
		sprintf(PDString, "Plot %d X", i+1);
		AddParameterP(&s_sg.seriesX[i], intType, PDString, s_g.dialogP);
		sprintf(PDString, "Plot %d Y", i+1);
		AddParameterP(&s_sg.seriesY[i], intType, PDString, s_g.dialogP);
		sprintf(PDString, "Plot %d F", i+1);
		AddParameterP(&s_sg.seriesF[i], intType, PDString, s_g.dialogP);
	}
	s_g.limXMin = s_sg.min[s_sg.seriesX[0]-1];
	s_g.limXMax = s_sg.max[s_sg.seriesX[0]-1];
	s_g.limYMin = s_sg.min[s_sg.seriesY[0]-1];
	s_g.limYMax = s_sg.max[s_sg.seriesY[0]-1];
	s_g2d.dx = s_sg.step[s_sg.seriesX[0]-1];
	s_g2d.dy = s_sg.step[s_sg.seriesY[0]-1];
}

void SGSeriesRange(void)
{
int series;
long i;
double step;

	for (series = 0; series < s_sg.seriesNum; series++)
	{
		s_sg.min[series] = s_sg.data[series][0];
		for (i = 0; i < s_sg.realDataNumber; i++)
		{
			if (s_sg.min[series] > s_sg.data[series][i])
				s_sg.min[series] = s_sg.data[series][i];
		}
		s_sg.max[series] = s_sg.data[series][0];
		for (i = 0; i < s_sg.realDataNumber; i++)
		{
			if (s_sg.max[series] < s_sg.data[series][i])
				s_sg.max[series] = s_sg.data[series][i];
		}
		s_sg.step[series] = s_sg.max[series] - s_sg.min[series];
		for (i = 0; i < s_sg.realDataNumber - 1; i++)
		{
			step = fabs(s_sg.data[series][i+1] - s_sg.data[series][i]);
			if (step > 0 && s_sg.step[series] > step)
				s_sg.step[series] = step;
		}
	}
}


void SG2DArray(void)
{
long i;
int indX, indY;

	if ((s_sg.oldSX != s_sg.seriesX[0]) || (s_sg.oldSY != s_sg.seriesY[0])
		|| (s_sg.oldSF != s_sg.seriesF[0]))
	{

		if ((s_sg.oldSX != s_sg.seriesX[0]) || (s_sg.oldSY != s_sg.seriesY[0]))
		{
			if (s_g.dim == 2)
			{
				s_g.limXMin = s_sg.min[s_sg.seriesX[0]-1];
				s_g.limXMax = s_sg.max[s_sg.seriesX[0]-1];
				s_g.limYMin = s_sg.min[s_sg.seriesY[0]-1];
				s_g.limYMax = s_sg.max[s_sg.seriesY[0]-1];
				s_g2d.dx = s_sg.step[s_sg.seriesX[0]-1];
				s_g2d.dy = s_sg.step[s_sg.seriesY[0]-1];
			}
			else if (s_g.dim == 3)
			{
				s_g3d.xMin = s_sg.min[s_sg.seriesX[0]-1];
				s_g3d.xMax = s_sg.max[s_sg.seriesX[0]-1];
				s_g3d.yMin = s_sg.min[s_sg.seriesY[0]-1];
				s_g3d.yMax = s_sg.max[s_sg.seriesY[0]-1];
				s_g3d.xIncrement = s_sg.step[s_sg.seriesX[0]-1];
				s_g3d.yIncrement = s_sg.step[s_sg.seriesY[0]-1];
			}
			if (s_sg.data2D)
				FreeArrayDouble2(s_sg.data2D, s_sg.data2DXNum, s_sg.data2DYNum);
			s_sg.data2DXNum = (s_sg.max[s_sg.seriesX[0]-1] - s_sg.min[s_sg.seriesX[0]-1])
								/ s_sg.step[s_sg.seriesX[0]-1] + 1.5;
			s_sg.data2DYNum = (s_sg.max[s_sg.seriesY[0]-1] - s_sg.min[s_sg.seriesY[0]-1])
								/ s_sg.step[s_sg.seriesY[0]-1] + 1.5;
			s_sg.data2D = MakeArrayDouble2(s_sg.data2DXNum, s_sg.data2DYNum);
		}

		s_sg.oldSX = s_sg.seriesX[0];
		s_sg.oldSY = s_sg.seriesY[0];
		s_sg.oldSF = s_sg.seriesF[0];

		for (i = 0; i < s_sg.realDataNumber; i++)
		{
			indX = (s_sg.data[s_sg.seriesX[0]-1][i] - s_sg.min[s_sg.seriesX[0]-1])
					/ s_sg.step[s_sg.seriesX[0]-1] + 0.5;
			indY = (s_sg.data[s_sg.seriesY[0]-1][i] - s_sg.min[s_sg.seriesY[0]-1])
					/ s_sg.step[s_sg.seriesY[0]-1] + 0.5;
			s_sg.data2D[indX][indY] = s_sg.data[s_sg.seriesF[0]-1][i];
		}
	}
}

double Graph2DSSFunc(double x, double y)
{
int indX, indY;

		indX = (x - s_sg.min[s_sg.seriesX[0]-1])
				/ s_sg.step[s_sg.seriesX[0]-1] + 0.5;
		indY = (y - s_sg.min[s_sg.seriesY[0]-1])
				/ s_sg.step[s_sg.seriesY[0]-1] + 0.5;
		return s_sg.data2D[indX][indY];
}

void Graph3DSS(void)
{
	s_g.spreadSheet = 1;
	SetGraph3DFunc(&Graph2DSSFunc);
	Graph3DF();
}

void SG3DInit(void)
{
int i;
char PDString[32];

	s_sg.maxUseSeriesNum = 1;
	SGInit();
	AddParameterP(&s_sg.realDataNumber, longType, "Data Number", s_g.dialogP);
	for (i = 0; i < s_sg.useSeriesNum; i++)
	{
		sprintf(PDString, "Plot %d X", i+1);
		AddParameterP(&s_sg.seriesX[i], intType, PDString, s_g.dialogP);
		sprintf(PDString, "Plot %d Y", i+1);
		AddParameterP(&s_sg.seriesY[i], intType, PDString, s_g.dialogP);
		sprintf(PDString, "Plot %d F", i+1);
		AddParameterP(&s_sg.seriesF[i], intType, PDString, s_g.dialogP);
	}
	s_g3d.xMin = s_sg.min[s_sg.seriesX[0]-1];
	s_g3d.xMax = s_sg.max[s_sg.seriesX[0]-1];
	s_g3d.yMin = s_sg.min[s_sg.seriesY[0]-1];
	s_g3d.yMax = s_sg.max[s_sg.seriesY[0]-1];
	s_g3d.xIncrement = s_sg.step[s_sg.seriesX[0]-1];
	s_g3d.yIncrement = s_sg.step[s_sg.seriesY[0]-1];
	s_g3d.useResolution = 0;
	s_g3d.equalIncrement = 0;
}
#endif /*pragma once*/
