/*
 *  HugeArray.c
 *  Version 1.8.1 / 001110
 *  by Satoshi Takahashi, Hidekazu Koike and Ryuji Tsukamoto
 */


#ifndef HUGEARRAY_C /*pragma once*/
#define HUGEARRAY_C

#include <s_Config.h>

#include <stdlib.h>

#include <HugeArray.h>
#include <s_Common.h>
#include <StartSCL.h>

#if S_UNIX /* as s_UNIX.h is not in parts-main */
#include <s_UNIX.h>
#endif

#include <s_undef.c>


int* MakeArrayInt1(long d1)
{
int *array;

	d1 = max(d1,1);
	if(!(array = (int *)calloc(d1,sizeof(int))))
		s_OOMError();
	return(array);
}

int** MakeArrayInt2(long d1, long d2)
{
int **array;
long i;

	d1 = max(d1,1);
	if(!(array = (int **)calloc(d1,sizeof(int *))))
		s_OOMError();
	for (i= 0; i < d1; i++)
	{
			array[i] = MakeArrayInt1(d2);
	}
	return(array);
}

int*** MakeArrayInt3(long d1, long d2, long d3)
{
int ***array;
long i;

	d1 = max(d1,1);
	if(!(array = (int ***)calloc(d1,sizeof(int **))))
		s_OOMError();
	for (i= 0; i < d1; i++)
	{
			array[i] = MakeArrayInt2(d2,d3);
	}
	return(array);
}

int**** MakeArrayInt4(long d1, long d2, long d3, long d4)
{
int ****array;
long i;

	d1 = max(d1,1);
	if(!(array = (int ****)calloc(d1,sizeof(int ***))))
		s_OOMError();
	for (i= 0; i < d1; i++)
	{
			array[i] = MakeArrayInt3(d2,d3,d4);
	}
	return(array);
}

double* MakeArrayDouble1(long d1)
{
double *array;

	d1 = max(d1,1);
	if(!(array = (double *)calloc(d1,sizeof(double))))
		s_OOMError();
	return(array);
}

double** MakeArrayDouble2(long d1, long d2)
{
double **array;
long i;

	d1 = max(d1,1);
	if(!(array = (double **)calloc(d1,sizeof(double *))))
		s_OOMError();
	for (i= 0; i < d1; i++)
	{
			array[i] = MakeArrayDouble1(d2);
	}
	return(array);
}

double*** MakeArrayDouble3(long d1, long d2, long d3)
{
double ***array;
long i;

	d1 = max(d1,1);
	if(!(array = (double ***)calloc(d1,sizeof(double **))))
		s_OOMError();
	for (i= 0; i < d1; i++)
	{
			array[i] = MakeArrayDouble2(d2,d3);
	}
	return(array);
}

double**** MakeArrayDouble4(long d1, long d2, long d3, long d4)
{
double ****array;
long i;

        d1 = max(d1,1);
        if(!(array = (double ****)calloc(d1,sizeof(double ***))))
                s_OOMError();
        for (i= 0; i < d1; i++)
        {
                        array[i] = MakeArrayDouble3(d2,d3,d4);
        }
        return(array);
}


double***** MakeArrayDouble5(long d1, long d2, long d3, long d4, long d5)
{
double *****array;
long i;

        d1 = max(d1,1);
        if(!(array = (double *****)calloc(d1,sizeof(double ****))))
                s_OOMError();
        for (i= 0; i < d1; i++)
        {
                        array[i] = MakeArrayDouble4(d2,d3,d4,d5);
        }
        return(array);
}

double****** MakeArrayDouble6(long d1, long d2, long d3, long d4, long d5,
long d6)
{
double ******array;
long i;

        d1 = max(d1,1);
        if(!(array = (double ******)calloc(d1,sizeof(double *****))))
                s_OOMError();
        for (i= 0; i < d1; i++)
        {
                        array[i] = MakeArrayDouble5(d2,d3,d4,d5,d6);
        }
        return(array);
}

long* MakeArrayLong1(long d1)
{
long *array;

	d1 = max(d1,1);
	if(!(array = (long *)calloc(d1,sizeof(long))))
		s_OOMError();
	return(array);
}

long** MakeArrayLong2(long d1, long d2)
{
long **array;
long i;

	d1 = max(d1,1);
	if(!(array = (long **)calloc(d1,sizeof(long *))))
		s_OOMError();
	for (i= 0; i < d1; i++)
	{
			array[i] = MakeArrayLong1(d2);
	}
	return(array);
}

long*** MakeArrayLong3(long d1, long d2, long d3)
{
long ***array;
long i;

	d1 = max(d1,1);
	if(!(array = (long ***)calloc(d1,sizeof(long **))))
		s_OOMError();
	for (i= 0; i < d1; i++)
	{
			array[i] = MakeArrayLong2(d2,d3);
	}
	return(array);
}

long**** MakeArrayLong4(long d1, long d2, long d3, long d4)
{
long ****array;
long i;

	d1 = max(d1,1);
	if(!(array = (long ****)calloc(d1,sizeof(long ***))))
		s_OOMError();
	for (i= 0; i < d1; i++)
	{
			array[i] = MakeArrayLong3(d2,d3,d4);
	}
	return(array);
}


void FreeArrayInt2(int** array,long d1, long d2)
{
long i;

	for (i= 0; i < d1; i++)
	{
			free(array[i]);
	}
	free(array);
}

void FreeArrayDouble2(double** array,long d1, long d2)
{
long i;

	for (i= 0; i < d1; i++)
	{
			free(array[i]);
	}
	free(array);
}

#if !S_UNIX
#include <s_define.c>
#endif

#endif /* pragma once */

