/*
 *  LinearSolve.c
 *  Version 1.9.1 / 010224
 *  by Satoshi Takahashi and Jiro Kikuchi
 */


#ifndef LINEAR_SOLVE_C /*prgma once*/
#define LINEAR_SOLVE_C

#include <math.h>
#include <HugeArray.h>
#include <StartSCL.h>
#include <LinearSolve.h>

double s_ls_eps = 0.00000000001;

int LinearSolve(int N, double **matIn, double *ans, double *vec)
{
int i, j, k;
double div = 0.0, coef = 0.0;
double **mat;

	mat = MakeArrayDouble2(N, N);
	for(i = 0; i < N; i++)
		ans[i] = vec[i];

	for(i = 0; i < N; i++)
		for (j = 0; j < N; j++)
			mat[i][j] = matIn[i][j];
	
	for(i = 0; i < N; i++)
	{
		if (fabs(mat[i][i]) < s_ls_eps)
		{
			for (k = i+1; k < N; k++)
			{
				if (fabs(mat[k][i]) > s_ls_eps)
				{
					LinearExchange(N, i, k, mat, ans);
					goto no_singular;
				}
			}
			s_printf("Error in LinearSolve: Singular Matrix.");
			return -1;
		no_singular:
			;
		}
		
		div = mat[i][i];
		
		for(j = 0; j < N; j++)
		{
			mat[i][j]/=div;
		}
		ans[i]/=div;
		
		for (k = 0; k < N; k++)
		{
			if (k == i)
				continue;
			
			coef = mat[k][i];
			for(j = i; j < N; j++)
			{
				mat[k][j] -= coef*mat[i][j];
			}
			ans[k] -= coef*ans[i];
		}
	}
	FreeArrayDouble2(mat,N,N);
	return 0;
}

void LinearExchange(int N, int i, int k, double **mat, double *vec)
{
int j;
double tmp;

	for(j = i; j < N; j++)
	{
		tmp = mat[i][j];
		mat[i][j] = mat[k][j];
		mat[k][j] = tmp;
	}

	tmp = vec[i];
	vec[i] = vec[k];
	vec[k] = tmp;
}

void InverseMatrix(int N, double **mat, double **ans)
{
int i,j;
    double *anscolumn,*vec;
    
    anscolumn=MakeArrayDouble1(N);
    vec=MakeArrayDouble1(N);
    
    for(i=0;i<N;i++)
    {
	    for(j=0;j<N;j++)
        {
	        vec[j]=0;
        }
	    vec[i]=1;
	    LinearSolve(N,mat,anscolumn,vec);
	    for(j=0;j<N;j++)
	    {
		    ans[j][i]=anscolumn[j];
	    }
    }
    free(anscolumn);
    free(vec);
}

void SetLinearSolvePrecision(double prec)
{
	s_ls_eps = prec;
}
#include <s_define.c>
#endif /*prgma once*/