/*
 *  RungeKutta.c
 *  Version 2.0.2 / 020718
 *  by Satoshi Takahashi, Yumiko Takahashi & Kayo Yasufuku
 */

#ifndef RUNGEKUTTA_C /*pragma once*/
#define RUNGEKUTTA_C

#include <s_Config.h>


#ifdef SYMANTEC_C
#include <PPC Macheaders>
#endif


#include <stdarg.h>
#include <stdlib.h>

#include <RungeKutta.h>
#include <HugeArray.h>
#include <s_Common.h>
#include <StartSCL.h>

/*** RungeKutta1 ***/

double (*s_r_f)(double);
double s_r_dt = 0.01;

void SetDt(double dt)
{
        s_r_dt = dt;
}


void SetRungeKutta1(double (*pf)(double),double dt)
{
        s_r_f = pf;
        s_r_dt = dt;
}

void RungeKutta1(double *px)
{
double  x,vx1, vx2, vx3, vx4, xx;

        x = *px;
        vx1 = s_r_dt * (*s_r_f)(x);
        xx = x + vx1 / 2;
        vx2 = s_r_dt * (*s_r_f)(xx);
        xx = x + vx2 / 2;
        vx3 = s_r_dt * (*s_r_f)(xx);
        xx = x + vx3;
        vx4 = s_r_dt * (*s_r_f)(xx);
        *px = x + (vx1 + vx2 * 2 + vx3 * 2 + vx4) / 6;
}


/*** RungeKutta1T ***/

double (*s_r_ft)(double,double);

void SetRungeKutta1T(double (*pf)(double,double),double dt)
{
        s_r_ft = pf;
        s_r_dt = dt;
}

void RungeKutta1T(double *px,double *pt)
{
double  x,vx1, vx2, vx3, vx4, xx,
                t,tt;

        x = *px;
        t = *pt;
        vx1 = s_r_dt * (*s_r_ft)(x,t);
        xx = x + vx1 / 2;
        tt = t + s_r_dt/2;
        vx2 = s_r_dt * (*s_r_ft)(xx,tt);
        xx = x + vx2 / 2;
        tt = t + s_r_dt/2;
        vx3 = s_r_dt * (*s_r_ft)(xx,tt);
        xx = x + vx3;
        tt = t + s_r_dt;
        vx4 = s_r_dt * (*s_r_ft)(xx,tt);
        *px = x + (vx1 + vx2 * 2 + vx3 * 2 + vx4) / 6;
        *pt += s_r_dt;
}


/*** RungeKutta2 ***/

double  (*s_r_fx2)(double x,double y),
                (*s_r_fy2)(double x,double y);

void SetRungeKutta2(    double (*pfx)(double x,double y),
                        double (*pfy)(double x,double y),
                        double dt)
{
        s_r_fx2 = pfx;
        s_r_fy2 = pfy;
        s_r_dt = dt;
}

void RungeKutta2(double *px,double *py)
{
double  x,vx1, vx2, vx3, vx4, xx,
                y,vy1, vy2, vy3, vy4, yy;

        x = *px;
        y = *py;
        vx1 = s_r_dt * (*s_r_fx2)(x,y);
        vy1 = s_r_dt * (*s_r_fy2)(x,y);
        xx = x + vx1 / 2;
        yy = y + vy1 / 2;
        vx2 = s_r_dt * (*s_r_fx2)(xx,yy);
        vy2 = s_r_dt * (*s_r_fy2)(xx,yy);
        xx = x + vx2 / 2;
        yy = y + vy2 / 2;
        vx3 = s_r_dt * (*s_r_fx2)(xx,yy);
        vy3 = s_r_dt * (*s_r_fy2)(xx,yy);
        xx = x + vx3;
        yy = y + vy3;
        vx4 = s_r_dt * (*s_r_fx2)(xx,yy);
        vy4 = s_r_dt * (*s_r_fy2)(xx,yy);
        *px = x + (vx1 + vx2 * 2 + vx3 * 2 + vx4) / 6;
        *py = y + (vy1 + vy2 * 2 + vy3 * 2 + vy4) / 6;
}


/*** RungeKutta2T ***/

double  (*s_r_fx2t)(double x,double y,double t),
                (*s_r_fy2t)(double x,double y,double t),
                (*s_r_fz2t)(double x,double y,double t);

void SetRungeKutta2T(   double (*pfx)(double x,double y,double t),
                        double (*pfy)(double x,double y,double t),
                        double dt)
{
        s_r_fx2t = pfx;
        s_r_fy2t = pfy;
        s_r_dt = dt;
}

void RungeKutta2T(double *px,double *py,
double *pt)
{
double  x,vx1, vx2, vx3, vx4, xx,
                y,vy1, vy2, vy3, vy4, yy,
                t,tt;

        x = *px;
        y = *py;
        t = *pt;
        vx1 = s_r_dt * (*s_r_fx2t)(x,y,t);
        vy1 = s_r_dt * (*s_r_fy2t)(x,y,t);
        xx = x + vx1 / 2;
        yy = y + vy1 / 2;
        tt = t + s_r_dt/2;
        vx2 = s_r_dt * (*s_r_fx2t)(xx,yy,tt);
        vy2 = s_r_dt * (*s_r_fy2t)(xx,yy,tt);
        xx = x + vx2 / 2;
        yy = y + vy2 / 2;
        tt = t + s_r_dt/2;
        vx3 = s_r_dt * (*s_r_fx2t)(xx,yy,tt);
        vy3 = s_r_dt * (*s_r_fy2t)(xx,yy,tt);
        xx = x + vx3;
        yy = y + vy3;
        tt = t + s_r_dt;
        vx4 = s_r_dt * (*s_r_fx2t)(xx,yy,tt);
        vy4 = s_r_dt * (*s_r_fy2t)(xx,yy,tt);
        *px = x + (vx1 + vx2 * 2 + vx3 * 2 + vx4) / 6;
        *py = y + (vy1 + vy2 * 2 + vy3 * 2 + vy4) / 6;
        *pt += s_r_dt;
}


/*** RungeKutta3 ***/

double  (*s_r_fx3)(double x,double y,double z),
                (*s_r_fy3)(double x,double y,double z),
                (*s_r_fz3)(double x,double y,double z);

void SetRungeKutta3(    double (*pfx)(double x,double y,double z),
                        double (*pfy)(double x,double y,double z),
                        double (*pfz)(double x,double y,double z),
                        double dt)
{
        s_r_fx3 = pfx;
        s_r_fy3 = pfy;
        s_r_fz3 = pfz;
        s_r_dt = dt;
}

void RungeKutta3(double *px,double *py,double *pz)
{
double  x,vx1, vx2, vx3, vx4, xx,
                y,vy1, vy2, vy3, vy4, yy,
                z,vz1, vz2, vz3, vz4, zz;

        x = *px;
        y = *py;
        z = *pz;
        vx1 = s_r_dt * (*s_r_fx3)(x,y,z);
        vy1 = s_r_dt * (*s_r_fy3)(x,y,z);
        vz1 = s_r_dt * (*s_r_fz3)(x,y,z);
        xx = x + vx1 / 2;
        yy = y + vy1 / 2;
        zz = z + vz1 / 2;
        vx2 = s_r_dt * (*s_r_fx3)(xx,yy,zz);
        vy2 = s_r_dt * (*s_r_fy3)(xx,yy,zz);
        vz2 = s_r_dt * (*s_r_fz3)(xx,yy,zz);
        xx = x + vx2 / 2;
        yy = y + vy2 / 2;
        zz = z + vz2 / 2;
        vx3 = s_r_dt * (*s_r_fx3)(xx,yy,zz);
        vy3 = s_r_dt * (*s_r_fy3)(xx,yy,zz);
        vz3 = s_r_dt * (*s_r_fz3)(xx,yy,zz);
        xx = x + vx3;
        yy = y + vy3;
        zz = z + vz3;
        vx4 = s_r_dt * (*s_r_fx3)(xx,yy,zz);
        vy4 = s_r_dt * (*s_r_fy3)(xx,yy,zz);
        vz4 = s_r_dt * (*s_r_fz3)(xx,yy,zz);
        *px = x + (vx1 + vx2 * 2 + vx3 * 2 + vx4) / 6;
        *py = y + (vy1 + vy2 * 2 + vy3 * 2 + vy4) / 6;
        *pz = z + (vz1 + vz2 * 2 + vz3 * 2 + vz4) / 6;
}


/*** RungeKutta3T ***/

double  (*s_r_fx3t)(double x,double y,double z,double t),
                (*s_r_fy3t)(double x,double y,double z,double t),
                (*s_r_fz3t)(double x,double y,double z,double t);

void SetRungeKutta3T(   double (*pfx)(double x,double y,double z,double t),
                        double (*pfy)(double x,double y,double z,double t),
                        double (*pfz)(double x,double y,double z,double t),
                        double dt)
{
        s_r_fx3t = pfx;
        s_r_fy3t = pfy;
        s_r_fz3t = pfz;
        s_r_dt = dt;
}

void RungeKutta3T(double *px,double *py,double *pz,double *pt)
{
double  x,vx1, vx2, vx3, vx4, xx,
                y,vy1, vy2, vy3, vy4, yy,
                z,vz1, vz2, vz3, vz4, zz,
                t,tt;

        x = *px;
        y = *py;
        z = *pz;
        t = *pt;
        vx1 = s_r_dt * (*s_r_fx3t)(x,y,z,t);
        vy1 = s_r_dt * (*s_r_fy3t)(x,y,z,t);
        vz1 = s_r_dt * (*s_r_fz3t)(x,y,z,t);
        xx = x + vx1 / 2;
        yy = y + vy1 / 2;
        zz = z + vz1 / 2;
        tt = t + s_r_dt/2;
        vx2 = s_r_dt * (*s_r_fx3t)(xx,yy,zz,tt);
        vy2 = s_r_dt * (*s_r_fy3t)(xx,yy,zz,tt);
        vz2 = s_r_dt * (*s_r_fz3t)(xx,yy,zz,tt);
        xx = x + vx2 / 2;
        yy = y + vy2 / 2;
        zz = z + vz2 / 2;
        tt = t + s_r_dt/2;
        vx3 = s_r_dt * (*s_r_fx3t)(xx,yy,zz,tt);
        vy3 = s_r_dt * (*s_r_fy3t)(xx,yy,zz,tt);
        vz3 = s_r_dt * (*s_r_fz3t)(xx,yy,zz,tt);
        xx = x + vx3;
        yy = y + vy3;
        zz = z + vz3;
        tt = t + s_r_dt;
        vx4 = s_r_dt * (*s_r_fx3t)(xx,yy,zz,tt);
        vy4 = s_r_dt * (*s_r_fy3t)(xx,yy,zz,tt);
        vz4 = s_r_dt * (*s_r_fz3t)(xx,yy,zz,tt);
        *px = x + (vx1 + vx2 * 2 + vx3 * 2 + vx4) / 6;
        *py = y + (vy1 + vy2 * 2 + vy3 * 2 + vy4) / 6;
        *pz = z + (vz1 + vz2 * 2 + vz3 * 2 + vz4) / 6;
        *pt += s_r_dt;
}
/*** RungeKutta4 ***/

double  (*s_r_fx4)(double x,double y,double z,double u),
                (*s_r_fy4)(double x,double y,double z,double u),
                (*s_r_fz4)(double x,double y,double z,double u),
                (*s_r_fu4)(double x,double y,double z,double u);

void SetRungeKutta4(	double (*pfx)(double x,double y,double z,double u),
						double (*pfy)(double x,double y,double z,double u),
                        double (*pfz)(double x,double y,double z,double u),
                        double (*pfu)(double x,double y,double z,double u),
                        double dt)
{
        s_r_fx4 = pfx;
        s_r_fy4 = pfy;
        s_r_fz4 = pfz;
        s_r_fu4 = pfu;
        s_r_dt = dt;
}

void RungeKutta4(double *px,double *py,double *pz,double *pu)
{
double  x,vx1, vx2, vx3, vx4, xx,
                y,vy1, vy2, vy3, vy4, yy,
                z,vz1, vz2, vz3, vz4, zz,
                u,vu1, vu2, vu3, vu4, uu;

        x = *px;
        y = *py;
        z = *pz;
        u = *pu;
        vx1 = s_r_dt * (*s_r_fx4)(x,y,z,u);
        vy1 = s_r_dt * (*s_r_fy4)(x,y,z,u);
        vz1 = s_r_dt * (*s_r_fz4)(x,y,z,u);
        vu1 = s_r_dt * (*s_r_fu4)(x,y,z,u);
        xx = x + vx1 / 2;
        yy = y + vy1 / 2;
        zz = z + vz1 / 2;
        uu = u + vu1 / 2;
        vx2 = s_r_dt * (*s_r_fx4)(xx,yy,zz,uu);
        vy2 = s_r_dt * (*s_r_fy4)(xx,yy,zz,uu);
        vz2 = s_r_dt * (*s_r_fz4)(xx,yy,zz,uu);
        vu2 = s_r_dt * (*s_r_fu4)(xx,yy,zz,uu);
        xx = x + vx2 / 2;
        yy = y + vy2 / 2;
        zz = z + vz2 / 2;
        uu = u + vu2 / 2;
        vx3 = s_r_dt * (*s_r_fx4)(xx,yy,zz,uu);
        vy3 = s_r_dt * (*s_r_fy4)(xx,yy,zz,uu);
        vz3 = s_r_dt * (*s_r_fz4)(xx,yy,zz,uu);
        vu3 = s_r_dt * (*s_r_fu4)(xx,yy,zz,uu);
        xx = x + vx3;
        yy = y + vy3;
        zz = z + vz3;
        uu = u + vu3;
        vx4 = s_r_dt * (*s_r_fx4)(xx,yy,zz,uu);
        vy4 = s_r_dt * (*s_r_fy4)(xx,yy,zz,uu);
        vz4 = s_r_dt * (*s_r_fz4)(xx,yy,zz,uu);
        vu4 = s_r_dt * (*s_r_fu4)(xx,yy,zz,uu);
        *px = x + (vx1 + vx2 * 2 + vx3 * 2 + vx4) / 6;
        *py = y + (vy1 + vy2 * 2 + vy3 * 2 + vy4) / 6;
        *pz = z + (vz1 + vz2 * 2 + vz3 * 2 + vz4) / 6;
        *pu = u + (vu1 + vu2 * 2 + vu3 * 2 + vu4) / 6;
}

/*** RungeKutta4T ***/

double  (*s_r_fx4t)(double x,double y,double z,double u,double t),
                (*s_r_fy4t)(double x,double y,double z,double u,double t),
                (*s_r_fz4t)(double x,double y,double z,double u,double t),
                (*s_r_fu4t)(double x,double y,double z,double u,double t);

void SetRungeKutta4T(	double (*pfx)(double x,double y,double z,double u,double t),
						double (*pfy)(double x,double y,double z,double u,double t),
						double (*pfz)(double x,double y,double z,double u,double t),
						double (*pfu)(double x,double y,double z,double u,double t),
						double dt)
{
        s_r_fx4t = pfx;
        s_r_fy4t = pfy;
        s_r_fz4t = pfz;
        s_r_fu4t = pfu;
        s_r_dt = dt;
}

void RungeKutta4T(double *px,double *py,double *pz,double *pu,double *pt)
{
double  x,vx1, vx2, vx3, vx4, xx,
                y,vy1, vy2, vy3, vy4, yy,
                z,vz1, vz2, vz3, vz4, zz,
                u,vu1, vu2, vu3, vu4, uu,
                t,tt;

        x = *px;
        y = *py;
        z = *pz;
        u = *pu;
        t = *pt;
        vx1 = s_r_dt * (*s_r_fx4t)(x,y,z,u,t);
        vy1 = s_r_dt * (*s_r_fy4t)(x,y,z,u,t);
        vz1 = s_r_dt * (*s_r_fz4t)(x,y,z,u,t);
        vu1 = s_r_dt * (*s_r_fu4t)(x,y,z,u,t);
        xx = x + vx1 / 2;
        yy = y + vy1 / 2;
        zz = z + vz1 / 2;
        uu = u + vu1 / 2;
        tt = t + s_r_dt/2;
        vx2 = s_r_dt * (*s_r_fx4t)(xx,yy,zz,uu,tt);
        vy2 = s_r_dt * (*s_r_fy4t)(xx,yy,zz,uu,tt);
        vz2 = s_r_dt * (*s_r_fz4t)(xx,yy,zz,uu,tt);
        vu2 = s_r_dt * (*s_r_fu4t)(xx,yy,zz,uu,tt);
        xx = x + vx2 / 2;
        yy = y + vy2 / 2;
        zz = z + vz2 / 2;
        uu = u + vu2 / 2;
        tt = t + s_r_dt/2;
        vx3 = s_r_dt * (*s_r_fx4t)(xx,yy,zz,uu,tt);
        vy3 = s_r_dt * (*s_r_fy4t)(xx,yy,zz,uu,tt);
        vz3 = s_r_dt * (*s_r_fz4t)(xx,yy,zz,uu,tt);
        vu3 = s_r_dt * (*s_r_fu4t)(xx,yy,zz,uu,tt);
        xx = x + vx3;
        yy = y + vy3;
        zz = z + vz3;
        uu = u + vu3;
        tt = t + s_r_dt;
        vx4 = s_r_dt * (*s_r_fx4t)(xx,yy,zz,uu,tt);
        vy4 = s_r_dt * (*s_r_fy4t)(xx,yy,zz,uu,tt);
        vz4 = s_r_dt * (*s_r_fz4t)(xx,yy,zz,uu,tt);
        vu4 = s_r_dt * (*s_r_fu4t)(xx,yy,zz,uu,tt);
        *px = x + (vx1 + vx2 * 2 + vx3 * 2 + vx4) / 6;
        *py = y + (vy1 + vy2 * 2 + vy3 * 2 + vy4) / 6;
        *pz = z + (vz1 + vz2 * 2 + vz3 * 2 + vz4) / 6;
        *pu = u + (vu1 + vu2 * 2 + vu3 * 2 + vu4) / 6;
        *pt +=s_r_dt;
}

/*** RungeKutta6 ***/

double  (*s_r_fx6)(double x,double y,double z,double s,double t,double u),
                (*s_r_fy6)(double x,double y,double z,double s,double t,double u),
                (*s_r_fz6)(double x,double y,double z,double s,double t,double u),
                (*s_r_fs6)(double x,double y,double z,double s,double t,double u),
                (*s_r_ft6)(double x,double y,double z,double s,double t,double u),
                (*s_r_fu6)(double x,double y,double z,double s,double t,double u);

void SetRungeKutta6(	double (*pfx)(double x,double y,double z,double s,double t,double u),
						double (*pfy)(double x,double y,double z,double s,double t,double u),
                        double (*pfz)(double x,double y,double z,double s,double t,double u),
                        double (*pfs)(double x,double y,double z,double s,double t,double u),
                        double (*pft)(double x,double y,double z,double s,double t,double u),
                        double (*pfu)(double x,double y,double z,double s,double t,double u),
                        double dt)
{
        s_r_fx6 = pfx;
        s_r_fy6 = pfy;
        s_r_fz6 = pfz;
        s_r_fs6 = pfs;
        s_r_ft6 = pft;
        s_r_fu6 = pfu;
        s_r_dt = dt;
}

void RungeKutta6(double *px,double *py,double *pz,double *ps,double *pt,double *pu)
{
double  x,vx1, vx2, vx3, vx4, xx,
                y,vy1, vy2, vy3, vy4, yy,
                z,vz1, vz2, vz3, vz4, zz,
                s,vs1, vs2, vs3, vs4, ss,
                t,vt1, vt2, vt3, vt4, tt,
                u,vu1, vu2, vu3, vu4, uu;

        x = *px;
        y = *py;
        z = *pz;  
        s = *ps;  
        t = *pt;
        u = *pu;
        vx1 = s_r_dt * (*s_r_fx6)(x,y,z,s,t,u);
        vy1 = s_r_dt * (*s_r_fy6)(x,y,z,s,t,u);
        vz1 = s_r_dt * (*s_r_fz6)(x,y,z,s,t,u);
        vs1 = s_r_dt * (*s_r_fs6)(x,y,z,s,t,u);
        vt1 = s_r_dt * (*s_r_ft6)(x,y,z,s,t,u);
        vu1 = s_r_dt * (*s_r_fu6)(x,y,z,s,t,u);

        xx = x + vx1 / 2;
        yy = y + vy1 / 2;
        zz = z + vz1 / 2;
        ss = s + vs1 / 2;
        tt = t + vt1 / 2;
        uu = u + vu1 / 2;
        vx2 = s_r_dt * (*s_r_fx6)(xx,yy,zz,ss,tt,uu);
        vy2 = s_r_dt * (*s_r_fy6)(xx,yy,zz,ss,tt,uu);
        vz2 = s_r_dt * (*s_r_fz6)(xx,yy,zz,ss,tt,uu);
        vs2 = s_r_dt * (*s_r_fs6)(xx,yy,zz,ss,tt,uu);
        vt2 = s_r_dt * (*s_r_ft6)(xx,yy,zz,ss,tt,uu);
        vu2 = s_r_dt * (*s_r_fu6)(xx,yy,zz,ss,tt,uu);

        xx = x + vx2 / 2;
        yy = y + vy2 / 2;
        zz = z + vz2 / 2;
        ss = s + vs2 / 2;
        tt = t + vt2 / 2;
        uu = u + vu2 / 2;
        vx3 = s_r_dt * (*s_r_fx6)(xx,yy,zz,ss,tt,uu);
        vy3 = s_r_dt * (*s_r_fy6)(xx,yy,zz,ss,tt,uu);
        vz3 = s_r_dt * (*s_r_fz6)(xx,yy,zz,ss,tt,uu);
        vs3 = s_r_dt * (*s_r_fs6)(xx,yy,zz,ss,tt,uu);
        vt3 = s_r_dt * (*s_r_ft6)(xx,yy,zz,ss,tt,uu);
        vu3 = s_r_dt * (*s_r_fu6)(xx,yy,zz,ss,tt,uu);

        xx = x + vx3;
        yy = y + vy3;
        zz = z + vz3;
        ss = s + vs3;
        tt = t + vt3;
        uu = u + vu3;
        vx4 = s_r_dt * (*s_r_fx6)(xx,yy,zz,ss,tt,uu);
        vy4 = s_r_dt * (*s_r_fy6)(xx,yy,zz,ss,tt,uu);
        vz4 = s_r_dt * (*s_r_fz6)(xx,yy,zz,ss,tt,uu);
        vs4 = s_r_dt * (*s_r_fs6)(xx,yy,zz,ss,tt,uu);
        vt4 = s_r_dt * (*s_r_ft6)(xx,yy,zz,ss,tt,uu);
        vu4 = s_r_dt * (*s_r_fu6)(xx,yy,zz,ss,tt,uu);
        
        *px = x + (vx1 + vx2 * 2 + vx3 * 2 + vx4) / 6;
        *py = y + (vy1 + vy2 * 2 + vy3 * 2 + vy4) / 6;
        *pz = z + (vz1 + vz2 * 2 + vz3 * 2 + vz4) / 6; 
        *ps = s + (vs1 + vs2 * 2 + vs3 * 2 + vs4) / 6;
        *pt = t + (vt1 + vt2 * 2 + vt3 * 2 + vt4) / 6;
        *pu = u + (vu1 + vu2 * 2 + vu3 * 2 + vu4) / 6;

}

/*** RungeKuttaV ***/

typedef double(*s_vfunc)(double*);
s_vfunc *s_r_fv;
int s_r_n;
double  *s_r_vx1, *s_r_vx2, *s_r_vx3, *s_r_vx4, *s_r_xx;

void SetRungeKuttaV(int n,...)
{
int i;
va_list ap;

        s_r_n = n;
        if (!(s_r_fv = (double (**)(double*))calloc(n,sizeof(double (**)(double*)))))
                s_OOMError();
        va_start(ap,n);
        for (i=0;i<n;i++)
        {
                s_r_fv[i] = va_arg(ap,s_vfunc);
        }
        s_r_dt = va_arg(ap,double);
        va_end(ap);
        s_r_vx1 = MakeArrayDouble1(s_r_n);
        s_r_vx2 = MakeArrayDouble1(s_r_n);
        s_r_vx3 = MakeArrayDouble1(s_r_n);
        s_r_vx4 = MakeArrayDouble1(s_r_n);
        s_r_xx = MakeArrayDouble1(s_r_n);

}

void RungeKuttaV(double *px)
{
int i;

        for(i=0;i<s_r_n;i++)
                s_r_vx1[i] = s_r_dt * (*s_r_fv[i])(px);
        for(i=0;i<s_r_n;i++)
                s_r_xx[i] = px[i] + s_r_vx1[i] / 2;
        for(i=0;i<s_r_n;i++)
                s_r_vx2[i] = s_r_dt * (*s_r_fv[i])(s_r_xx);
        for(i=0;i<s_r_n;i++)
                s_r_xx[i] = px[i] + s_r_vx2[i] / 2;
        for(i=0;i<s_r_n;i++)
                s_r_vx3[i] = s_r_dt * (*s_r_fv[i])(s_r_xx);
        for(i=0;i<s_r_n;i++)
                s_r_xx[i] = px[i] + s_r_vx3[i];
        for(i=0;i<s_r_n;i++)
                s_r_vx4[i] = s_r_dt * (*s_r_fv[i])(s_r_xx);
        for(i=0;i<s_r_n;i++)
                px[i] = px[i] + (s_r_vx1[i] + s_r_vx2[i] * 2 + s_r_vx3[i] * 2 + s_r_vx4[i]) / 6;
}

/*** RungeKutta1D ***/

double *s_r_delayArrayX;
int s_r_delayIntX;
int s_r_tx =0;
int s_r_delayMax = 0;
int s_r_t = 0;

void SetRungeKutta1D(double (*pf)(double,double),double dt,double delayX)
{
        s_r_ft = pf;
        s_r_dt = dt;
        s_r_delayIntX = delayX/dt;
	s_r_delayMax = s_r_delayIntX; 

	s_r_delayMax++;

    s_r_delayArrayX = MakeArrayDouble1(s_r_delayMax);

	s_r_t = 0;
	s_r_tx = s_r_delayMax - s_r_delayIntX;
}

void RungeKutta1D(double *px)
{
double  x,vx1, vx2, vx3, vx4, xx,
                d,dd,ddxx;
int tx0,tx1,tx2,tx3;
double tmp;

        s_r_t = (s_r_t+1) % s_r_delayMax;
        s_r_tx = (s_r_tx+1) % s_r_delayMax;

        x = *px;
        d = s_r_delayArrayX[s_r_tx];
        vx1 = s_r_dt * (*s_r_ft)(x,d);
        xx = x + vx1 / 2;
        tx0 = (s_r_tx + s_r_delayMax - 1) % s_r_delayMax;
        tx1 = s_r_tx;
        tx2 = (s_r_tx + 1) % s_r_delayMax;
        tx3 = (s_r_tx + 2) % s_r_delayMax;

	tmp = s_r_delayArrayX[tx1]+s_r_delayArrayX[tx2];
	ddxx = (9*tmp - (s_r_delayArrayX[tx0]+s_r_delayArrayX[tx3]))/16;

        vx2 = s_r_dt * (*s_r_ft)(xx,ddxx);
        xx = x + vx2 / 2;
        vx3 = s_r_dt * (*s_r_ft)(xx,ddxx);
        xx = x + vx3;
        dd = s_r_delayArrayX[tx2];
        vx4 = s_r_dt * (*s_r_ft)(xx,dd);
        *px = x + (vx1 + vx2 * 2 + vx3 * 2 + vx4) / 6;
        s_r_delayArrayX[s_r_t] = *px;
}

void SetRungeKutta1DDelay(double x)
{
int i;

	for (i=0; i< s_r_delayMax;i++)
		s_r_delayArrayX[i] = x;
}

void SetRungeKutta1DOneDelay(double x)
{
#if 1
	SetRungeKutta1DDelay(x);
#else
        s_r_delayArrayX[s_r_tx] = x;
#endif
}


/*** RungeKutta2D ***/

double *s_r_delayArrayY;
int s_r_delayIntY;
int s_r_ty = 0;

double (*s_r_fx2d)(double,double,double,double);
double (*s_r_fy2d)(double,double,double,double);

void SetRungeKutta2D(double (*pfx)(double,double,double,double),
                     double (*pfy)(double,double,double,double),
                     double dt,double delayX,double delayY)
{
        s_r_fx2d = pfx;
        s_r_fy2d = pfy;
        s_r_dt = dt;

        s_r_delayIntX = delayX/dt;
	s_r_delayMax = s_r_delayIntX; 

        s_r_delayIntY = delayY/dt;
	if (s_r_delayMax < s_r_delayIntY)
		s_r_delayMax = s_r_delayIntY; 

	s_r_delayMax++;

	s_r_delayArrayX = MakeArrayDouble1(s_r_delayMax);
	s_r_delayArrayY = MakeArrayDouble1(s_r_delayMax);

	s_r_t = 0;
	s_r_tx = s_r_delayMax - s_r_delayIntX;
	s_r_ty = s_r_delayMax - s_r_delayIntY;
}


void RungeKutta2D(double *px,double *py)
{
double  x,vx1, vx2, vx3, vx4, xx,
                y,vy1, vy2, vy3, vy4, yy,
                dx,ddx,ddxx,ddxy,dy,ddy,ddyx,ddyy;
int tx0,tx1,tx2,tx3,
        ty0,ty1,ty2,ty3;
double tmp;

	s_r_t = (s_r_t+1) % s_r_delayMax;
	s_r_tx = (s_r_tx+1) % s_r_delayMax;
	s_r_ty = (s_r_ty+1) % s_r_delayMax;

	x = *px;
	y = *py;
	dx = s_r_delayArrayX[s_r_tx];
	dy = s_r_delayArrayY[s_r_tx];
	vx1 = s_r_dt * (*s_r_fx2d)(x,y,dx,dy);
	dx = s_r_delayArrayX[s_r_ty];
	dy = s_r_delayArrayY[s_r_ty];
	vy1 = s_r_dt * (*s_r_fy2d)(x,y,dx,dy);
	xx = x + vx1 / 2;
	yy = y + vy1 / 2;
	tx0 = (s_r_tx + s_r_delayMax - 1) % s_r_delayMax;
	ty0 = (s_r_ty + s_r_delayMax - 1) % s_r_delayMax;
	tx1 = s_r_tx;
	ty1 = s_r_ty;
	tx2 = (s_r_tx + 1) % s_r_delayMax;
	ty2 = (s_r_ty + 1) % s_r_delayMax;
	tx3 = (s_r_tx + 2) % s_r_delayMax;
	ty3 = (s_r_ty + 2) % s_r_delayMax;

	tmp = s_r_delayArrayX[tx1]+s_r_delayArrayX[tx2];
	ddxx = (9*tmp - (s_r_delayArrayX[tx0]+s_r_delayArrayX[tx3]))/16;
	tmp = s_r_delayArrayY[tx1]+s_r_delayArrayY[tx2];
	ddyx = (9*tmp - (s_r_delayArrayY[tx0]+s_r_delayArrayY[tx3]))/16;
	tmp = s_r_delayArrayX[ty1]+s_r_delayArrayX[ty2];
	ddxy = (9*tmp - (s_r_delayArrayX[ty0]+s_r_delayArrayX[ty3]))/16;
	tmp = s_r_delayArrayY[ty1]+s_r_delayArrayY[ty2];
	ddyy = (9*tmp - (s_r_delayArrayY[ty0]+s_r_delayArrayY[ty3]))/16;

	vx2 = s_r_dt * (*s_r_fx2d)(xx,yy,ddxx,ddyx);
	vy2 = s_r_dt * (*s_r_fy2d)(xx,yy,ddxy,ddyy);
	xx = x + vx2 / 2;
	yy = y + vy2 / 2;
	vx3 = s_r_dt * (*s_r_fx2d)(xx,yy,ddxx,ddyx);
	vy3 = s_r_dt * (*s_r_fy2d)(xx,yy,ddxy,ddyy);
	xx = x + vx3;
	yy = y + vy3;
	ddx = s_r_delayArrayX[tx2];
	ddy = s_r_delayArrayY[tx2];
	vx4 = s_r_dt * (*s_r_fx2d)(xx,yy,ddx,ddy);
	ddx = s_r_delayArrayX[ty2];
	ddy = s_r_delayArrayY[ty2];
	vy4 = s_r_dt * (*s_r_fy2d)(xx,yy,ddx,ddy);
	*px = x + (vx1 + vx2 * 2 + vx3 * 2 + vx4) / 6;
	*py = y + (vy1 + vy2 * 2 + vy3 * 2 + vy4) / 6;
	s_r_delayArrayX[s_r_t] = *px;
	s_r_delayArrayY[s_r_t] = *py;
}

void SetRungeKutta2DDelay(double x, double y)
{
int i;

	for (i=0; i< s_r_delayMax;i++)
		s_r_delayArrayX[i] = x;
	for (i=0; i< s_r_delayMax;i++)
		s_r_delayArrayY[i] = y;
}

void SetRungeKutta2DOneDelay(double x, double y)
{
#if 1
	SetRungeKutta2DDelay(x, y);
#else
        s_r_delayArrayX[s_r_tx] = x;
        s_r_delayArrayY[s_r_ty] = y;
#endif
}

/*** RungeKutta3D ***/

double *s_r_delayArrayZ;
int s_r_delayIntZ;
int s_r_tz =0;

double (*s_r_fx3d)(double,double,double,double,double,double);
double (*s_r_fy3d)(double,double,double,double,double,double);
double (*s_r_fz3d)(double,double,double,double,double,double);

void SetRungeKutta3D(double (*pfx)(double,double,double,double,double,double),
                     double (*pfy)(double,double,double,double,double,double),
                     double (*pfz)(double,double,double,double,double,double),
                     double dt,double delayX,double delayY,double delayZ)
{
	s_r_fx3d = pfx;
	s_r_fy3d = pfy;
	s_r_fz3d = pfz;
	s_r_dt = dt;

	s_r_delayIntX = delayX/dt;
	s_r_delayMax = s_r_delayIntX; 

	s_r_delayIntY = delayY/dt;
	if (s_r_delayMax < s_r_delayIntY)
		s_r_delayMax = s_r_delayIntY; 

	s_r_delayIntZ = delayZ/dt;
	if (s_r_delayMax < s_r_delayIntZ)
		s_r_delayMax = s_r_delayIntZ; 

	s_r_delayMax++;

    s_r_delayArrayX = MakeArrayDouble1(s_r_delayMax);
    s_r_delayArrayY = MakeArrayDouble1(s_r_delayMax);
    s_r_delayArrayZ = MakeArrayDouble1(s_r_delayMax);

	s_r_t = 0;
	s_r_tx = s_r_delayMax - s_r_delayIntX;
	s_r_ty = s_r_delayMax - s_r_delayIntY;
	s_r_tz = s_r_delayMax - s_r_delayIntZ;
}


void RungeKutta3D(double *px,double *py,double *pz)
{
double  x,vx1, vx2, vx3, vx4, xx,
                y,vy1, vy2, vy3, vy4, yy,
                z,vz1, vz2, vz3, vz4, zz,
                dx,ddx,ddxx,ddxy,ddxz,dy,ddy,ddyx,ddyy,ddyz,
		dz,ddz,ddzx,ddzy,ddzz;
int tx0,tx1,tx2,tx3,
        ty0,ty1,ty2,ty3,
        tz0,tz1,tz2,tz3;
double tmp;

	s_r_t = (s_r_t+1) % s_r_delayMax;
	s_r_tx = (s_r_tx+1) % s_r_delayMax;
	s_r_ty = (s_r_ty+1) % s_r_delayMax;
	s_r_tz = (s_r_tz+1) % s_r_delayMax;

	x = *px;
	y = *py;
	z = *pz;
	dx = s_r_delayArrayX[s_r_tx];
	dy = s_r_delayArrayY[s_r_tx];
	dz = s_r_delayArrayZ[s_r_tx];
	vx1 = s_r_dt * (*s_r_fx3d)(x,y,z,dx,dy,dz);
	dx = s_r_delayArrayX[s_r_ty];
	dy = s_r_delayArrayY[s_r_ty];
	dz = s_r_delayArrayZ[s_r_ty];
	vy1 = s_r_dt * (*s_r_fy3d)(x,y,z,dx,dy,dz);
	dx = s_r_delayArrayX[s_r_tz];
	dy = s_r_delayArrayY[s_r_tz];
	dz = s_r_delayArrayZ[s_r_tz];
	vz1 = s_r_dt * (*s_r_fz3d)(x,y,z,dx,dy,dz);
	xx = x + vx1 / 2;
	yy = y + vy1 / 2;
	zz = z + vz1 / 2;
	tx0 = (s_r_tx + s_r_delayMax - 1) % s_r_delayMax;
	ty0 = (s_r_ty + s_r_delayMax - 1) % s_r_delayMax;
	tz0 = (s_r_tz + s_r_delayMax - 1) % s_r_delayMax;
	tx1 = s_r_tx;
	ty1 = s_r_ty;
	tz1 = s_r_tz;
	tx2 = (s_r_tx + 1) % s_r_delayMax;
	ty2 = (s_r_ty + 1) % s_r_delayMax;
	tz2 = (s_r_tz + 1) % s_r_delayMax;
	tx3 = (s_r_tx + 2) % s_r_delayMax;
	ty3 = (s_r_ty + 2) % s_r_delayMax;
	tz3 = (s_r_tz + 2) % s_r_delayMax;

	tmp = s_r_delayArrayX[tx1]+s_r_delayArrayX[tx2];
	ddxx = (9*tmp - (s_r_delayArrayX[tx0]+s_r_delayArrayX[tx3]))/16;
	tmp = s_r_delayArrayY[tx1]+s_r_delayArrayY[tx2];
	ddyx = (9*tmp - (s_r_delayArrayY[tx0]+s_r_delayArrayY[tx3]))/16;
	tmp = s_r_delayArrayZ[tx1]+s_r_delayArrayZ[tx2];
	ddzx = (9*tmp - (s_r_delayArrayZ[tx0]+s_r_delayArrayZ[tx3]))/16;
	tmp = s_r_delayArrayX[ty1]+s_r_delayArrayX[ty2];
	ddxy = (9*tmp - (s_r_delayArrayX[ty0]+s_r_delayArrayX[ty3]))/16;
	tmp = s_r_delayArrayY[ty1]+s_r_delayArrayY[ty2];
	ddyy = (9*tmp - (s_r_delayArrayY[ty0]+s_r_delayArrayY[ty3]))/16;
	tmp = s_r_delayArrayZ[ty1]+s_r_delayArrayZ[ty2];
	ddzy = (9*tmp - (s_r_delayArrayZ[ty0]+s_r_delayArrayZ[ty3]))/16;
	tmp = s_r_delayArrayX[tz1]+s_r_delayArrayX[tz2];
	ddxz = (9*tmp - (s_r_delayArrayX[tz0]+s_r_delayArrayX[tz3]))/16;
	tmp = s_r_delayArrayY[tz1]+s_r_delayArrayY[tz2];
	ddyz = (9*tmp - (s_r_delayArrayY[tz0]+s_r_delayArrayY[tz3]))/16;
	tmp = s_r_delayArrayZ[tz1]+s_r_delayArrayZ[tz2];
	ddzz = (9*tmp - (s_r_delayArrayZ[tz0]+s_r_delayArrayZ[tz3]))/16;

	vx2 = s_r_dt * (*s_r_fx3d)(xx,yy,zz,ddxx,ddyx,ddzx);
	vy2 = s_r_dt * (*s_r_fy3d)(xx,yy,zz,ddxy,ddyy,ddzy);
	vz2 = s_r_dt * (*s_r_fz3d)(xx,yy,zz,ddxz,ddyz,ddzz);
	xx = x + vx2 / 2;
	yy = y + vy2 / 2;
	zz = z + vz2 / 2;
	vx3 = s_r_dt * (*s_r_fx3d)(xx,yy,zz,ddxx,ddyx,ddzx);
	vy3 = s_r_dt * (*s_r_fy3d)(xx,yy,zz,ddxy,ddyy,ddzy);
	vz3 = s_r_dt * (*s_r_fz3d)(xx,yy,zz,ddxz,ddyz,ddzz);
	xx = x + vx3;
	yy = y + vy3;
	zz = z + vz3;
	ddx = s_r_delayArrayX[tx2];
	ddy = s_r_delayArrayY[tx2];
	ddz = s_r_delayArrayZ[tx2];
	vx4 = s_r_dt * (*s_r_fx3d)(xx,yy,zz,ddx,ddy,ddz);
	ddx = s_r_delayArrayX[ty2];
	ddy = s_r_delayArrayY[ty2];
	ddz = s_r_delayArrayZ[ty2];
	vy4 = s_r_dt * (*s_r_fy3d)(xx,yy,zz,ddx,ddy,ddz);
	ddx = s_r_delayArrayX[tz2];
	ddy = s_r_delayArrayY[tz2];
	ddz = s_r_delayArrayZ[tz2];
	vz4 = s_r_dt * (*s_r_fz3d)(xx,yy,zz,ddx,ddy,ddz);
	*px = x + (vx1 + vx2 * 2 + vx3 * 2 + vx4) / 6;
	*py = y + (vy1 + vy2 * 2 + vy3 * 2 + vy4) / 6;
	*pz = z + (vz1 + vz2 * 2 + vz3 * 2 + vz4) / 6;
	s_r_delayArrayX[s_r_t] = *px;
	s_r_delayArrayY[s_r_t] = *py;
	s_r_delayArrayZ[s_r_t] = *pz;
}

void SetRungeKutta3DDelay(double x, double y, double z)
{
int i;

	for (i=0; i< s_r_delayMax;i++)
		s_r_delayArrayX[i] = x;
	for (i=0; i< s_r_delayMax;i++)
		s_r_delayArrayY[i] = y;
	for (i=0; i< s_r_delayMax;i++)
		s_r_delayArrayZ[i] = z;
}

void SetRungeKutta3DOneDelay(double x, double y, double z)
{
#if 1
	SetRungeKutta3DDelay(x, y, z);
#else
	s_r_delayArrayX[s_r_tx] = x;
	s_r_delayArrayY[s_r_ty] = y;
	s_r_delayArrayZ[s_r_tz] = z;
#endif /*else 1*/
}

/*** RungeKutta4D ***/

double *s_r_delayArrayU;
int s_r_delayIntU;
int s_r_tu =0;

double (*s_r_fx4d)(double,double,double,double,double,double,double,double);
double (*s_r_fy4d)(double,double,double,double,double,double,double,double);
double (*s_r_fz4d)(double,double,double,double,double,double,double,double);
double (*s_r_fu4d)(double,double,double,double,double,double,double,double);

void SetRungeKutta4D(double (*pfx)(double,double,
				double,double,double,double,double,double),
			double (*pfy)(double,double,
				double,double,double,double,double,double),
			double (*pfz)(double,double,
				double,double,double,double,double,double),
			double (*pfu)(double,double,
				double,double,double,double,double,double),
			double dt,double delayX,double delayY,
			double delayZ,double delayU)
{
	s_r_fx4d = pfx;
	s_r_fy4d = pfy;
	s_r_fz4d = pfz;
	s_r_fu4d = pfu;
	s_r_dt = dt;

	s_r_delayIntX = delayX/dt;
	s_r_delayMax = s_r_delayIntX; 

	s_r_delayIntY = delayY/dt;
	if (s_r_delayMax < s_r_delayIntY)
		s_r_delayMax = s_r_delayIntY; 

	s_r_delayIntZ = delayZ/dt;
	if (s_r_delayMax < s_r_delayIntZ)
		s_r_delayMax = s_r_delayIntZ; 

	s_r_delayIntU = delayU/dt;
	if (s_r_delayMax < s_r_delayIntU)
		s_r_delayMax = s_r_delayIntU; 
	
	s_r_delayMax++;

	s_r_delayArrayX = MakeArrayDouble1(s_r_delayMax);
    s_r_delayArrayY = MakeArrayDouble1(s_r_delayMax);
    s_r_delayArrayZ = MakeArrayDouble1(s_r_delayMax);
    s_r_delayArrayU = MakeArrayDouble1(s_r_delayMax);

	s_r_t = 0;
	s_r_tx = s_r_delayMax - s_r_delayIntX;
	s_r_ty = s_r_delayMax - s_r_delayIntY;
	s_r_tz = s_r_delayMax - s_r_delayIntZ;
	s_r_tu = s_r_delayMax - s_r_delayIntU;
}

void RungeKutta4D(double *px,double *py,double *pz,double *pu)
{
double  x,vx1, vx2, vx3, vx4, xx,
                y,vy1, vy2, vy3, vy4, yy,
                z,vz1, vz2, vz3, vz4, zz,
                u,vu1, vu2, vu3, vu4, uu,
                dx,ddx,ddxx,ddxy,ddxz,ddxu,dy,ddy,ddyx,ddyy,ddyz,ddyu,
		dz,ddz,ddzx,ddzy,ddzz,ddzu,du,ddu,ddux,dduy,dduz,dduu;
int tx0,tx1,tx2,tx3,
        ty0,ty1,ty2,ty3,
        tz0,tz1,tz2,tz3,
        tu0,tu1,tu2,tu3;
double tmp;

	/* moved a top so that Set4DOneDlay works */
	s_r_t = (s_r_t+1) % s_r_delayMax;
	s_r_tx = (s_r_tx+1) % s_r_delayMax;
	s_r_ty = (s_r_ty+1) % s_r_delayMax; 
	s_r_tz = (s_r_tz+1) % s_r_delayMax; 
	s_r_tu = (s_r_tu+1) % s_r_delayMax; 
	x = *px; 
	y = *py; 
	z = *pz; 
	u = *pu; 
	dx = s_r_delayArrayX[s_r_tx];
	dy = s_r_delayArrayY[s_r_tx];
	dz = s_r_delayArrayZ[s_r_tx];
	du = s_r_delayArrayU[s_r_tx];
	vx1 = s_r_dt * (*s_r_fx4d)(x,y,z,u,dx,dy,dz,du);
	dx = s_r_delayArrayX[s_r_ty];
	dy = s_r_delayArrayY[s_r_ty];
	dz = s_r_delayArrayZ[s_r_ty];
	du = s_r_delayArrayU[s_r_ty];
	vy1 = s_r_dt * (*s_r_fy4d)(x,y,z,u,dx,dy,dz,du);
	dx = s_r_delayArrayX[s_r_tz];
	dy = s_r_delayArrayY[s_r_tz];
	dz = s_r_delayArrayZ[s_r_tz];
	du = s_r_delayArrayU[s_r_tz];
	vz1 = s_r_dt * (*s_r_fz4d)(x,y,z,u,dx,dy,dz,du);
	dx = s_r_delayArrayX[s_r_tu];
	dy = s_r_delayArrayY[s_r_tu];
	dz = s_r_delayArrayZ[s_r_tu];
	du = s_r_delayArrayU[s_r_tu];
	vu1 = s_r_dt * (*s_r_fu4d)(x,y,z,u,dx,dy,dz,du);
	xx = x + vx1 / 2;
	yy = y + vy1 / 2;
	zz = z + vz1 / 2;
	uu = u + vu1 / 2;
	tx0 = (s_r_tx + s_r_delayMax - 1) % s_r_delayMax;
	ty0 = (s_r_ty + s_r_delayMax - 1) % s_r_delayMax;
	tz0 = (s_r_tz + s_r_delayMax - 1) % s_r_delayMax;
	tu0 = (s_r_tu + s_r_delayMax - 1) % s_r_delayMax;
	tx1 = s_r_tx;
	ty1 = s_r_ty;
	tz1 = s_r_tz;
	tu1 = s_r_tu;
	tx2 = (s_r_tx + 1) % s_r_delayMax;
	ty2 = (s_r_ty + 1) % s_r_delayMax;
	tz2 = (s_r_tz + 1) % s_r_delayMax;
	tu2 = (s_r_tu + 1) % s_r_delayMax;
	tx3 = (s_r_tx + 2) % s_r_delayMax;
	ty3 = (s_r_ty + 2) % s_r_delayMax;
	tz3 = (s_r_tz + 2) % s_r_delayMax;
	tu3 = (s_r_tu + 2) % s_r_delayMax;

	tmp = s_r_delayArrayX[tx1]+s_r_delayArrayX[tx2];
	ddxx = (9*tmp - (s_r_delayArrayX[tx0]+s_r_delayArrayX[tx3]))/16;
	tmp = s_r_delayArrayY[tx1]+s_r_delayArrayY[tx2];
	ddyx = (9*tmp - (s_r_delayArrayY[tx0]+s_r_delayArrayY[tx3]))/16;
	tmp = s_r_delayArrayZ[tx1]+s_r_delayArrayZ[tx2];
	ddzx = (9*tmp - (s_r_delayArrayZ[tx0]+s_r_delayArrayZ[tx3]))/16;
	tmp = s_r_delayArrayU[tx1]+s_r_delayArrayU[tx2];
	ddux = (9*tmp - (s_r_delayArrayU[tx0]+s_r_delayArrayU[tx3]))/16;
	tmp = s_r_delayArrayX[ty1]+s_r_delayArrayX[ty2];
	ddxy = (9*tmp - (s_r_delayArrayX[ty0]+s_r_delayArrayX[ty3]))/16;
	tmp = s_r_delayArrayY[ty1]+s_r_delayArrayY[ty2];
	ddyy = (9*tmp - (s_r_delayArrayY[ty0]+s_r_delayArrayY[ty3]))/16;
	tmp = s_r_delayArrayZ[ty1]+s_r_delayArrayZ[ty2];
	ddzy = (9*tmp - (s_r_delayArrayZ[ty0]+s_r_delayArrayZ[ty3]))/16;
	tmp = s_r_delayArrayU[ty1]+s_r_delayArrayU[ty2];
	dduy = (9*tmp - (s_r_delayArrayU[ty0]+s_r_delayArrayU[ty3]))/16;
	tmp = s_r_delayArrayX[tz1]+s_r_delayArrayX[tz2];
	ddxz = (9*tmp - (s_r_delayArrayX[tz0]+s_r_delayArrayX[tz3]))/16;
	tmp = s_r_delayArrayY[tz1]+s_r_delayArrayY[tz2];
	ddyz = (9*tmp - (s_r_delayArrayY[tz0]+s_r_delayArrayY[tz3]))/16;
	tmp = s_r_delayArrayZ[tz1]+s_r_delayArrayZ[tz2];
	ddzz = (9*tmp - (s_r_delayArrayZ[tz0]+s_r_delayArrayZ[tz3]))/16;
	tmp = s_r_delayArrayU[tz1]+s_r_delayArrayU[tz2];
	dduz = (9*tmp - (s_r_delayArrayU[tz0]+s_r_delayArrayU[tz3]))/16;
	tmp = s_r_delayArrayX[tu1]+s_r_delayArrayX[tu2];
	ddxu = (9*tmp - (s_r_delayArrayX[tu0]+s_r_delayArrayX[tu3]))/16;
	tmp = s_r_delayArrayY[tu1]+s_r_delayArrayY[tu2];
	ddyu = (9*tmp - (s_r_delayArrayY[tu0]+s_r_delayArrayY[tu3]))/16;
	tmp = s_r_delayArrayZ[tu1]+s_r_delayArrayZ[tu2];
	ddzu = (9*tmp - (s_r_delayArrayZ[tu0]+s_r_delayArrayZ[tu3]))/16;
	tmp = s_r_delayArrayU[tu1]+s_r_delayArrayU[tu2];
	dduu = (9*tmp - (s_r_delayArrayU[tu0]+s_r_delayArrayU[tu3]))/16;

	vx2 = s_r_dt * (*s_r_fx4d)(xx,yy,zz,uu,ddxx,ddyx,ddzx,ddux);
	vy2 = s_r_dt * (*s_r_fy4d)(xx,yy,zz,uu,ddxy,ddyy,ddzy,dduy);
	vz2 = s_r_dt * (*s_r_fz4d)(xx,yy,zz,uu,ddxz,ddyz,ddzz,dduz);
	vu2 = s_r_dt * (*s_r_fu4d)(xx,yy,zz,uu,ddxu,ddyu,ddzu,dduu);
	xx = x + vx2 / 2;
	yy = y + vy2 / 2;
	zz = z + vz2 / 2;
	uu = u + vu2 / 2;
	vx3 = s_r_dt * (*s_r_fx4d)(xx,yy,zz,uu,ddxx,ddyx,ddzx,ddux);
	vy3 = s_r_dt * (*s_r_fy4d)(xx,yy,zz,uu,ddxy,ddyy,ddzy,dduy);
	vz3 = s_r_dt * (*s_r_fz4d)(xx,yy,zz,uu,ddxz,ddyz,ddzz,dduz);
	vu3 = s_r_dt * (*s_r_fu4d)(xx,yy,zz,uu,ddxu,ddyu,ddzu,dduu);
	xx = x + vx3;
	yy = y + vy3;
	zz = z + vz3;
	uu = u + vu3;
	ddx = s_r_delayArrayX[tx2];
	ddy = s_r_delayArrayY[tx2];
	ddz = s_r_delayArrayZ[tx2];
	ddu = s_r_delayArrayU[tx2];
	vx4 = s_r_dt * (*s_r_fx4d)(xx,yy,zz,uu,ddx,ddy,ddz,ddu);
	ddx = s_r_delayArrayX[ty2];
	ddy = s_r_delayArrayY[ty2];
	ddz = s_r_delayArrayZ[ty2];
	ddu = s_r_delayArrayU[ty2];
	vy4 = s_r_dt * (*s_r_fy4d)(xx,yy,zz,uu,ddx,ddy,ddz,ddu);
	ddx = s_r_delayArrayX[tz2];
	ddy = s_r_delayArrayY[tz2];
	ddz = s_r_delayArrayZ[tz2];
	ddu = s_r_delayArrayU[tz2];
	vz4 = s_r_dt * (*s_r_fz4d)(xx,yy,zz,uu,ddx,ddy,ddz,ddu);
	ddx = s_r_delayArrayX[tu2];
	ddy = s_r_delayArrayY[tu2];
	ddz = s_r_delayArrayZ[tu2];
	ddu = s_r_delayArrayU[tu2];
	vu4 = s_r_dt * (*s_r_fu4d)(xx,yy,zz,uu,ddx,ddy,ddz,ddu);
	*px = x + (vx1 + vx2 * 2 + vx3 * 2 + vx4) / 6;
	*py = y + (vy1 + vy2 * 2 + vy3 * 2 + vy4) / 6;
	*pz = z + (vz1 + vz2 * 2 + vz3 * 2 + vz4) / 6;
	*pu = u + (vu1 + vu2 * 2 + vu3 * 2 + vu4) / 6;

	s_r_delayArrayX[s_r_t] = *px;
	s_r_delayArrayY[s_r_t] = *py;
	s_r_delayArrayZ[s_r_t] = *pz;
	s_r_delayArrayU[s_r_t] = *pu;
}

void SetRungeKutta4DDelay(double x, double y, double z, double u)
{
int i;

	for (i=0; i< s_r_delayMax;i++)
		s_r_delayArrayX[i] = x;
	for (i=0; i< s_r_delayMax;i++)
		s_r_delayArrayY[i] = y;
	for (i=0; i< s_r_delayMax;i++)
		s_r_delayArrayZ[i] = z;
	for (i=0; i< s_r_delayMax;i++)
		s_r_delayArrayU[i] = u;
}

void SetRungeKutta4DOneDelay(double x, double y, double z, double u)
{
#if 1
	SetRungeKutta4DDelay(x, y, z, u);
#else
	s_r_delayArrayX[s_r_tx] = x;
	s_r_delayArrayY[s_r_ty] = y;
	s_r_delayArrayZ[s_r_tz] = z;
	s_r_delayArrayU[s_r_tu] = u;
#endif /*else 1*/
}

/*** RungeKutta6D ***/

double *s_r_delayArrayV;
int s_r_delayIntV;
int s_r_tv =0;
double *s_r_delayArrayW;
int s_r_delayIntW;
int s_r_tw =0;

double (*s_r_fx6d)(double,double,double,double,double,
	double,double,double,double,double,double,double);
double (*s_r_fy6d)(double,double,double,double,double,
	double,double,double,double,double,double,double);
double (*s_r_fz6d)(double,double,double,double,double,
	double,double,double,double,double,double,double);
double (*s_r_fu6d)(double,double,double,double,double,
	double,double,double,double,double,double,double);
double (*s_r_fv6d)(double,double,double,double,double,
	double,double,double,double,double,double,double);
double (*s_r_fw6d)(double,double,double,double,double,
	double,double,double,double,double,double,double);

void SetRungeKutta6D(double (*pfx)(double,double,double,double,
			double,double,double,double,double,double,double,double),
			double (*pfy)(double,double,double,double,
			double,double,double,double,double,double,double,double),
			double (*pfz)(double,double,double,double,
			double,double,double,double,double,double,double,double),
			double (*pfu)(double,double,double,double,
			double,double,double,double,double,double,double,double),
			double (*pfv)(double,double,double,double,
			double,double,double,double,double,double,double,double),
			double (*pfw)(double,double,double,double,
			double,double,double,double,double,double,double,double),
			double dt,double delayX,double delayY,
			double delayZ,double delayU,double delayV,double delayW)
{
        s_r_fx6d = pfx;
        s_r_fy6d = pfy;
        s_r_fz6d = pfz;
        s_r_fu6d = pfu;
        s_r_fv6d = pfv;
        s_r_fw6d = pfw;
        s_r_dt = dt;

        s_r_delayIntX = delayX/dt;
	s_r_delayMax = s_r_delayIntX; 

        s_r_delayIntY = delayY/dt;
	if (s_r_delayMax < s_r_delayIntY)
		s_r_delayMax = s_r_delayIntY; 

        s_r_delayIntZ = delayZ/dt;
	if (s_r_delayMax < s_r_delayIntZ)
		s_r_delayMax = s_r_delayIntZ; 

        s_r_delayIntU = delayU/dt;
	if (s_r_delayMax < s_r_delayIntU)
		s_r_delayMax = s_r_delayIntU; 
		
	s_r_delayIntV = delayV/dt;
	if (s_r_delayMax < s_r_delayIntV)
		s_r_delayMax = s_r_delayIntV; 
		
	s_r_delayIntW = delayW/dt;
	if (s_r_delayMax < s_r_delayIntW)
		s_r_delayMax = s_r_delayIntW; 
		
	s_r_delayMax++;

        if(!(s_r_delayArrayX = MakeArrayDouble1(s_r_delayMax )))
                s_OOMError();
        if(!(s_r_delayArrayY = MakeArrayDouble1(s_r_delayMax )))
                s_OOMError();
        if(!(s_r_delayArrayZ = MakeArrayDouble1(s_r_delayMax )))
                s_OOMError();
        if(!(s_r_delayArrayU = MakeArrayDouble1(s_r_delayMax )))
                s_OOMError();
        if(!(s_r_delayArrayV = MakeArrayDouble1(s_r_delayMax )))
                s_OOMError();
        if(!(s_r_delayArrayW = MakeArrayDouble1(s_r_delayMax )))
                s_OOMError();
                
    s_r_t = 0;
	s_r_tx = s_r_delayMax - s_r_delayIntX;
	s_r_ty = s_r_delayMax - s_r_delayIntY;
	s_r_tz = s_r_delayMax - s_r_delayIntZ;
	s_r_tu = s_r_delayMax - s_r_delayIntU;
	s_r_tv = s_r_delayMax - s_r_delayIntV;
	s_r_tw = s_r_delayMax - s_r_delayIntW;
}

void RungeKutta6D(double *px,double *py,double *pz,double *pu,double *pv,double *pw)
{
double  x,vx1, vx2, vx3, vx4, xx,
                y,vy1, vy2, vy3, vy4, yy,
                z,vz1, vz2, vz3, vz4, zz,
                u,vu1, vu2, vu3, vu4, uu,
                v,vv1, vv2, vv3, vv4, vv,
                w,vw1, vw2, vw3, vw4, ww,
                dx,ddx,ddxx,ddxy,ddxz,ddxu,ddxv,ddxw,dy,ddy,ddyx,ddyy,ddyz,ddyu,ddyv,ddyw,
		dz,ddz,ddzx,ddzy,ddzz,ddzu,ddzv,ddzw,du,ddu,ddux,dduy,dduz,dduu,dduv,dduw,
		dv,ddv,ddvx,ddvy,ddvz,ddvu,ddvv,ddvw,dw,ddw,ddwx,ddwy,ddwz,ddwu,ddwv,ddww;
int tx0,tx1,tx2,tx3,
        ty0,ty1,ty2,ty3,
        tz0,tz1,tz2,tz3,
        tu0,tu1,tu2,tu3,
        tv0,tv1,tv2,tv3,
        tw0,tw1,tw2,tw3;
double tmp;

	/* moved a top so that Set4DOneDlay works */
	s_r_t = (s_r_t+1) % s_r_delayMax;
	s_r_tx = (s_r_tx+1) % s_r_delayMax;
	s_r_ty = (s_r_ty+1) % s_r_delayMax; 
	s_r_tz = (s_r_tz+1) % s_r_delayMax;
	s_r_tu = (s_r_tu+1) % s_r_delayMax;
	s_r_tv = (s_r_tv+1) % s_r_delayMax;
	s_r_tw = (s_r_tw+1) % s_r_delayMax; 
	x = *px; 
	y = *py; 
	z = *pz;
	u = *pu;
	v = *pv;
	w = *pw; 
	dx = s_r_delayArrayX[s_r_tx];
	dy = s_r_delayArrayY[s_r_tx];
	dz = s_r_delayArrayZ[s_r_tx];
	du = s_r_delayArrayU[s_r_tx];
	dv = s_r_delayArrayV[s_r_tx];
	dw = s_r_delayArrayW[s_r_tx];
	vx1 = s_r_dt * (*s_r_fx6d)(x,y,z,u,v,w,dx,dy,dz,du,dv,dw);
	dx = s_r_delayArrayX[s_r_ty];
	dy = s_r_delayArrayY[s_r_ty];
	dz = s_r_delayArrayZ[s_r_ty];
	du = s_r_delayArrayU[s_r_ty];
	dv = s_r_delayArrayV[s_r_ty];
	dw = s_r_delayArrayW[s_r_ty];
	vy1 = s_r_dt * (*s_r_fy6d)(x,y,z,u,v,w,dx,dy,dz,du,dv,dw);
	dx = s_r_delayArrayX[s_r_tz];
	dy = s_r_delayArrayY[s_r_tz];
	dz = s_r_delayArrayZ[s_r_tz];
	du = s_r_delayArrayU[s_r_tz];
	dv = s_r_delayArrayV[s_r_tz];
	dw = s_r_delayArrayW[s_r_tz];
	vz1 = s_r_dt * (*s_r_fz6d)(x,y,z,u,v,w,dx,dy,dz,du,dv,dw);
	dx = s_r_delayArrayX[s_r_tu];
	dy = s_r_delayArrayY[s_r_tu];
	dz = s_r_delayArrayZ[s_r_tu];
	du = s_r_delayArrayU[s_r_tu];
	dv = s_r_delayArrayV[s_r_tu];
	dw = s_r_delayArrayW[s_r_tu];
	vu1 = s_r_dt * (*s_r_fu6d)(x,y,z,u,v,w,dx,dy,dz,du,dv,dw);
	dx = s_r_delayArrayX[s_r_tv];
	dy = s_r_delayArrayY[s_r_tv];
	dz = s_r_delayArrayZ[s_r_tv];
	du = s_r_delayArrayU[s_r_tv];
	dv = s_r_delayArrayV[s_r_tv];
	dw = s_r_delayArrayW[s_r_tv];
	vv1 = s_r_dt * (*s_r_fv6d)(x,y,z,u,v,w,dx,dy,dz,du,dv,dw);
	dx = s_r_delayArrayX[s_r_tw];
	dy = s_r_delayArrayY[s_r_tw];
	dz = s_r_delayArrayZ[s_r_tw];
	du = s_r_delayArrayU[s_r_tw];
	dv = s_r_delayArrayV[s_r_tw];
	dw = s_r_delayArrayW[s_r_tw];
	vw1 = s_r_dt * (*s_r_fw6d)(x,y,z,u,v,w,dx,dy,dz,du,dv,dw);
	xx = x + vx1 / 2;
	yy = y + vy1 / 2;
	zz = z + vz1 / 2;
	uu = u + vu1 / 2;
	vv = v + vv1 / 2;
	ww = w + vw1 / 2;
	
	tx0 = (s_r_tx + s_r_delayMax - 1) % s_r_delayMax;
	ty0 = (s_r_ty + s_r_delayMax - 1) % s_r_delayMax;
	tz0 = (s_r_tz + s_r_delayMax - 1) % s_r_delayMax;
	tu0 = (s_r_tu + s_r_delayMax - 1) % s_r_delayMax;
	tv0 = (s_r_tv + s_r_delayMax - 1) % s_r_delayMax;
	tw0 = (s_r_tw + s_r_delayMax - 1) % s_r_delayMax;
	tx1 = s_r_tx;
	ty1 = s_r_ty;
	tz1 = s_r_tz;
	tu1 = s_r_tu;
	tv1 = s_r_tv;
	tw1 = s_r_tw;
	tx2 = (s_r_tx + 1) % s_r_delayMax;
	ty2 = (s_r_ty + 1) % s_r_delayMax;
	tz2 = (s_r_tz + 1) % s_r_delayMax;
	tu2 = (s_r_tu + 1) % s_r_delayMax;
	tv2 = (s_r_tv + 1) % s_r_delayMax;
	tw2 = (s_r_tw + 1) % s_r_delayMax;
	tx3 = (s_r_tx + 2) % s_r_delayMax;
	ty3 = (s_r_ty + 2) % s_r_delayMax;
	tz3 = (s_r_tz + 2) % s_r_delayMax;
	tu3 = (s_r_tu + 2) % s_r_delayMax;
	tv3 = (s_r_tv + 2) % s_r_delayMax;
	tw3 = (s_r_tw + 2) % s_r_delayMax;
	



	tmp = s_r_delayArrayX[tx1]+s_r_delayArrayX[tx2];
	ddxx = (9*tmp - (s_r_delayArrayX[tx0]+s_r_delayArrayX[tx3]))/16;
	tmp = s_r_delayArrayY[tx1]+s_r_delayArrayY[tx2];
	ddyx = (9*tmp - (s_r_delayArrayY[tx0]+s_r_delayArrayY[tx3]))/16;
	tmp = s_r_delayArrayZ[tx1]+s_r_delayArrayZ[tx2];
	ddzx = (9*tmp - (s_r_delayArrayZ[tx0]+s_r_delayArrayZ[tx3]))/16;
	tmp = s_r_delayArrayU[tx1]+s_r_delayArrayU[tx2];
	ddux = (9*tmp - (s_r_delayArrayU[tx0]+s_r_delayArrayU[tx3]))/16;
	tmp = s_r_delayArrayV[tx1]+s_r_delayArrayV[tx2];
	ddvx = (9*tmp - (s_r_delayArrayV[tx0]+s_r_delayArrayV[tx3]))/16;
	tmp = s_r_delayArrayW[tx1]+s_r_delayArrayW[tx2];
	ddwx = (9*tmp - (s_r_delayArrayW[tx0]+s_r_delayArrayW[tx3]))/16;
	
	tmp = s_r_delayArrayX[ty1]+s_r_delayArrayX[ty2];
	ddxy = (9*tmp - (s_r_delayArrayX[ty0]+s_r_delayArrayX[ty3]))/16;
	tmp = s_r_delayArrayY[ty1]+s_r_delayArrayY[ty2];
	ddyy = (9*tmp - (s_r_delayArrayY[ty0]+s_r_delayArrayY[ty3]))/16;
	tmp = s_r_delayArrayZ[ty1]+s_r_delayArrayZ[ty2];
	ddzy = (9*tmp - (s_r_delayArrayZ[ty0]+s_r_delayArrayZ[ty3]))/16;
	tmp = s_r_delayArrayU[ty1]+s_r_delayArrayU[ty2];
	dduy = (9*tmp - (s_r_delayArrayU[ty0]+s_r_delayArrayU[ty3]))/16;
	tmp = s_r_delayArrayV[ty1]+s_r_delayArrayV[ty2];
	ddvy = (9*tmp - (s_r_delayArrayV[ty0]+s_r_delayArrayV[ty3]))/16;
	tmp = s_r_delayArrayW[ty1]+s_r_delayArrayW[ty2];
	ddwy = (9*tmp - (s_r_delayArrayW[ty0]+s_r_delayArrayW[ty3]))/16;
	
	tmp = s_r_delayArrayX[tz1]+s_r_delayArrayX[tz2];
	ddxz = (9*tmp - (s_r_delayArrayX[tz0]+s_r_delayArrayX[tz3]))/16;
	tmp = s_r_delayArrayY[tz1]+s_r_delayArrayY[tz2];
	ddyz = (9*tmp - (s_r_delayArrayY[tz0]+s_r_delayArrayY[tz3]))/16;
	tmp = s_r_delayArrayZ[tz1]+s_r_delayArrayZ[tz2];
	ddzz = (9*tmp - (s_r_delayArrayZ[tz0]+s_r_delayArrayZ[tz3]))/16;
	tmp = s_r_delayArrayU[tz1]+s_r_delayArrayU[tz2];
	dduz = (9*tmp - (s_r_delayArrayU[tz0]+s_r_delayArrayU[tz3]))/16;
	tmp = s_r_delayArrayV[tz1]+s_r_delayArrayV[tz2];
	ddvz = (9*tmp - (s_r_delayArrayV[tz0]+s_r_delayArrayV[tz3]))/16;
	tmp = s_r_delayArrayW[tz1]+s_r_delayArrayW[tz2];
	ddwz = (9*tmp - (s_r_delayArrayW[tz0]+s_r_delayArrayW[tz3]))/16;
	
	tmp = s_r_delayArrayX[tu1]+s_r_delayArrayX[tu2];
	ddxu = (9*tmp - (s_r_delayArrayX[tu0]+s_r_delayArrayX[tu3]))/16;
	tmp = s_r_delayArrayY[tu1]+s_r_delayArrayY[tu2];
	ddyu = (9*tmp - (s_r_delayArrayY[tu0]+s_r_delayArrayY[tu3]))/16;
	tmp = s_r_delayArrayZ[tu1]+s_r_delayArrayZ[tu2];
	ddzu = (9*tmp - (s_r_delayArrayZ[tu0]+s_r_delayArrayZ[tu3]))/16;
	tmp = s_r_delayArrayU[tu1]+s_r_delayArrayU[tu2];
	dduu = (9*tmp - (s_r_delayArrayU[tu0]+s_r_delayArrayU[tu3]))/16;
    tmp = s_r_delayArrayV[tu1]+s_r_delayArrayV[tu2];
	ddvu = (9*tmp - (s_r_delayArrayV[tu0]+s_r_delayArrayV[tu3]))/16;
    tmp = s_r_delayArrayW[tu1]+s_r_delayArrayW[tu2];
	ddwu = (9*tmp - (s_r_delayArrayW[tu0]+s_r_delayArrayW[tu3]))/16;

    tmp = s_r_delayArrayX[tv1]+s_r_delayArrayX[tv2];
	ddxv = (9*tmp - (s_r_delayArrayX[tv0]+s_r_delayArrayX[tv3]))/16;
	tmp = s_r_delayArrayY[tv1]+s_r_delayArrayY[tv2];
	ddyv = (9*tmp - (s_r_delayArrayY[tv0]+s_r_delayArrayY[tv3]))/16;
	tmp = s_r_delayArrayZ[tv1]+s_r_delayArrayZ[tv2];
	ddzv = (9*tmp - (s_r_delayArrayZ[tv0]+s_r_delayArrayZ[tv3]))/16;
	tmp = s_r_delayArrayU[tv1]+s_r_delayArrayU[tv2];
	dduv = (9*tmp - (s_r_delayArrayU[tv0]+s_r_delayArrayU[tv3]))/16;
    tmp = s_r_delayArrayV[tv1]+s_r_delayArrayV[tv2];
	ddvv = (9*tmp - (s_r_delayArrayV[tv0]+s_r_delayArrayV[tv3]))/16;
    tmp = s_r_delayArrayW[tv1]+s_r_delayArrayW[tv2];
	ddwv = (9*tmp - (s_r_delayArrayW[tv0]+s_r_delayArrayW[tv3]))/16;

	tmp = s_r_delayArrayX[tw1]+s_r_delayArrayX[tw2];
	ddxw = (9*tmp - (s_r_delayArrayX[tw0]+s_r_delayArrayX[tw3]))/16;
	tmp = s_r_delayArrayY[tw1]+s_r_delayArrayY[tw2];
	ddyw = (9*tmp - (s_r_delayArrayY[tw0]+s_r_delayArrayY[tw3]))/16;
	tmp = s_r_delayArrayZ[tw1]+s_r_delayArrayZ[tw2];
	ddzw = (9*tmp - (s_r_delayArrayZ[tw0]+s_r_delayArrayZ[tw3]))/16;
	tmp = s_r_delayArrayU[tw1]+s_r_delayArrayU[tw2];
	dduw = (9*tmp - (s_r_delayArrayU[tw0]+s_r_delayArrayU[tw3]))/16;
    tmp = s_r_delayArrayV[tw1]+s_r_delayArrayV[tw2];
	ddvw = (9*tmp - (s_r_delayArrayV[tw0]+s_r_delayArrayV[tw3]))/16;
    tmp = s_r_delayArrayW[tw1]+s_r_delayArrayW[tw2];
	ddww = (9*tmp - (s_r_delayArrayW[tw0]+s_r_delayArrayW[tw3]))/16;






	vx2 = s_r_dt * (*s_r_fx6d)(xx,yy,zz,uu,vv,ww,ddxx,ddyx,ddzx,ddux,ddvx,ddwx);
	vy2 = s_r_dt * (*s_r_fy6d)(xx,yy,zz,uu,vv,ww,ddxy,ddyy,ddzy,dduy,ddvy,ddwy);
	vz2 = s_r_dt * (*s_r_fz6d)(xx,yy,zz,uu,vv,ww,ddxz,ddyz,ddzz,dduz,ddvx,ddwz);
	vu2 = s_r_dt * (*s_r_fu6d)(xx,yy,zz,uu,vv,ww,ddxu,ddyu,ddzu,dduu,ddvu,ddwu);
	vv2 = s_r_dt * (*s_r_fv6d)(xx,yy,zz,uu,vv,ww,ddxv,ddyv,ddzv,dduv,ddvv,ddwv);
	vw2 = s_r_dt * (*s_r_fw6d)(xx,yy,zz,uu,vv,ww,ddxw,ddyw,ddzw,dduw,ddvw,ddww);
	xx = x + vx2 / 2;
	yy = y + vy2 / 2;
	zz = z + vz2 / 2;
	uu = u + vu2 / 2;
	vv = v + vv2 / 2;
	ww = w + vw2 / 2;
	vx3 = s_r_dt * (*s_r_fx6d)(xx,yy,zz,uu,vv,ww,ddxx,ddyx,ddzx,ddux,ddvx,ddwx);
	vy3 = s_r_dt * (*s_r_fy6d)(xx,yy,zz,uu,vv,ww,ddxy,ddyy,ddzy,dduy,ddvy,ddwy);
	vz3 = s_r_dt * (*s_r_fz6d)(xx,yy,zz,uu,vv,ww,ddxz,ddyz,ddzz,dduz,ddvz,ddwz);
	vu3 = s_r_dt * (*s_r_fu6d)(xx,yy,zz,uu,vv,ww,ddxu,ddyu,ddzu,dduu,ddvu,ddwu);
	vv3 = s_r_dt * (*s_r_fv6d)(xx,yy,zz,uu,vv,ww,ddxv,ddyv,ddzv,dduv,ddvv,ddwv);
	vw3 = s_r_dt * (*s_r_fw6d)(xx,yy,zz,uu,vv,ww,ddxw,ddyw,ddzw,dduw,ddvw,ddww);
	xx = x + vx3;
	yy = y + vy3;
	zz = z + vz3;
	uu = u + vu3;
	vv = v + vv3;
	ww = w + vw3;
	
	ddx = s_r_delayArrayX[tx2];
	ddy = s_r_delayArrayY[tx2];
	ddz = s_r_delayArrayZ[tx2];
	ddu = s_r_delayArrayU[tx2];
	ddv = s_r_delayArrayV[tx2];
	ddw = s_r_delayArrayW[tx2];
	vx4 = s_r_dt * (*s_r_fx6d)(xx,yy,zz,uu,vv,ww,ddx,ddy,ddz,ddu,ddv,ddw);
	ddx = s_r_delayArrayX[ty2];
	ddy = s_r_delayArrayY[ty2];
	ddz = s_r_delayArrayZ[ty2];
	ddu = s_r_delayArrayU[ty2];
	ddv = s_r_delayArrayV[ty2];
	ddw = s_r_delayArrayW[ty2];
	vy4 = s_r_dt * (*s_r_fy6d)(xx,yy,zz,uu,vv,ww,ddx,ddy,ddz,ddu,ddv,ddw);
	ddx = s_r_delayArrayX[tz2];
	ddy = s_r_delayArrayY[tz2];
	ddz = s_r_delayArrayZ[tz2];
	ddu = s_r_delayArrayU[tz2];
	ddv = s_r_delayArrayV[tz2];
	ddw = s_r_delayArrayW[tz2];
	vz4 = s_r_dt * (*s_r_fz6d)(xx,yy,zz,uu,vv,ww,ddx,ddy,ddz,ddu,ddv,ddw);
	ddx = s_r_delayArrayX[tu2];
	ddy = s_r_delayArrayY[tu2];
	ddz = s_r_delayArrayZ[tu2];
	ddu = s_r_delayArrayU[tu2];
	ddv = s_r_delayArrayV[tu2];
	ddw = s_r_delayArrayW[tu2];
	vu4 = s_r_dt * (*s_r_fu6d)(xx,yy,zz,uu,vv,ww,ddx,ddy,ddz,ddu,ddv,ddw);
	ddx = s_r_delayArrayX[tv2];
	ddy = s_r_delayArrayY[tv2];
	ddz = s_r_delayArrayZ[tv2];
	ddu = s_r_delayArrayU[tv2];
	ddv = s_r_delayArrayV[tv2];
	ddw = s_r_delayArrayW[tv2];
	vv4 = s_r_dt * (*s_r_fv6d)(xx,yy,zz,uu,vv,ww,ddx,ddy,ddz,ddu,ddv,ddw);
	ddx = s_r_delayArrayX[tw2];
	ddy = s_r_delayArrayY[tw2];
	ddz = s_r_delayArrayZ[tw2];
	ddu = s_r_delayArrayU[tw2];
	ddv = s_r_delayArrayV[tw2];
	ddw = s_r_delayArrayW[tw2];
	vw4 = s_r_dt * (*s_r_fw6d)(xx,yy,zz,uu,vv,ww,ddx,ddy,ddz,ddu,ddv,ddw);

//s_printf("vx1 = %f, vx2 = %f, \nvx3 = %f, vx4 = %f\n", vx1, vx2, vx3, vx4);
//s_printf("vv1 = %f, vv2 = %f, \nvv3 = %f, vv4 = %f\n", vv1, vv2, vv3, vv4);
	*px = x + (vx1 + vx2 * 2 + vx3 * 2 + vx4) / 6;
	*py = y + (vy1 + vy2 * 2 + vy3 * 2 + vy4) / 6;
	*pz = z + (vz1 + vz2 * 2 + vz3 * 2 + vz4) / 6;
	*pu = u + (vu1 + vu2 * 2 + vu3 * 2 + vu4) / 6;
	*pv = v + (vv1 + vv2 * 2 + vv3 * 2 + vv4) / 6;
	*pw = w + (vw1 + vw2 * 2 + vw3 * 2 + vw4) / 6;


	s_r_delayArrayX[s_r_t] = *px;
	s_r_delayArrayY[s_r_t] = *py;
	s_r_delayArrayZ[s_r_t] = *pz;
	s_r_delayArrayU[s_r_t] = *pu;
	s_r_delayArrayV[s_r_t] = *pv;
	s_r_delayArrayW[s_r_t] = *pw;
	
}

void SetRungeKutta6DDelay(double x, double y, double z, double u, double v, double w)
{
int i;

	for (i=0; i< s_r_delayMax;i++)
		s_r_delayArrayX[i] = x;
	for (i=0; i< s_r_delayMax;i++)
		s_r_delayArrayY[i] = y;
	for (i=0; i< s_r_delayMax;i++)
		s_r_delayArrayZ[i] = z;
	for (i=0; i< s_r_delayMax;i++)
		s_r_delayArrayU[i] = u;
	for (i=0; i< s_r_delayMax;i++)
		s_r_delayArrayV[i] = v;
	for (i=0; i< s_r_delayMax;i++)
		s_r_delayArrayW[i] = w;
		
}


/* Runge-Kutta All Number */

typedef double (*doublevoidFunc)(void);

doublevoidFunc *s_r_fall;

double **s_r_mainVar;
double **s_r_funcVar;
double *s_r_v1, *s_r_v2, *s_r_v3, *s_r_v4;

void SetRungeKuttaAll(int n, ...)
{
va_list ap;
int i;

	s_r_n = n;
	s_r_fall = calloc(n, sizeof (doublevoidFunc));
	if (!s_r_fall) s_OOMError();
	s_r_mainVar = calloc(n, sizeof (double *));
	if (!s_r_mainVar) s_OOMError();
	s_r_funcVar = calloc(n, sizeof (double *));
	if (!s_r_funcVar) s_OOMError();

	s_r_v1 = MakeArrayDouble1(n);
	s_r_v2 = MakeArrayDouble1(n);
	s_r_v3 = MakeArrayDouble1(n);
	s_r_v4 = MakeArrayDouble1(n);
	
	va_start(ap, n);
		
	for (i = 0; i < n; i++)
		s_r_fall[i] = va_arg(ap, doublevoidFunc);
	for (i = 0; i < n; i++)
		s_r_mainVar[i] = va_arg(ap, double *);
	for (i = 0; i < n; i++)
		s_r_funcVar[i] = va_arg(ap, double *);
    s_r_dt = va_arg(ap, double);

	va_end(ap);
}

void RungeKuttaAll(void)
{
int i;

	//x = *px;
	for(i = 0; i < s_r_n; i++)
	{
		*s_r_funcVar[i] = *s_r_mainVar[i];
	}
	for(i = 0; i < s_r_n; i++)
	{
		s_r_v1[i] = s_r_dt * (*s_r_fall[i])();
	}

	for(i = 0; i < s_r_n; i++)
	{
		*s_r_funcVar[i] = *s_r_mainVar[i] + s_r_v1[i]/2;
	}
	for(i = 0; i < s_r_n; i++)
	{
		s_r_v2[i] = s_r_dt * (*s_r_fall[i])();
	}

	for(i = 0; i < s_r_n; i++)
	{
		*s_r_funcVar[i] = *s_r_mainVar[i] + s_r_v2[i]/2;
	}
	for(i = 0; i < s_r_n; i++)
	{
		s_r_v3[i] = s_r_dt * (*s_r_fall[i])();
	}

	for(i = 0; i < s_r_n; i++)
	{
		*s_r_funcVar[i] = *s_r_mainVar[i] + s_r_v3[i];
	}
	for(i = 0; i < s_r_n; i++)
	{
		s_r_v4[i] = s_r_dt * (*s_r_fall[i])();
	}

	for(i = 0; i < s_r_n; i++)
	{
		*s_r_mainVar[i] += (s_r_v1[i] + s_r_v2[i] + s_r_v3[i] + s_r_v4[i])/6;
	}
	//*px = x + (vx1 + vx2 * 2 + vx3 * 2 + vx4) / 6;
}


/* Runge-Kutta All Number Delay */

double **s_r_funcDelayVar;
double **s_r_delayArrayAll;
int *s_r_delayIntAll;
int *s_r_tAll;

void SetRungeKuttaAllD(int n, ...)
{
va_list ap;
int i;
double *delayAll;

	s_r_n = n;
	s_r_fall = calloc(n, sizeof (doublevoidFunc));
	if (!s_r_fall) s_OOMError();
	s_r_mainVar = calloc(n, sizeof (double *));
	if (!s_r_mainVar) s_OOMError();
	s_r_funcVar = calloc(n, sizeof (double *));
	if (!s_r_funcVar) s_OOMError();
	s_r_funcDelayVar = calloc(n, sizeof (double *));
	if (!s_r_funcDelayVar) s_OOMError();

	s_r_delayArrayAll = calloc(n, sizeof (double *));
	if (!s_r_delayArrayAll) s_OOMError();
	s_r_delayIntAll = calloc(n, sizeof (int));
	if (!s_r_delayIntAll) s_OOMError();
	s_r_tAll = calloc(n, sizeof (int));
	if (!s_r_tAll) s_OOMError();

	delayAll = MakeArrayDouble1(n);

	s_r_v1 = MakeArrayDouble1(n);
	s_r_v2 = MakeArrayDouble1(n);
	s_r_v3 = MakeArrayDouble1(n);
	s_r_v4 = MakeArrayDouble1(n);
	
	va_start(ap, n);
		
	for (i = 0; i < n; i++)
		s_r_fall[i] = va_arg(ap, doublevoidFunc);
	for (i = 0; i < n; i++)
		s_r_mainVar[i] = va_arg(ap, double *);
	for (i = 0; i < n; i++)
		s_r_funcVar[i] = va_arg(ap, double *);
	for (i = 0; i < n; i++)
		s_r_funcDelayVar[i] = va_arg(ap, double *);

    s_r_dt = va_arg(ap, double);

	for (i = 0; i < n; i++)
		delayAll[i] = va_arg(ap, double);
	va_end(ap);
	
	s_r_delayMax = 0;
	for (i = 0; i < n; i++)
	{
    	s_r_delayIntAll[i] = delayAll[i]/s_r_dt;
		s_r_delayMax = max(s_r_delayMax, s_r_delayIntAll[i]);
	}

	s_r_delayMax++;
	
	free(delayAll);

	for (i = 0; i < n; i++)
    	s_r_delayArrayAll[i] = MakeArrayDouble1(s_r_delayMax);

	s_r_t = 0;
	for (i = 0; i < n; i++)
		s_r_tAll[i] = s_r_delayMax - s_r_delayIntAll[i];
}

void RungeKuttaAllD(void)
{
int t0,t1,t2,t3;
double tmp;
int i, j;

	s_r_t = (s_r_t+1) % s_r_delayMax;
	for(i = 0; i < s_r_n; i++)
		s_r_tAll[i] = (s_r_tAll[i]+1) % s_r_delayMax;

	//x = *px;
	for(i = 0; i < s_r_n; i++)
	{
		*s_r_funcVar[i] = *s_r_mainVar[i];
	}
	for(i = 0; i < s_r_n; i++)
	{
		for(j = 0; j < s_r_n; j++)
			*s_r_funcDelayVar[j] = s_r_delayArrayAll[j][s_r_tAll[i]];
		s_r_v1[i] = s_r_dt * (*s_r_fall[i])();
	}

	for(i = 0; i < s_r_n; i++)
	{
		*s_r_funcVar[i] = *s_r_mainVar[i] + s_r_v1[i]/2;
	}
	for(i = 0; i < s_r_n; i++)
	{
		t0 = (s_r_tAll[i] + s_r_delayMax - 1) % s_r_delayMax;
		t1 = s_r_tAll[i];
		t2 = (s_r_tAll[i] + 1) % s_r_delayMax;
		t3 = (s_r_tAll[i] + 2) % s_r_delayMax;
		for(j = 0; j < s_r_n; j++)
		{
			tmp = s_r_delayArrayAll[j][t1]+s_r_delayArrayAll[j][t2];
			*s_r_funcDelayVar[j] = (9*tmp - (s_r_delayArrayAll[j][t0]+s_r_delayArrayAll[j][t3]))/16;
		}
		s_r_v2[i] = s_r_dt * (*s_r_fall[i])();
	}

	for(i = 0; i < s_r_n; i++)
	{
		*s_r_funcVar[i] = *s_r_mainVar[i] + s_r_v2[i]/2;
	}
	for(i = 0; i < s_r_n; i++)
	{
		t0 = (s_r_tAll[i] + s_r_delayMax - 1) % s_r_delayMax;
		t1 = s_r_tAll[i];
		t2 = (s_r_tAll[i] + 1) % s_r_delayMax;
		t3 = (s_r_tAll[i] + 2) % s_r_delayMax;
		for(j = 0; j < s_r_n; j++)
		{
			tmp = s_r_delayArrayAll[j][t1]+s_r_delayArrayAll[j][t2];
			*s_r_funcDelayVar[j] = (9*tmp - (s_r_delayArrayAll[j][t0]+s_r_delayArrayAll[j][t3]))/16;
		}
		s_r_v3[i] = s_r_dt * (*s_r_fall[i])();
	}

	for(i = 0; i < s_r_n; i++)
	{
		*s_r_funcVar[i] = *s_r_mainVar[i] + s_r_v3[i];
	}
	for(i = 0; i < s_r_n; i++)
	{
		t2 = (s_r_tAll[i] + 1) % s_r_delayMax;
		for(j = 0; j < s_r_n; j++)
		{
			*s_r_funcDelayVar[j] = s_r_delayArrayAll[j][t2];
		}
		s_r_v4[i] = s_r_dt * (*s_r_fall[i])();
	}

	for(i = 0; i < s_r_n; i++)
	{
		*s_r_mainVar[i] += (s_r_v1[i] + s_r_v2[i] + s_r_v3[i] + s_r_v4[i])/6;
		s_r_delayArrayAll[i][s_r_t] = *s_r_mainVar[i];
	}
	//*px = x + (vx1 + vx2 * 2 + vx3 * 2 + vx4) / 6;
	//s_r_delayArrayX[s_r_t] = *px;
}


void SetRungeKuttaAllDDelay(double x, ...)
{
int i, j;
va_list ap;

	va_start(ap, x);
	for (j = 0; j < s_r_n; j++)
	{
		for (i=0; i< s_r_delayMax;i++)
			s_r_delayArrayAll[j][i] = x;
		x = va_arg(ap, double);
	}
	va_end(ap);
}



/* Runge-Kutta using delay corresponding all variables*/

double **s_r_delayArrayVec = 0;
int *s_r_delayIntVec = 0;
int *s_r_tV = 0;
int s_r_vNum = 0;
int s_r_vCount = 0;
double (**s_r_fVDA)(double*, double**) = 0;

double **s_r_delayA;
double *s_r_vv;
double *s_r_v1, *s_r_v2, *s_r_v3, *s_r_v4;
int *s_r_t0, *s_r_t1, *s_r_t2, *s_r_t3;
double **s_r_vdd;

void SetRungeKuttaVDAG(int vNum, double dt)
{
	s_r_vNum = vNum;
	s_r_dt = dt;
	s_r_fVDA = calloc(vNum, sizeof(double (*)(double*, double**)));
	if (!s_r_fVDA)
		s_OOMError();
	s_r_delayIntVec = MakeArrayInt1(vNum);
	s_r_tV = MakeArrayInt1(vNum);

	s_r_t = 0;
	s_r_vCount = 0;
}

void SetRungeKuttaVDAI(double (*pf)(double*, double**), double delay)
{
int i;

	if (s_r_vCount >= s_r_vNum)
		PrintError("Too many functions");
	
	s_r_fVDA[s_r_vCount] = pf;
    s_r_delayIntVec[s_r_vCount] = delay/s_r_dt;
	if (s_r_delayMax < s_r_delayIntVec[s_r_vCount])
		s_r_delayMax = s_r_delayIntVec[s_r_vCount]; 

	s_r_vCount++;
	
	if (s_r_vCount >= s_r_vNum)
	{
		s_r_delayMax++;
		s_r_delayArrayVec = MakeArrayDouble2(s_r_vNum, s_r_delayMax);
		for (i = 0; i < s_r_vNum; i++)
			s_r_tV[i] = s_r_delayMax - s_r_delayIntVec[i];
		
		s_r_delayA = MakeArrayDouble2(s_r_vCount, s_r_vCount);
		s_r_vv = MakeArrayDouble1(s_r_vCount);
		s_r_v1 = MakeArrayDouble1(s_r_vCount);
		s_r_v2 = MakeArrayDouble1(s_r_vCount);
		s_r_v3 = MakeArrayDouble1(s_r_vCount);
		s_r_v4 = MakeArrayDouble1(s_r_vCount);
		s_r_t0 = MakeArrayInt1(s_r_vCount);
		s_r_t1 = MakeArrayInt1(s_r_vCount);
		s_r_t2 = MakeArrayInt1(s_r_vCount);
		s_r_t3 = MakeArrayInt1(s_r_vCount);

		s_r_vdd = MakeArrayDouble2(s_r_vCount, s_r_vCount);
	}
}


void FreeRungeKuttaVDA(void)
{
	free(s_r_fVDA);
	free(s_r_delayIntVec);
	free(s_r_tV);
	FreeArrayDouble2(s_r_delayArrayVec, s_r_vNum, s_r_delayMax);
	FreeArrayDouble2(s_r_delayA, s_r_vCount, s_r_vCount);
	free(s_r_vv);
	free(s_r_v1);
	free(s_r_v2);
	free(s_r_v3);
	free(s_r_v4);
	free(s_r_t0);
	free(s_r_t1);
	free(s_r_t2);
	free(s_r_t3);
	FreeArrayDouble2(s_r_vdd, s_r_vCount, s_r_vCount);
}


void RungeKuttaVDA(double *v)
{
	in_RungeKuttaVDAC(0, v, 0);
}

void RungeKuttaVDAC(double *v, double dump)
{
	in_RungeKuttaVDAC(1, v, dump);
}

void in_RungeKuttaVDAC(int accelFlag, double *v, double dump)
{
double tmp;
int i, j;
static double *vv;

	if (s_r_vCount < s_r_vNum)
		PrintError("Function Number is not enough\n");
	
	if (!vv)
		vv = calloc(s_r_vNum, sizeof(double));

	/* moved to top so that Set4DOneDlay works */
	s_r_t = (s_r_t+1) % s_r_delayMax;
	for (i = 0; i < s_r_vNum; i++)
	{
		s_r_tV[i] = (s_r_tV[i] + 1) % s_r_delayMax;
	}

	for (i = 0; i < s_r_vNum; i++)
		for (j = 0; j < s_r_vNum; j++)
		{
			s_r_delayA[i][j] = s_r_delayArrayVec[i][s_r_tV[j]];
		}
	
	for (i = 0; i < s_r_vNum; i++)
	{
		s_r_v1[i] = s_r_dt * (*(s_r_fVDA[i]))(v, s_r_delayA);
	}
	for (i = 0; i < s_r_vNum; i++)
	{
		s_r_vv[i] = v[i] + s_r_v1[i]/2;
	}
	for (i = 0; i < s_r_vNum; i++)
	{
		s_r_t0[i] = (s_r_tV[i] + s_r_delayMax - 1) % s_r_delayMax;
		s_r_t1[i] = s_r_tV[i];
		s_r_t2[i] = (s_r_tV[i] + 1) % s_r_delayMax;
		s_r_t3[i] = (s_r_tV[i] + 2) % s_r_delayMax;
	}

	for (i = 0; i < s_r_vNum; i++)
		for (j = 0; j < s_r_vNum; j++)
		{
			tmp = s_r_delayArrayVec[i][s_r_t1[j]]+s_r_delayArrayVec[i][s_r_t2[j]];
			s_r_vdd[i][j] = (9*tmp - (s_r_delayArrayVec[i][s_r_t0[j]]
								+s_r_delayArrayVec[i][s_r_t3[j]]))/16;
		}

	for (i = 0; i < s_r_vNum; i++)
	{
		s_r_v2[i] = s_r_dt * (*(s_r_fVDA[i]))(s_r_vv, s_r_vdd);
	}

	for (i = 0; i < s_r_vNum; i++)
	{
		s_r_vv[i] = v[i] + s_r_v2[i]/2;
	}

	for (i = 0; i < s_r_vNum; i++)
	{
		s_r_v3[i] = s_r_dt * (*(s_r_fVDA[i]))(s_r_vv, s_r_vdd);
	}

	for (i = 0; i < s_r_vNum; i++)
	{
		s_r_vv[i] = v[i] + s_r_v3[i];
	}

	for (i = 0; i < s_r_vNum; i++)
		for (j = 0; j < s_r_vNum; j++)
		{
			s_r_vdd[i][j] = s_r_delayArrayVec[i][s_r_t2[j]];
		}


	for (i = 0; i < s_r_vNum; i++)
	{
		s_r_v4[i] = s_r_dt * (*(s_r_fVDA[i]))(s_r_vv, s_r_vdd);
	}

	for (i = 0; i < s_r_vNum; i++)
	{
		if (accelFlag)
		{
			vv[i] = vv[i]*dump + (s_r_v1[i] + 2 * s_r_v2[i] + 2 * s_r_v3[i] + s_r_v4[i]) / 6;
			v[i] += vv[i];
		}
		else
			v[i] += (s_r_v1[i] + 2 * s_r_v2[i] + 2 * s_r_v3[i] + s_r_v4[i]) / 6;
	}

	for (i = 0; i < s_r_vNum; i++)
	{
        s_r_delayArrayVec[i][s_r_t] = v[i];
	}
}

void SetRungeKuttaVDDelay(double *v)
{
int i, j;

	for (i=0; i< s_r_vNum;i++)
		for (j=0; j< s_r_delayMax;j++)
			s_r_delayArrayVec[i][j] = v[i];
}




/*** RungeKuttaA3 was not used***

int s_r_n1, s_r_n2, s_r_n3;

double (*s_r_fa3d)(int,int,int,double***,double);

void SetRungeKuttaA3D(double (*pf)(int,int,int,double***,double),
                      double dt,int n1,int n2,int n3)
{
        s_r_fa3d = pf;
        s_r_dt = dt;
        s_r_n1 = n1;
        s_r_n2 = n2;
        s_r_n3 = n3;
}

void RungeKuttaA3S1(double*** *px,double* *pd)
{
double  ***x, ***vx1, ***vx2, ***vx3, ***vx4, ***xx;
double  *d;
int i1,i2,i3;

        x = *px;
        d = *pd;
        
#define xvec x[i1][i2][i3]
#define xxvec xx[i1][i2][i3]
#define vx1vec vx1[i1][i2][i3]
#define vx2vec vx2[i1][i2][i3]
#define vx3vec vx3[i1][i2][i3]
#define vx4vec vx4[i1][i2][i3]
#define pxvec px[i1][i2][i3]
#define forvec for(i1=0;i1<s_r_n1;i1++)for(i2=0;i2<s_r_n2;i2++)for(i3=0;i3<s_r_n3;i3++) 
        forvec
                vx1vec = s_r_dt * (*s_r_fa3d)(i1,i2,i3,x,d[0]);
        forvec
                xxvec = xvec + vx1vec / 2;
        forvec
                vx2vec = s_r_dt * (*s_r_fa3d)(i1,i2,i3,xx,d[0]);
        forvec
                xxvec = xvec + vx2vec / 2;
        forvec
                vx3vec = s_r_dt * (*s_r_fa3d)(i1,i2,i3,xx,d[0]);
        forvec
                xxvec = xvec + vx3vec;
        forvec
                vx4vec = s_r_dt * (*s_r_fa3d)(i1,i2,i3,xx,d[0]);
        forvec
                *pxvec = xvec + (vx1vec + vx2vec * 2 + vx3vec * 2 + vx4vec) / 6;
}
*/

#endif /*pragma once*/

