/*
* RungeKutta.h
* Version 1.8 / 000803
* by Satoshi Takahashi, Yumiko Takahashi & Kayo Yasufuku
*/


#ifndef RUNGEKUTTA_H /*pragma once*/
#define RUNGEKUTTA_H

#include <s_Config.h>


void SetDt(double dt);
void SetRungeKutta1(double (*pf)(double),double dt);
void RungeKutta1(double *px);
void SetRungeKutta1T(double (*pf)(double,double),double dt);
void RungeKutta1T(double *px,double *pt);
void SetRungeKutta2(	double (*pfx)(double x,double y), 
											double (*pfy)(double x,double y),
											double dt);
void RungeKutta2(double *px,double *py);
void SetRungeKutta2T(	double (*pfx)(double x,double y,double t), 
											double (*pfy)(double x,double y,double t),
											double dt);
void RungeKutta2T(double *px,double *py,double *pt);
void SetRungeKutta3(	double (*pfx)(double x,double y,double z), 
											double (*pfy)(double x,double y,double z),
											double (*pfz)(double x,double y,double z),
											double dt);
void RungeKutta3(double *px,double *py,double *pz);
void SetRungeKutta3T( double (*pfx)(double x,double y,double z,double t), 
											double (*pfy)(double x,double y,double z,double t),
											double (*pfz)(double x,double y,double z,double t),
											double dt);
void RungeKutta3T(double *px,double *py,double *pz,double *pt);
void SetRungeKutta4(	double (*pfx)(double x,double y,double z,double u), 
											double (*pfy)(double x,double y,double z,double u),
											double (*pfz)(double x,double y,double z,double u),
											double (*pfu)(double x,double y,double z,double u),
											double dt);
void RungeKutta4(double *px,double *py,double *pz,double *pu);
void SetRungeKutta4T( double (*pfx)(double x,double y,double z,double u,double t), 
											double (*pfy)(double x,double y,double z,double u,double t),
											double (*pfz)(double x,double y,double z,double u,double t),
											double (*pfu)(double x,double y,double z,double u,double t),
											double dt);
void RungeKutta4T(double *px,double *py,double *pz,double *pu,double *pt);

void SetRungeKutta6(	double (*pfx)(double x,double y,double z,double s,double t,double u), 
											double (*pfy)(double x,double y,double z,double s,double t,double u),
											double (*pfz)(double x,double y,double z,double s,double t,double u),
											double (*pfs)(double x,double y,double z,double s,double t,double u),
											double (*pft)(double x,double y,double z,double s,double t,double u),
											double (*pfu)(double x,double y,double z,double s,double t,double u),
											double dt);
void RungeKutta6(double *px,double *py,double *pz,double *ps,double *pt,double *pu);

void SetRungeKuttaV(int n,...);
void RungeKuttaV(double *px);

void SetRungeKutta1D(double (*pf)(double,double),double dt,double delay);
void RungeKutta1D(double *px);
void SetRungeKutta1DDelay(double x);
void SetRungeKutta1DOneDelay(double x);
void SetRungeKutta2D(	double (*pfx)(double,double,double,double), 
											double (*pfy)(double,double,double,double),
											double dt,double delayX,double delayY);
void RungeKutta2D(double *px,double *py); 
void SetRungeKutta2DDelay(double x, double y);
void SetRungeKutta2DOneDelay(double x, double y);
void SetRungeKutta3D(	double (*pfx)(double,double,double,double,double,double), 
											double (*pfy)(double,double,double,double,double,double),
											double (*pfz)(double,double,double,double,double,double),
											double dt,double delayX,double delayY,double delayZ);
void RungeKutta3D(double *px,double *py,double *pz); 
void SetRungeKutta3DDelay(double x, double y, double z);
void SetRungeKutta3DOneDelay(double x, double y, double z);
void SetRungeKutta4D(	double (*pfx)(double,double,double,double,double,double,double,double), 
											double (*pfy)(double,double,double,double,double,double,double,double),
											double (*pfz)(double,double,double,double,double,double,double,double),
											double (*pfu)(double,double,double,double,double,double,double,double),
											double dt,double delayX,double delayY,double delayZ,double delayU);
void RungeKutta4D(double *px,double *py,double *pz,double *pu); 
void SetRungeKutta4DDelay(double x, double y, double z, double u);
void SetRungeKutta4DOneDelay(double x, double y, double z, double u);

void SetRungeKutta6D(double (*pfx)(double,double,double,double,
			double,double,double,double,double,double,double,double),
			double (*pfy)(double,double,double,double,
			double,double,double,double,double,double,double,double),
			double (*pfz)(double,double,double,double,
			double,double,double,double,double,double,double,double),
			double (*pfu)(double,double,double,double,
			double,double,double,double,double,double,double,double),
			double (*pfv)(double,double,double,double,
			double,double,double,double,double,double,double,double),
			double (*pfw)(double,double,double,double,
			double,double,double,double,double,double,double,double),
			double dt,double delayX,double delayY,
			double delayZ,double delayU,double delayV,double delayW);
void RungeKutta6D(double *px,double *py,double *pz,double *pu,double *pv,double *pw);
void SetRungeKutta6DDelay(double x, double y, double z, double u, double v, double w);

/* Runge-Kutta All Number */
void SetRungeKuttaAll(int n, ...);
void RungeKuttaAll(void);
void SetRungeKuttaAllD(int n, ...);
void RungeKuttaAllD(void);
void SetRungeKuttaAllDDelay(double x, ...);


void SetRungeKuttaVDAG(int vNum, double dt);
void SetRungeKuttaVDAI(double (*pf)(double*, double**),double delay);
void FreeRungeKuttaVDA(void);
void RungeKuttaVDA(double *v);
void RungeKuttaVDAC(double *v, double dump);
void in_RungeKuttaVDAC(int accelFlag, double *v, double dump);
void SetRungeKuttaVDDelay(double *v);


#if S_UNIX
#include <RungeKutta.c>
#endif

#endif /*pragma once*/

