/*
 *  Save.c
 *  Version 1.5.1 / 980117
 *  by Satoshi Takahashi & Akiya Iwasaki
 */


#ifndef SAVE_C /*pragma once*/
#define SAVE_C
#include <s_Config.h>

#include <stdio.h>
#include <string.h>

#if !S_CARBON && !S_UNIX
#include <QDOffScreen.h>
#endif

#include <Save.h>
#include <s_Drawing.h>
#include <StartSCL.h>
#include <s_Common.h>
#include <s_RedrawFunc.h>
#include <s_undef.c>

#if S_UNIX /* as s_UNIX.h is not in parts-main */
#include <s_UNIX.h>
#endif

#if !S_UNIX /******************** !!!! UNIX ********************/

#include <s_QDGlobal.h>
#include <DialogFOpen.h>


/***** Definition of Variables and Functions *****/

#if defined(NO_COLOR_DRAWING) || defined(NO_REDRAW)
#define SetGWorld(a,b) SetPort((GrafPtr)a)
#endif

#endif /*!S_UNIX*/ /******************** UNIX ********************/

#include <VarBody.h>
#include <Save-v.c>


void BeginSaving(void)
{
#if !S_UNIX
GrafPtr currPort;
GDHandle currDev;

	if (s_save.openPictureFlag)
		PrintError("Error: Consecutive call of BeginSaving or BeginPrinting\n");
	
	s_GetGWorld((GrafPtr*)&currPort,&currDev);
	s_SetGWorld(GetWindowPort(s_drawingWindow),nil);
	if (s_save.pict) KillPicture(s_save.pict);
	s_save.pict = 0;
	ClipRect(SCWindowBounds(s_drawingWindow));
	s_save.pict = OpenPicture(SCWindowBounds(s_drawingWindow));
	s_save.openPictureFlag = 1;
	s_ShowPen();
	s_SetGWorld(currPort,currDev);
	CheckPause();
#else/*!S_UNIX*/
	if (s_UNIX.drawingFlag)
		fprintf(s_UNIX.drawingFile, "BeginSaving();\n");
#endif
}


void EndSaving(void) 
{
#if !S_UNIX
GrafPtr currPort;
GDHandle currDev;

	s_GetGWorld((GrafPtr*)&currPort,&currDev);
	s_SetGWorld(GetWindowPort(s_drawingWindow),nil);
	s_HidePen();
	ClosePicture();
	s_save.openPictureFlag = 0;
	SavePict(s_save.pict);
	s_SetGWorld(currPort,currDev);
	CheckPause();
#else/*!S_UNIX*/
char command[50];

	if (s_UNIX.drawingFlag)
	{
		fprintf(s_UNIX.drawingFile, "EndSaving();\n");
		if (s_Config.XFlag)
		{
			sprintf(command, "xwd -id %d>SCLXDrawing", thePort->win);
			system(command);
		}
	}
#endif
}


#if !S_UNIX
void SetDrawingCreator(OSType c)
{
	s_save.drawCreator = c;
}
#endif

#if !S_UNIX /******************** !!!! UNIX ********************/

extern GDHandle s_display;

void SavePict(PicHandle lPict)
{
short	globalRef;
OSErr	err;
int		i;
Point	wher;
long	longCount, hlen=512L;
FSSpec fss;
Boolean good;
char	hbuf[512];

#ifdef PICT_CREATOR
	s_drawingCreator=PICT_CREATOR;
#endif /* defPICT_CREATOR */
	for (i=0;i<512;i++)
		hbuf[i]=0;
	wher.h = 20; wher.v = 20;
	if (s_save.bmColor)
	{
		fss = s_save.bmFss;
        good = s_save.bmGood;
	}
	else
	{
		SCLPutFile("drawing", 'PICT', &fss, &good);
	}
	if (good)
	{
		FSpDelete(&fss);
		err = FSpCreate(&fss, s_save.drawCreator, 'PICT', smSystemScript);
		if ((err == noErr) || (err == dupFNErr))
		{
			FSpOpenDF(&fss, fsCurPerm, &globalRef);
			longCount = GetHandleSize((Handle)lPict);
			SetFPos(globalRef, fsFromStart, 0L);
			FSWrite(globalRef, &hlen, hbuf);
			SetFPos(globalRef, fsFromStart, 512L);
			FSWrite(globalRef, &longCount, *lPict);
			FSClose(globalRef);
			FlushVol(nil, fss.vRefNum);
		}
	}
}


#endif /*!S_UNIX*/ /******************** UNIX ********************/

#if !S_UNIX
void SetBitMapCreator(OSType c)
{
	s_save.bmCreator = c;
}
#endif



void BeginBitMapSaving()
{
#if !S_UNIX
	SCLPutFile("bitmap", nil, &s_save.bmFss, &s_save.bmGood);
	CheckPause();
#else /*!S_UNIX*/
	if (s_UNIX.drawingFlag)
		fprintf(s_UNIX.drawingFile, "BeginBitMapSaving();\n");
#endif /*else !S_UNIX*/
}

void EndBitMapSavingAuto()
{
#if !S_UNIX
	NumToString(s_save.bmNum++, s_save.bmFss.name);
	EndBitMapSaving();
	CheckPause();
#else /*!S_UNIX*/
char command[50];

	if (s_UNIX.drawingFlag)
	{
		fprintf(s_UNIX.drawingFile, "EndBitMapSavingAuto();\n");
		if (s_Config.XFlag)
		{
			sprintf(command,"xwd -id %d>SCLXDrawing%d", thePort->win, s_save.bmNum);
			system(command);
		}
	}
	s_save.bmNum++;
#endif /*else !S_UNIX*/
}

void EndBitMapSaving()
{
#if !S_UNIX
	if(s_colorFlag)
	{
		EndBitMapSavingColor();
	}
	else
	{
		EndBitMapSavingMono();
	}
	CheckPause();
#else /*!S_UNIX*/
char command[50];

	if (s_UNIX.drawingFlag)
	{
		fprintf(s_UNIX.drawingFile, "EndBitMapSaving();\n");
		if (s_Config.XFlag)
		{
			sprintf(command, "xwd -id %d>SCLXDrawing", thePort->win);
			system(command);
		}
	}
#endif /*else !S_UNIX*/
}

void EndBitMapSavingColor()
{
#if !S_UNIX
GWorldPtr currPort;
GDHandle currDev;

	s_save.bmColor = 1;
	BeginSaving();

	s_GetGWorld((GrafPtr*)&currPort,&currDev);
    if (s_drawingWindow)
        s_SetGWorld(GetWindowPort(s_drawingWindow), nil);
	ForeColor(blackColor);
	BackColor(whiteColor);
//		LockPixels (((CGrafPtr)s_drawingWindow)->portPixMap);
	CopyBits(GetWindowBits(s_drawingWindow), GetWindowBits(s_drawingWindow),
		SCWindowBounds(s_drawingWindow), SCWindowBounds(s_drawingWindow), srcCopy, 0);
	s_SetGWorld((GrafPtr)currPort,currDev);

	EndSaving();
	s_save.bmColor = 0;
	CheckPause();
#else /*!S_UNIX*/
	if (s_UNIX.drawingFlag)
		fprintf(s_UNIX.drawingFile, "EndBitMapSavingColor();\n");
#endif /*else !S_UNIX*/
}
	
void EndBitMapSavingMono()
{
#if !S_UNIX
short	fRef;
OSErr	err;
int 	i;
long	len,j;
Ptr			sptr,dptr,hd;
BitMap	vBitMap;

#ifdef BITMAP_CREATOR
	s_save.bmCreator=BITMAP_CREATOR;
#endif /* defBITMAP_CREATOR */
	if (s_save.bmGood)
	{
		hd = NewPtr( 52352L );
		vBitMap.baseAddr = NewPtr(51840L);
		vBitMap.rowBytes = 72;
		SetRect(&vBitMap.bounds,0,0,72*8,720);
		sptr=vBitMap.baseAddr;
		for(j=0;j<51840L;j++)
			*sptr++=0;
		CopyBits(GetWindowBits(s_drawingWindow), &vBitMap,
			SCWindowBounds(s_drawingWindow), SCWindowBounds(s_drawingWindow),
            srcCopy, nil);
		sptr = vBitMap.baseAddr;
		dptr = hd;
		for( i=0;i<512;i++ )
			*dptr++=0;
		for( i=0; i<720; i++ )
			PackBits( &sptr,&dptr,72 );
		len=dptr-hd;
		FSpDelete(&s_save.bmFss);
		err = FSpCreate(&s_save.bmFss, s_save.bmCreator, 'PNTG', smSystemScript);
		if ((err == noErr) || (err == dupFNErr))
		{
			FSpOpenDF(&s_save.bmFss, fsCurPerm, &fRef);
			
			FSWrite( fRef, &len, hd );
			
			FSClose(fRef);
			FlushVol(nil, s_save.bmFss.vRefNum);
		}
		DisposePtr(vBitMap.baseAddr);
		DisposePtr( hd );
	}
	CheckPause();
#else /*!S_UNIX*/
	if (s_UNIX.drawingFlag)
		fprintf(s_UNIX.drawingFile, "EndBitMapSavingMono();\n");
#endif /*else !S_UNIX*/
}

#if !S_UNIX
void SetTextCreator(OSType c)
{
	s_save.textCreator = c;
}
#endif

short BeginTextSaving(void)
{
#if !S_UNIX

OSErr	err;

#else

char filename[256] = "text";
int fileNameLength = 0;

#endif

#if !S_UNIX

#ifdef TEXT_CREATOR
	s_save.textCreator = TEXT_CREATOR;
#endif /* defTEXT_CREATOR */
	SCLPutFile("text", s_save.textCreator, &s_save.textFss, &s_save.textGood);
	if (s_save.textGood)
	{
		FSpDelete(&s_save.textFss);
		err=FSpCreate(&s_save.textFss, s_save.textCreator, 'TEXT', smSystemScript);
		if ((err == noErr) || (err == dupFNErr))
		{
			if(FSpOpenDF(&s_save.textFss, fsCurPerm, &s_save.textFRef)==noErr)
			{
				s_textSaveFlag=1;
				SaveTextP = &SaveText;
				SetFPos(s_save.textFRef, fsFromStart, 0L);
				CheckPause();
				return(s_save.textFRef);
			}
		}
	}
	CheckPause();
	return -1;

#else /*!S_UNIX*/

	if (s_Config.foreground)
	{
		printf("Enter file name for saving text, type 'c' or 'n' to cancel: ");
		fgets(filename, 256, stdin);

		fileNameLength = strlen(filename);
		if (fileNameLength && ( filename[fileNameLength-1] == '\n' ))
			filename[fileNameLength-1] = 0;
		
		if ( CancelName(filename) )
			return -2;
		
		if ( (filename[0] == '\n') || (!filename[0]))
			strcpy(filename, "text");
	}	
	
	if ( s_save.textSaveFile = fopen(filename, "w") )
	{
		s_textSaveFlag=1;
		SaveTextP = &SaveText;
		return 0;
	}
	
	return -1;	

#endif /*else !S_UNIX*/
}

void EndTextSaving(void)
{
	if (s_textSaveFlag)
	{
#if !S_UNIX
		FSClose(s_save.textFRef);
		FlushVol(nil,s_save.textFss.vRefNum);
#else /*!S_UNIX*/
		fclose(s_save.textSaveFile);
#endif /*else !S_UNIX*/

		s_textSaveFlag=0;
		SaveTextP=&s_longDummyString;
	}

#if !S_UNIX
	CheckPause();
#endif

}

long SaveText(char *s)
{
long l;

#if !S_UNIX

	l = strlen(s);
	FSWrite(s_save.textFRef, &l, s);

#else

	l = fputs(s, s_save.textSaveFile);

#endif

	return l;
}

#if !S_UNIX /******************** !!!! UNIX ********************/

short BeginTextReading(void)
{
	textFileEnd=0;
	SCLGetFile('TEXT', &s_save.textFss, &s_save.textGood);
	if (s_save.textGood)
	{
		if(FSpOpenDF(&s_save.textFss, fsCurPerm, &s_save.textReadFRef)==noErr)
		{
			s_textReadFlag=1;
			ReadTextP = &ReadText;
			SetFPos(s_save.textReadFRef, fsFromStart, 0L);
			CheckPause();
			return(s_save.textReadFRef);
		}
	}
	CheckPause();
	return(-1);
}

void EndTextReading(void)
{

	FSClose(s_save.textReadFRef);
	FlushVol(nil, s_save.textFss.vRefNum);
	s_textReadFlag=0;
	ReadTextP = &s_longDummyString;
	textFileEnd=0;
	CheckPause();
}

long ReadText(char *s)
{
long l=1L;

	FSRead(s_save.textReadFRef, &l, (Ptr)s);
	return l;
}

#include <s_define.c>

#if defined(NO_COLOR_DRAWING) || defined(NO_REDRAW)
#undef SetGWorld(a,b)
#endif

#endif /*!S_UNIX*/ /******************** UNIX ********************/
#endif /*pragma once*/
