/*
 *  StartSCL.h
 *  Version 2.0 / 010910
 *  by Satoshi Takahashi & Akiya Iwasaki
 */


#ifndef STARTSCL_H /*pragma once*/
#define STARTSCL_H

#ifndef SCLLocal
#include <s_Config.h>
#else
#include "s_Config.h"
#endif

#ifndef SCL_USE_WINDOW
#define SCL_USE_WINDOW 1
#endif

#define S_SATOSHI

#include <stdio.h>
#include <stdarg.h>

#if SCL_USE_WINDOW
#ifndef SCLLocal
#include <s_Drawing.h>
#else
#include "s_Drawing.h"
#endif
#endif

#if S_UNIX
#include <s_UNIX.h>
#endif

#ifndef S_WNTIME /*sleep time for WaitNextEvent*/
#define S_WNTIME 20
#endif

typedef struct s_ssType
{
	int startFlag;
	int initDrawingFlag;
#if S_CARBON
    int drawCount;
    int directDraw;
#endif
}	s_ssType;

#if S_CARBON
#define DRAW_PERIOD 0
#endif

extern s_ssType s_ss;

#if !S_UNIX
#if SCL_USE_WINDOW
extern GDHandle s_display;
extern int s_redrawFlag;
extern short s_MyResFile;
#endif
#endif

/***** Initialize and Stop *****/

#if !S_UNIX
void InitToolbox(void);
#endif

#if SCL_USE_WINDOW
void Start1(void);
#endif

void MinimumStart(void); /* use for no window applications like filter */


#if SCL_USE_WINDOW
void Start(void);

void s_NoRedrawInitDrawing(void);

void StartNoRedraw(void);

#ifdef NO_REDRAW
#define Start() StartNoRedraw()
#endif /* ndef NO_REDRAW */

void StartColor(void);
void StartRedraw(void);
void StartBW(void);

void StartText(void);
void StartDrawing(void);

void StartTextOnly(void);

void StartDrawingOnly(void);
void StartDrawingOnlyColor(void);
void StartColorDrawingOnly(void);

#if S_CARBON
void SetDirectDrawOn(void);
void SetDirectDrawOff(void);
#endif
#endif /*SCL_USE_WINDOW*/

#if !S_UNIX 
extern	EventRecord s_event;
#endif /*!S_UNIX*/

#if SCL_USE_WINDOW
extern int s_colorFlag;


void Stop(void);


extern Str255 s_fontName;
extern short s_fontSize;

#if !S_UNIX /******************** !!!! UNIX ********************/
/***** Setting Fonts Before Start() *****/

#define SetFontName SetStartFont
void SetStartFont(char* s);
#define SetStartSize(i) (s_fontSize=(i))
#define SetFontSize(i) (s_fontSize=(i))

/***** Setting Bounds Before Start() *****/

#endif /*!S_UNIX*/ /******************** UNIX ********************/
void SetDrawingBounds(int l,int t,int r,int b);

#if !S_UNIX /******************** !!!! UNIX ********************/
void SetTextBounds(int l,int t,int r,int b);

#define SetTextDrawingBound(i) \
(s_windowBounds.left=i+1,s_textBounds.right=i)

void SetGoodBounds(void);
#endif /*!S_UNIX*/ /******************** UNIX ********************/

void SetDrawingBoundsFull(void);

#define SetScale(scale) (s_d_scale=(scale))

#if !S_UNIX /******************** !!!! UNIX ********************/
#define PSet(x,y) (s_MoveTo(x,y),LineTo(x,y))
#else
void PSet(short x, short y);
#endif /*else !S_UNIX*/

void SetFont(char *fontName);
#define DrawCString(s) DrawText(s,0,strlen(s))

#if !S_UNIX /******************** !!!! UNIX ********************/
/***** Setting Window Bounds from Resource *****/

void SetWindowBoundsResource(void);
#define UseWindowBoundsResource() (s_windowResFlag=1)
#endif /*!S_UNIX*/ /******************** UNIX ********************/



/***** Text *****/

extern	Rect			s_textBounds;

void SetTextBoundsFull(void);

void SetTextCharNum(int n);

int	s_printf(char const *p, ...);
int s_vprintf(const char *p, va_list ap);
int	s_fprintf(FILE *fp, char const *p, ...);

#if !S_UNIX
int	s_sprintfModifyLF(char *string, char const *p, ...);
void LFToF(char const *inFormat, char *outFormat);
int s_puts(const char *str);
#endif


extern int echoInputFlag;

#if !S_UNIX
int	s_scanf(const char* p,...);
#else /*!S_UNIX*/
#define s_scanf scanf
#endif /*else !S_UNIX*/
char* s_fgets(char* s,int n, FILE* stream);
int s_getchar(void);
int s_fflush(FILE *stream);

#if !S_UNIX /******************** !!! UNIX ********************/

extern	WindowPtr	s_textWindow, s_inputWindow;
extern	TEHandle	s_textH, s_inputH;
extern	char			s_inputChar;
extern	Rect			s_textRect,	s_inputRect;

void InitText(void);

#endif /*!S_UNIX*/ /******************** UNIX ********************/

#if !S_UNIX
#define ShowText() (ShowWindow(s_textWindow),SelectWindow(s_textWindow))
#else
#define ShowText()
#endif

#if !S_UNIX
#define HideText() HideWindow(s_textWindow)
#else
#define HideText() (0)
#endif

void EraseText(void);

#if !S_UNIX /******************** !!! UNIX ********************/
#define TextWindowWidth()	(SCWindowBounds(s_textWindow)->right)
#define TextHeight()	(SCWindowBounds(s_textWindow)->bottom)


extern	WindowPtr	s_inputWindow;
extern	Rect			s_inputBounds;

#define ShowInput() ShowWindow(s_inputWindow);SelectWindow(s_inputWindow)

#define HideInput() HideWindow(s_inputWindow)

#define EraseInput() \
(SetPort(s_inputWindow),EraseRect(&s_inputWindow->portRect),\
SetPort(s_drawingWindow))

#define SetInputBounds(l,t,r,b) \
(SetRect(&s_inputBounds,l,t,r,b),s_i_set=1,s_i_save=1)
#endif /*!S_UNIX*/ /******************** UNIX ********************/


/*
 * s_textSaveFlag, s_textPrintFlag, SaveText() and PrintText()
 * is used only with satoshi2.h
 */

extern Boolean s_textSaveFlag;
extern Boolean s_textPrintFlag;
extern Boolean s_textReadFlag;
extern int textFileEnd;

#if S_CARBON
extern Pattern s_CarbonQDPat;
#endif


extern long (*SaveTextP)(char *s);
extern long (*ReadTextP)(char *s);
extern void (*PrintTextP)(char* s);


/***** Text and Drawing *****/


void ChangeWindowBoundary(void);

/***** Events *****/

#define KeyPressed() KeyPressedFast()
#define MousePressed() MousePressedFast()

#if !S_UNIX
#define stop_ending Stop /* for compatibility to older version */
#define KeyPressedFast() GetNextEvent(keyDownMask,&s_event)
#define MousePressedFast() GetNextEvent(mDownMask,&s_event)
#else /*!S_UNIX*/
#if 0
#define KeyPressed() \
(s_UNIX.sec=time(0), s_UNIX.stopSec > s_UNIX.sec ? \
(0) : (s_UNIX.stopSec = s_UNIX.sec + s_UNIX.stopInterval, \
printf("Continue ? (y/n) "), \
fgets(s_UNIX.ans,256,stdin), \
(s_UNIX.ans[0]=='y'?(0):(1))))
#endif /*0*/
#define KeyPressedFast() KeyPressedCheckPause()
#define MousePressedFast() MousePressedCheckPause()
#endif /*else !S_UNIX*/
#endif /*SCL_USE_WINDOW*/

int MousePressedCheckPause();
int KeyPressedCheckPause();
void WaitTick(long tick);
void CheckPause(void);

#if SCL_USE_WINDOW
void SetKeyPressedTime(unsigned long t);
#define SetMousePressedTime SetKeyPressedTime

#if !S_UNIX /******************** !!!! UNIX ********************/
void s_Update(void);

extern int s_updateFlag;
void SetUpdateOff(void);
#endif /*!S_UNIX*/ /******************** UNIX ********************/


/***** Miscellaneus *****/


#ifndef max
#define max(a,b) (((a)>(b))?(a):(b))
#endif

#ifndef min
#define min(a,b) (((a)>(b))?(b):(a))
#endif


#define CopyRect(a,b) \
((b).top=(a).top,(b).left=(a).left,(b).bottom=(a).bottom,(b).right=(a).right)

#if !S_UNIX /******************** !!!! UNIX ********************/
void s_debug(char* s);
#endif /*!S_UNIX*/ /******************** UNIX ********************/

int s_Dprintf(short x, short y, char *f, ...);
#define s_DPrintf s_Dprintf
#define s_DPrintF s_Dprintf

void s_srand(unsigned int seed);

/***** Capitalize *****/

#define start Start
#define starttext StartText
#define startdrawing StartDrawing
#define stop Stop
#define moveto MoveTo
#define Moveto MoveTo
#define lineto LineTo
#define Lineto LineTo
#define move Move
#define line Line
#define pset PSet
#define rect Rect
#define setrect SetRect

#if !S_UNIX /******************** !!!! UNIX ********************/
#define showtext ShowText
#define showdrawing ShowDrawing
#define settextbounds SetTextBounds
#define setdrawingbounds SetDrawingBounds
#define keypressed KeyPressed
#define mousepressed MousePressed
#define Pause CheckPause
#define pause CheckPause
#define eventrecord EventRecord
#define getnextevent GetNextEvent
#endif /*!S_UNIX*/ /******************** UNIX ********************/

#ifndef SCLLocal
#include <s_define.c>
#else
#include "s_define.c"
#endif

#if !S_UNIX
#define SetBackSpace()
#endif /*!S_UNIX*/

#endif /*SCL_USE_WINDOW*/

#endif /*pragma once*/
