/*
 *  s_Script.c
 *  Version 2.0 / 010912
 *  by Satoshi Takahashi
 */

#ifndef S_SCRIPT_C /*pragma once*/
#define S_SCRIPT_C

#include <s_Config.h>

#if S_CARBON
#include <Carbon/Carbon.h>
#endif

#include <stdio.h>
#include <ctype.h>

#include <FuncList.h>
#include <s_File.h>
#include <DragAndDrop.h>
#include <DialogFOpen.h>
#include <s_Process.h>
#include <s_Event.h>
#include <s_Common.h>

#if DEBUG_SCRIPT
#include <StartSCL.h>
#endif


#include <s_Script.h>
#include <VarBody.h>
/*#include <s_Script-v.c>*/


/* correspondence between command string and function */

FuncList s_scriptFunc =
{
	{"cp",	DoFileCopy},
#if !S_CARBON || 1
	{"update", DoUpdate},
#endif
	{"mv",	DoFileMove},
	{"mvr",	DoReverseFileMove},
	{"launch", DoLaunch},
	{"launchopen", DoLaunchOpen},
	{"kill", DoKill},
#if !S_CARBON
	{"killold", DoKillOld},
#endif
	{"touch", DoTouch},
	{"rmdir", DoRemoveDirectory},
#if !S_CARBON
	{"key", DoKeyDown},
	{"mouse", DoMouseDown},
	{"mouseup", DoMouseUp},
#endif
	{"sleep", DoSleep}, /*not meaningful*/
	{"quit", DoQuit},
	{"scriptcreator", DoScriptCreator},

	{"#", DoComment},

	{0,	0} /* must be at last */
};


/************************************************************
	void void Script(void)
	Call from main() and do all the work of script.
************************************************************/

void Script(void)
{
FILE *inputFile;

	if ( DDCount() )
		inputFile = DDFOpen("input", "r");
	else
		inputFile = DialogFOpen("input", "r");
	
	if(inputFile)
		ScriptLoop(inputFile);
}


/************************************************************
	void ScriptLoop(File *inputFile)
	read commands from input file and execute
	line by line.
************************************************************/

void ScriptLoop(FILE *inputFile)
{
char oneLine[1024];
char commandWord[256];
voidStringFunc *commandFunc;


	//SEMouseDown(1, 1); /* first mouse event always picked up as the first event */
	//SEMouseUp(1, 1); /* first mouse event always picked up as the first event */
		
	while( fgets(oneLine, 1024, inputFile) )
	{
#if DEBUG_SCRIPT
s_printf("oneLine: %s\n",oneLine);
#endif
		sscanf(oneLine, "%s", commandWord);
		if ( commandFunc = FindFunc(s_scriptFunc, commandWord) )
		{
#if DEBUG_SCRIPT
s_printf("commandWord: %s\n",commandWord);
#endif
			(*commandFunc)(oneLine); /*commandFunc may be Null */
		}
	}
}


/************************************************************
	DoFileCopy(inputLine);
	with inputLine
	cp "aVol:bDir:cFile" "dVol:edir:fFile"
	copies aVol:bDir:cFile to dVol:edir:fFile
*************************************************************/

void DoFileCopy(char *inputLine)
{
char command[16]="", sourceFile[256]="", destFile[256]="";
int inputNum;

	inputNum = sscanf(inputLine, "%s \"%[^\"]\" \"%[^\"]\"",
							command, sourceFile, destFile);
	
#if DEBUG_SCRIPT
s_printf("source: %s\ndest: %s\n", sourceFile, destFile);
#endif
	if (inputNum != 3)
	{
		return;
	}
	SFFileCopy(sourceFile, destFile);
}


/************************************************************
	void DoUpdate(char *inputLine)
	with inputLine
	update "aVol:bDir:cFile" "dVol:edir:fFile"
	copies aVol:bDir:cFile to dVol:edir:fFile
	only if aVol:bDir:cFile is newly modified
*************************************************************/

#if !S_CARBON || 1
void DoUpdate(char *inputLine)
{
char command[16]="", sourceFile[256]="", destFile[256]="";
int inputNum;

#if DEBUG_SCRIPT
	printf("DoUpdate\n");
#endif
	inputNum = sscanf(inputLine, "%s \"%[^\"]\" \"%[^\"]\"",
							command, sourceFile, destFile);
	
	if (inputNum != 3)
	{
		return;
	}
	SFUpdate(sourceFile, destFile);
}
#endif

/************************************************************
	DoFileMove(inputLine);
	with inputLine
	mv "aVol:bDir:cFile" "dVol:edir:fFile"
	mv "aVol:bDir:cFile" "dVol:edir"
	move aVol:bDir:cFile into dVol:edir:
*************************************************************/

void DoFileMove(char *inputLine)
{
char command[16]="", sourceFile[512]="", destFile[512]="";
int inputNum;

	inputNum = sscanf(inputLine, "%s \"%[^\"]\" \"%[^\"]\"",
							command, sourceFile, destFile);
	
#if DEBUG_SCRIPT
s_printf("source: %s\ndest: %s\n", sourceFile, destFile);
#endif
	if (inputNum != 3)
	{
		return;
	}
	SFFileMove(sourceFile, destFile);
}


/************************************************************
	DoFileMove(inputLine);
	with inputLine
	mv "aVol:bDir:cFile" "dVol:edir:fFile"
	mv "aVol:bDir:cFile" "dVol:edir"
	move aVol:bDir:cFile into dVol:edir:
*************************************************************/

void DoReverseFileMove(char *inputLine)
{
char command[16]="", sourceFile[512]="", destFile[512]="";
int inputNum;

	inputNum = sscanf(inputLine, "%s \"%[^\"]\" \"%[^\"]\"",
							command, sourceFile, destFile);
	
#if DEBUG_SCRIPT
s_printf("rmv:\nsource: %s\ndest: %s\n", sourceFile, destFile);
#endif
	if (inputNum != 3)
	{
		return;
	}
	SFFileMove(destFile, sourceFile);
}


/************************************************************
	void DoLaunch(char *inputLine)
	Launch application.
*************************************************************/

void DoLaunch(char *inputLine)
{
char command[16]="", appFile[256]="";
int inputNum;
FSSpec appSpec;

	inputNum = sscanf(inputLine, "%s \"%[^\"]\"", command, appFile);
	
	if (inputNum != 2)
	{
		return;
	}
	
	if (SFCheckExist(appFile, &appSpec))
		return;
	Launch(&appSpec);
}


/************************************************************
	void DoLaunchOpen(char *inputLine)
	Launch application.
*************************************************************/

void DoLaunchOpen(char *inputLine)
{
char command[16]="", appFile[256]="", docFile[256]="";
int inputNum;
FSSpec appSpec, docSpec;

	inputNum = sscanf(inputLine, "%s \"%[^\"]\" \"%[^\"]\"",
						command, appFile, docFile);
	
	if (inputNum != 3)
	{
		return;
	}
	if (SFCheckExist(appFile, &appSpec))
		return;
	if (SFCheckExist(docFile, &docSpec))
		return;
	SFPath2Spec(appFile, &appSpec);
	SFPath2Spec(docFile, &docSpec);
	LaunchOpen(&appSpec, &docSpec);
}


/************************************************************
	void DoKill(char *inputLine)
	Kill application by Apple Event
*************************************************************/

void DoKill(char *inputLine)
{
char command[16]="", baseName[64], dirName[256], pathName[256];
int inputNum;

	inputNum = sscanf(inputLine, "%s \"%[^\"]\"",
						command, pathName);
	
	if (inputNum != 2)
	{
		return;
	}
	SFGetBaseAndDirName(pathName, baseName, dirName);
	SPKillProcess(baseName);
}


/************************************************************
	void DoKillOld(char *inputLine)
	Kill application not accepting Apple Event
*************************************************************/

#if !S_CARBON
void DoKillOld(char *inputLine)
{
char command[16]="", baseName[64], dirName[256], pathName[256];
int inputNum;

	inputNum = sscanf(inputLine, "%s \"%[^\"]\"",
						command, pathName);
	
	if (inputNum != 2)
	{
		return;
	}
	SFGetBaseAndDirName(pathName, baseName, dirName);
	SPKillOldProcess(baseName);
}
#endif


void DoTouch(char *inputLine)
{
char command[16]="", touchFile[256]="";
int inputNum;
FSSpec fss;

	inputNum = sscanf(inputLine, "%s \"%[^\"]\"", command, touchFile);	
	if (inputNum != 2)
		return;
	
	if (SFCheckExist(touchFile, &fss))
		return;
	FSpBumpDate(&fss);
}


void DoRemoveDirectory(char *inputLine)
{
char command[16]="", removeDir[256]="";
int inputNum;

	inputNum = sscanf(inputLine, "%s \"%[^\"]\"", command, removeDir);	
	if (inputNum != 2)
		return;
	
	SFRemoveDirectory(removeDir);
}


#if !S_CARBON
void DoKeyDown(char *inputLine)
{
char command[16]="";
int inputNum;
char c;
short modifiers;

	inputNum = sscanf(inputLine, "%s", command);	
	if (inputNum != 1)
		return;
	
	inputLine += strlen(command);
	while (isspace(*inputLine))
		inputLine++;
	
	if (inputLine[0] == '\'')
	{
		KeyDownQuote(inputLine+1, 0);
	}
	else if (isupper(inputLine[0]))
	{
		KeyDownMod(inputLine);
	}
}


void KeyDownQuote(char *inputLine, short modifiers)
{
char c;
unsigned n = 0;


	if (inputLine[0] == '\\')
	{
		switch (inputLine[1])
		{
		case 'n':
			c = '\n';
			break;
		case 't':
			c = '\t';
			break;
		case 'v':
			c = '\v';
			break;
		case 'b':
			c = '\b';
			break;
		case 'r':
			c = '\r';
			break;
		case 'f':
			c = '\f';
			break;
		case 'a':
			c = '\a';
			break;
		case '\\':
			c = '\\';
			break;
		case '\?':
			c = '\?';
			break;
		case '\'':
			c = '\'';
			break;
		case '\"':
			c = '\"';
			break;
		case '<':
			c = '\x1C';
			break;
		case '>':
			c = '\x1D';
			break;
		case '^':
			c = 0x1E;
			break;
		case 'V':
			c = '\x1F';
			break;
		case '0':
		case '1':
		case '2':
		case '3':
		case '4':
		case '5':
		case '6':
		case '7':
			sscanf(inputLine+1, "%o", &n);
			c = n;
			break;
		case 'x':
			sscanf(inputLine+1, "%x", &n);
			c = n;
			break;
		}
	}
	else
		c = inputLine[0];
	
	SEKeyDown(c, modifiers);
}


void KeyDownMod(char *inputLine)
{
short modifiers = 0;

	for (; inputLine[0] && inputLine[0] != '+'; inputLine++)
	{
		if (isupper(inputLine[0]))
		{
			switch (inputLine[0])
			{
			case 'C':
				if (inputLine[1] == 'o')
					inputLine++;
			
				switch (inputLine[1])
				{
				case 'm':
					modifiers |= cmdKey;
					break;
				case 'n':
					modifiers |= controlKey;
					break;
				case 'a': /* Capslock */
					modifiers |= alphaLock;
					break;
				}
				break;
			case 'S':
				modifiers |= shiftKey;
				break;
			case 'A':
				modifiers |= alphaLock;
				break;
			case 'O':
				modifiers |= optionKey;
				break;
			}
		}
	}

	for (; inputLine[0] && inputLine[0] != '\''; inputLine++)
		;

	if (inputLine[0])
		KeyDownQuote(inputLine+1, modifiers);
}
		

void DoMouseDown(char *inputLine)
{
char command[16]="";
int inputNum;
short x, y;

	inputNum = sscanf(inputLine, "%s (%hd, %hd)",
					command, &x, &y);	
	if (inputNum != 3)
		return;
	SEMouseDown(x, y);
}



void DoMouseUp(char *inputLine)
{
char command[16]="";
int inputNum;
short x, y;

	inputNum = sscanf(inputLine, "%s (%hd, %hd)",
					command, &x, &y);	
	if (inputNum != 3)
		return;
	SEMouseUp(x, y);
}
#endif/*!S_CARBON*/



void DoSleep(char *inputLine)
{
char command[16]="";
int inputNum;
double t;
long tick;

	inputNum = sscanf(inputLine, "%s %lf",
					command, &t);	
	if (inputNum != 2)
		return;
#if !S_CARBON
	SystemTask();
#endif
	Delay(t*60, &tick);
}



void DoQuit(char *inputLine)
{
	ExitToShell();
}




void DoScriptCreator(char *inputLine)
{
char command[16]="";
int inputNum;
char CreatorString[5];
OSType creator;

	inputNum = sscanf(inputLine, "%s \'%4c",
					command, CreatorString);	
	if (inputNum != 2)
		return;

	CreatorString[4] = 0;
	CString2OSType(CreatorString, &creator);
	if (s_dd.fileTotal)
		SFSetCreator(&s_dd.fss, creator);
	else
		SFSetCreator(&s_df.fss, creator);
}



/************************************************************
	DoComment(inputLine);
	with inputLine
	# .....   space should follow '#'
	Do nothing
*************************************************************/

void DoComment(char *inputLine)
{
}

#endif /*pragma once*/