/*
 *  s_Toolbox.c
 *  Version 2.0.2 / 020718
 *  by Satoshi Takahashi & Hiroki Ueda
 */

#ifndef S_TOOLBOX_C /*pragma once*/
#define S_TOOLBOX_C
#include <s_Config.h>
#if S_UNIX


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#ifndef __MWERKS__ /* trick to compile on mac */
#include <sys/time.h>
#endif

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <X11/keysym.h>

/* To make dialog, we need X Toolkit and their headers */
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/AsciiText.h>
#include <X11/Xaw/Toggle.h>

#include <s_Common.h>
#include <s_Toolbox.h>
#include <s_UNIX.h>

#include <s_Debug.h>

#include <StartSCL.h>

/* Dialog uses SCLXResource */
#include <SCLXResource.h>


GrafPtr thePort;

s_tbType s_tb = 
{
	{0, 0, 0, 0},	/* curRect */
	{0}, /* polygon used array */
	{0}, /* region used array */
	{"Osaka", "Geneva", "Monaco", "Times", 
		"Courier", "Symbol", "Helvetica",
		"Monaco" /*default*/}, /* macFont */
#if 1
	{ 
#if Linux
		"fixed", 
#else
		"gothic medium bbb", 
#endif
		"helvetica", "courier", "times",
		"courier", "symbol", "helvetica",
		"courier" /*default*/}, /* XFont */
#if TBF
	{ 1, 0, 0, 0,
		0, 0, 0,
		0},
#endif /*TBF*/
#else
	{"Helvetica", "Helvetica", "Courier", "Times",
		"Courier", "Symbol", "Helvetica",
		"Courier" /*default*/}, /* XFont */
#endif
	2, /* fontNum "Monaco" */
#if 1
	"medium-r", /* fontStyle plain */
#else
	"", /* fontStyle plain */
#endif
	plain, /* fontStyleNum plain */
	12, /* fontSize */
	{0}, /* basicColors */
};

/********** Pattern Data **********/

char s_tbPatternData[39][8] = 	/* 8 x 8 is inough */
{
	/* 0 is not defined */
	{'\x00'},
	/* 1 black */
	{'\xff', '\xff', '\xff', '\xff', '\xff', '\xff', '\xff', '\xff', },
	/* 2 dark dark gray */
	{'\xdd', '\xff', '\x77', '\xff', '\xdd', '\xff', '\x77', '\xff', },
	/* 3 dark gray */
	{'\xdd', '\x77', '\xdd', '\x77', '\xdd', '\x77', '\xdd', '\x77', },
	/* 4 gray */
	{'\xaa', '\x55', '\xaa', '\x55', '\xaa', '\x55', '\xaa', '\x55', },
	/* 5 grid */
	{'\x55', '\xff', '\x55', '\xff', '\x55', '\xff', '\x55', '\xff', },
	/* 6 vertical stripe */
	{'\xaa', '\xaa', '\xaa', '\xaa', '\xaa', '\xaa', '\xaa', '\xaa', },
	/* 7 dark hatch */
	{'\xee', '\xdd', '\xbb', '\x77', '\xee', '\xdd', '\xbb', '\x77', },
	/* 8 light vertical stripe */
	{'\x88', '\x88', '\x88', '\x88', '\x88', '\x88', '\x88', '\x88', },
	/* 9 pebble */
	{'\xb1', '\x30', '\x03', '\x1b', '\xd8', '\xc0', '\x0c', '\x8d', },
	/* 10 light pebble */
	{'\x80', '\x10', '\x02', '\x20', '\x01', '\x08', '\x40', '\x04', },
	/* 11 light grid */
	{'\xff', '\x88', '\x88', '\x88', '\xff', '\x88', '\x88', '\x88', },
	/* 12 brick */
	{'\xff', '\x80', '\x80', '\x80', '\xff', '\x08', '\x08', '\x08', },
	/* 13 ultra light gray */
	{'\x80', '\x00', '\x00', '\x00', '\x00', '\x00', '\x00', '\x00', },
	/* 14 tick tack */
	{'\x80', '\x40', '\x20', '\x00', '\x02', '\x04', '\x08', '\x00', },
	/* 15 four arms */
	{'\x82', '\x44', '\x39', '\x44', '\x82', '\x01', '\x01', '\x01', },
	/* 16 weave */
	{'\xf8', '\x74', '\x22', '\x47', '\x8f', '\x17', '\x22', '\x71', },
	/* 17 arches */
	{'\x55', '\xa0', '\x40', '\x40', '\x55', '\x0a', '\x04', '\x04', },
	/* 18 bowels */
	{'\x20', '\x50', '\x88', '\x88', '\x88', '\x88', '\x05', '\x02', },
	/* 19 wafers */
	{'\xbf', '\x00', '\xbf', '\xbf', '\xb0', '\xb0', '\xb0', '\xb0', },
	/* 20 white */
	{'\x00', '\x00', '\x00', '\x00', '\x00', '\x00', '\x00', '\x00', },
	/* 21 light light light gray */
	{'\x80', '\x00', '\x08', '\x00', '\x80', '\x00', '\x08', '\x00', },
	/* 22 light light gray */
	{'\x88', '\x00', '\x22', '\x00', '\x88', '\x00', '\x22', '\x00', },
	/* 23 light gray */
	{'\x88', '\x22', '\x88', '\x22', '\x88', '\x22', '\x88', '\x22', },
	/* 24 white dense grid */
	{'\xaa', '\x00', '\xaa', '\x00', '\xaa', '\x00', '\xaa', '\x00', },
	/* 25 horizontal stripe */
	{'\xff', '\x00', '\xff', '\x00', '\xff', '\x00', '\xff', '\x00', },
	/* 26 light hatch */
	{'\x11', '\x22', '\x44', '\x88', '\x11', '\x22', '\x44', '\x88', },
	/* 27 light horizontal stripe */
	{'\xff', '\x00', '\x00', '\x00', '\xff', '\x00', '\x00', '\x00', },
	/* 28 light light hatch */
	{'\x01', '\x02', '\x04', '\x08', '\x10', '\x20', '\x40', '\x80', },
	/* 29 not yet */
	{'\xa0', '\x00', '\x80', '\x00', '\x88', '\x00', '\x80', '\x00', },
	/* 30 light light grid */
	{'\xff', '\x80', '\x80', '\x80', '\x80', '\x80', '\x80', '\x80', },
	/* 31 slant brick */
	{'\x08', '\x1a', '\x22', '\xc1', '\x80', '\x01', '\x02', '\x03', },
	/* 32 wave */
	{'\x88', '\x14', '\x22', '\x41', '\x88', '\x00', '\xaa', '\x00', },
	/* 33 kenzan */
	{'\x40', '\xa0', '\x00', '\x00', '\x04', '\x0a', '\x00', '\x00', },
	/* 34 kawara */
	{'\x03', '\x84', '\x48', '\x30', '\x0c', '\x02', '\x01', '\x01', },
	/* 35 scale */
	{'\x80', '\x80', '\x41', '\x3e', '\x08', '\x08', '\x14', '\xe3', },
	/* 36 slate */
	{'\x10', '\x20', '\x54', '\xaa', '\xff', '\x02', '\x04', '\x08', },
	/* 37 balls */
	{'\x77', '\x89', '\x8f', '\x8f', '\x77', '\x98', '\xf8', '\xf8', },
	/* 38 diamonds */
	{'\x00', '\x08', '\x14', '\x2a', '\x55', '\x4a', '\x24', '\x08', },
};

Pixmap s_tbSystemPattern[39];

Pattern white;
Pattern black;
Pattern gray;
Pattern ltGray;
Pattern dkGray;

/*** XToolkit Data for Dialog */
s_XtType s_Xt = {0, 0, 0, 0, 0, 0, 0,  {0},};

/******************************** QuickDraw **********************************/

/********** Internal **********/

void WriteRect(Rect* rp)
{
	if ( (rp->left == s_tb.curRect.left) && (rp->top == s_tb.curRect.top) &&
		 (rp->right == s_tb.curRect.right) &&
		 (rp->bottom == s_tb.curRect.bottom) )
		return; /* do nothing */
	else
	{
		s_tb.curRect = *rp;
		fprintf(s_UNIX.drawingFile,"SetRect(&r,%hd,%hd,%hd,%hd);\n",
				s_tb.curRect.left, s_tb.curRect.top,
				s_tb.curRect.right, s_tb.curRect.bottom);
	}
}

void STSort( short *px, short *py )
{
int tmp;

        if ( *px > *py ){
                tmp = *px;
                *px = *py;
                *py = tmp;
        }

}


/********** Line and Point**********/

void MoveTo(short x, short y)
{
	if (s_UNIX.drawingFlag)
	{
		if ( s_Config.XFlag && thePort->polySave)
		{
			if (thePort->polySave->num < POLY_SIZE_MAX)
			{
				thePort->polySave->num--;
				thePort->polySave->savePoints[
					thePort->polySave->num].x = x;
				thePort->polySave->savePoints[
					thePort->polySave->num].y = y;
				thePort->polySave->num++;
			}
		}
		fprintf(s_UNIX.drawingFile,"MoveTo(%hd,%hd);\n", x, y);
	}
	thePort->pnLoc.h = x;
	thePort->pnLoc.v = y;
}

void LineTo(short x, short y)
{
	dwf("LineTo\n");

	if (s_UNIX.drawingFlag)
		fprintf(s_UNIX.drawingFile,"LineTo(%hd,%hd);\n", x, y);

	if (s_Config.XFlag)
	{
		XDrawLine(thePort->display, thePort->win, thePort->gc,
			thePort->pnLoc.h,
			thePort->pnLoc.v,
			x, y);
		XFlush(thePort->display);

		if (thePort->polySave)
		{
			if (thePort->polySave->num < POLY_SIZE_MAX)
			{
				thePort->polySave->savePoints[
					thePort->polySave->num].x = x;
				thePort->polySave->savePoints[
					thePort->polySave->num].y = y;
				thePort->polySave->num++;
			}
		}
	}

	thePort->pnLoc.h = x;
	thePort->pnLoc.v = y;
}

void Move(short x, short y)
{
	if (s_UNIX.drawingFlag)
	{
		if ( s_Config.XFlag && thePort->polySave)
		{
			if (thePort->polySave->num < POLY_SIZE_MAX)
			{
				thePort->polySave->num--;
				thePort->polySave->savePoints[
					thePort->polySave->num].x = thePort->pnLoc.h + x;
				thePort->polySave->savePoints[
					thePort->polySave->num].y = thePort->pnLoc.v + y;
				thePort->polySave->num++;
			}
		}
		fprintf(s_UNIX.drawingFile,"Move(%hd,%hd);\n", x, y);
	}
	thePort->pnLoc.h += x;
	thePort->pnLoc.v += y;
}

void Line(short x, short y)
{
	if (s_UNIX.drawingFlag)
		fprintf(s_UNIX.drawingFile,"Line(%hd,%hd);\n", x, y);

	if (s_Config.XFlag)
	{
		XDrawLine(thePort->display, thePort->win, thePort->gc,
			thePort->pnLoc.h,
			thePort->pnLoc.v,
			thePort->pnLoc.h + x,
			thePort->pnLoc.v + y);
		XFlush(thePort->display);

		if (thePort->polySave)
		{
			if (thePort->polySave->num < POLY_SIZE_MAX)
			{
				thePort->polySave->savePoints[
					thePort->polySave->num].x = thePort->pnLoc.h + x;
				thePort->polySave->savePoints[
					thePort->polySave->num].y = thePort->pnLoc.v + y;
				thePort->polySave->num++;
			}
		}
	}

	thePort->pnLoc.h += x;
	thePort->pnLoc.v += y;
}

void GetPen(Point* pt)
{
	*pt = thePort->pnLoc;
}

/********** Rect **********/

void SetRect(Rect* rp, short left, short top, short right, short bottom)
{
	rp->left = left;
	rp->top = top;
	rp->right = right;
	rp->bottom = bottom;
}

void FrameRect(Rect* rp)
{
	if (s_UNIX.drawingFlag)
	{
		WriteRect(rp);
		fprintf(s_UNIX.drawingFile,"FrameRect(&r);\n");
	}

	if (s_Config.XFlag)
	{
		STSort(&(rp->left), &(rp->right) );
		STSort(&(rp->top), &(rp->bottom) );
		XDrawRectangle( thePort->display, thePort->win, thePort->gc,
				rp->left, rp->top, rp->right - rp->left,
				rp->bottom - rp->top);
		XFlush( thePort->display );
	}
}

void PaintRect(Rect* rp)
{
	if (s_UNIX.drawingFlag)
	{
		WriteRect(rp);
		fprintf(s_UNIX.drawingFile,"PaintRect(&r);\n");

		if (s_Config.XFlag)
		{
			STSort(&(rp->left), &(rp->right) );
			STSort(&(rp->top), &(rp->bottom) );
			XFillRectangle( thePort->display, thePort->win, thePort->gc,
					rp->left, rp->top, rp->right - rp->left,
					rp->bottom - rp->top);
			XFlush( thePort->display );
		}
	}
}

void EraseRect(Rect* rp)
{
	if (s_UNIX.drawingFlag)
	{
		WriteRect(rp);
		fprintf(s_UNIX.drawingFile,"EraseRect(&r);\n");
	}

	if (s_Config.XFlag)
	{
		STSort(&(rp->left), &(rp->right) );
		STSort(&(rp->top), &(rp->bottom) );
		XClearArea( thePort->display, thePort->win,
				rp->left, rp->top, rp->right - rp->left,
				rp->bottom - rp->top, 0);
		XFlush( thePort->display );
	}
}

void InvertRect(Rect* rp)
{
	if (s_UNIX.drawingFlag)
	{
		WriteRect(rp);
		fprintf(s_UNIX.drawingFile,"InvertRect(&r);\n");

		if (s_Config.XFlag)
		{
			STSort(&(rp->left), &(rp->right) );
			STSort(&(rp->top), &(rp->bottom) );
			XSetFunction(thePort->display, thePort->gc, GXxor);
			XFillRectangle( thePort->display, thePort->win, thePort->gc,
					rp->left, rp->top, rp->right - rp->left,
					rp->bottom - rp->top);
			XSetFunction(thePort->display, thePort->gc, thePort->pnMode);
			XFlush( thePort->display );
		}
	}
}

void FillRect(Rect* rp, Pattern pat)
{
	if (s_UNIX.drawingFlag)
	{
		WriteRect(rp);
		fprintf(s_UNIX.drawingFile,"GetIndPattern(pat, 0, %d);\n", pat[0].num);
		fprintf(s_UNIX.drawingFile,"FillRect(&r, pat);\n");

		if (s_Config.XFlag)
		{
			STSort(&(rp->left), &(rp->right) );
			STSort(&(rp->top), &(rp->bottom) );
			XSetStipple(thePort->display, thePort->gc, pat[0].pixmap);
			XSetFillStyle(thePort->display, thePort->gc,
				FillOpaqueStippled);
			XFillRectangle( thePort->display, thePort->win, thePort->gc,
					rp->left, rp->top, rp->right - rp->left,
					rp->bottom - rp->top);
			XSetStipple(thePort->display, thePort->gc, thePort->pnPat[0].pixmap);
			XFlush( thePort->display );
		}
	}
}

/********** Oval **********/

void FrameOval(Rect* rp)
{
	if (s_UNIX.drawingFlag)
	{
		WriteRect(rp);
		fprintf(s_UNIX.drawingFile,"FrameOval(&r);\n");
	}

	if (s_Config.XFlag)
	{
		STSort (&(rp->left), &(rp->right) );
		STSort (&(rp->top), &(rp->bottom) );
		XDrawArc( thePort->display, thePort->win, thePort->gc,
				rp->left, rp->top, rp->right - rp->left,
				rp->bottom - rp->top, 0, 23040); /* 360*64 */
		XFlush( thePort->display );
	}
}

void PaintOval(Rect* rp)
{
	if (s_UNIX.drawingFlag)
	{
		WriteRect(rp);
		fprintf(s_UNIX.drawingFile,"PaintOval(&r);\n");
	}

	if (s_Config.XFlag)
	{
		STSort (&(rp->left), &(rp->right) );
		STSort (&(rp->top), &(rp->bottom) );
		XFillArc( thePort->display, thePort->win, thePort->gc,
				rp->left, rp->top, rp->right - rp->left,
				rp->bottom - rp->top, 0, 23040); /* 360*64 */
		XFlush( thePort->display );
	}
}

void EraseOval(Rect* rp)
{
	if (s_UNIX.drawingFlag)
	{
		WriteRect(rp);
		fprintf(s_UNIX.drawingFile,"EraseOval(&r);\n");

		if (s_Config.XFlag)
		{
			STSort (&(rp->left), &(rp->right) );
			STSort (&(rp->top), &(rp->bottom) );
			XSetForeground( thePort->display, thePort->gc, thePort->backColor);
			XFillArc( thePort->display, thePort->win, thePort->gc,
					rp->left, rp->top, rp->right - rp->left,
					rp->bottom - rp->top, 0, 23040); /* 360*64 */
			XSetForeground( thePort->display, thePort->gc, thePort->foreColor);
			XFlush( thePort->display );
		}
	}
}

void InvertOval(Rect* rp)
{
	if (s_UNIX.drawingFlag)
	{
		WriteRect(rp);
		fprintf(s_UNIX.drawingFile,"InvertOval(&r);\n");
	}

	if (s_Config.XFlag)
	{
		STSort (&(rp->left), &(rp->right) );
		STSort (&(rp->top), &(rp->bottom) );
		XSetFunction(thePort->display, thePort->gc, GXxor);
		XFillArc( thePort->display, thePort->win, thePort->gc,
				rp->left, rp->top, rp->right - rp->left,
				rp->bottom - rp->top, 0, 23040); /* 360*64 */
		XSetFunction(thePort->display, thePort->gc, thePort->pnMode);
		XFlush( thePort->display );
	}
}

void FillOval(Rect* rp, Pattern pat)
{
	if (s_UNIX.drawingFlag)
	{
		WriteRect(rp);
		fprintf(s_UNIX.drawingFile,"GetIndPattern(pat, 0, %d);\n", pat[0].num);
		fprintf(s_UNIX.drawingFile,"FillOval(&r, pat);\n");

		if (s_Config.XFlag)
		{
			STSort(&(rp->left), &(rp->right) );
			STSort(&(rp->top), &(rp->bottom) );
			XSetStipple(thePort->display, thePort->gc, pat[0].pixmap);
			XSetFillStyle(thePort->display, thePort->gc,
				FillOpaqueStippled);
			XFillArc( thePort->display, thePort->win, thePort->gc,
				rp->left, rp->top, rp->right - rp->left,
				rp->bottom - rp->top, 0, 23040); /* 360*64 */
			XSetStipple(thePort->display, thePort->gc, thePort->pnPat[0].pixmap);
			XFlush( thePort->display );
		}
	}
}

/********** Oval **********/

/********** Text **********/

short TextWidth(char *string, short start, short length)
{
	if (s_Config.XFlag)
	{
		return XTextWidth(XQueryFont(thePort->display, 
			XGContextFromGC(thePort->gc)),
			string+start, length);
	}
	else
		return -1;
}


void DrawText(char* s1,short start,short length)
{
	if (s_UNIX.drawingFlag)
		fprintf(s_UNIX.drawingFile,
			"DrawText(\"%s\",%hd,%hd);\n",s1,start,length);

	if (s_Config.XFlag)
	{
#if TBF
		if (s_tb.fontTwoByte[thePort->txFont])
			DrawTextTwoByte(s1, start, length);
		else
		{
#endif
			XDrawString( thePort->display, thePort->win, 
				thePort->gc,
				thePort->pnLoc.h,
				thePort->pnLoc.v,
				(s1+start), length);
			thePort->pnLoc.h+=XTextWidth(XQueryFont(thePort->display, 
				XGContextFromGC(thePort->gc)),
				s1+start, length);
#if TBF
		}
#endif
		XFlush( thePort->display );
	}
}



#if TBF
int DrawTextTwoByte(char* s1,short start,short length)
{
char s2[256];
int count = 0;
char fullFontName[256];
Font fontID;
int listCount;
char **list;


	s1 += start;
	while ( s1[0] && length > 0 )
	{
		while ( !(s1[count] & 0x80 ) && count < length )
		{
			count++;
		}
		if (count)
		{
			sprintf(fullFontName, "*-%s-%s-normal-*-%hd-%hd-*-jisx0201*", 
				s_tb.XFont[thePort-> txFont],
				s_tb.fontStyle, thePort->txSize, 
				10*thePort->txSize);
			list = XListFonts( thePort->display, fullFontName, 
				1, &listCount);
			if (!listCount)
				return -1;
			fontID = XLoadFont(thePort->display, list[0]);
			XFreeFontNames(list);
			XSetFont(thePort->display, thePort->gc, fontID);

			XDrawString( thePort->display, thePort->win, 
				thePort->gc,
				thePort->pnLoc.h,
				thePort->pnLoc.v,
				s1, count);
			thePort->pnLoc.h+=XTextWidth(XQueryFont(thePort->display, 
				XGContextFromGC(thePort->gc)),
				s1, count);
			s1 += count;
			length -= count;
			count = 0;
		}

		while ( (s1[count] & 0x80 ) && count < length )
		{
			s2[count] = s1[count] & 0x7f;
			count++;
		}
		if (count)
		{
			sprintf(fullFontName, "*-%s-%s-normal-*-%hd-%hd-*-jisx0208*", 
				s_tb.XFont[thePort-> txFont],
				s_tb.fontStyle, thePort->txSize, 
				10*thePort->txSize);
			list = XListFonts( thePort->display, fullFontName, 
				1, &listCount);
			if (!listCount)
				return -1;
			fontID = XLoadFont(thePort->display, list[0]);
			XFreeFontNames(list);
			XSetFont(thePort->display, thePort->gc, fontID);

			XDrawString16( thePort->display, thePort->win, 
				thePort->gc,
				thePort->pnLoc.h,
				thePort->pnLoc.v,
				(XChar2b*)s2, count/2);
			thePort->pnLoc.h+=XTextWidth16(XQueryFont(thePort->display, 
				XGContextFromGC(thePort->gc)),
				(XChar2b*)s2, count/2);
			s1 += count;
			length -= count;
			count = 0;
		}
	}
	return 0;
		
}
#endif

void DrawChar(char c)
{
	DrawText(&c, 0, 1);
}

void DrawString(Str255 string)
{
	DrawText(string, 0, strlen(string));
}


/********** Color **********/

void ForeColor(long color)
{
int colorX;

	if (s_UNIX.drawingFlag)
	{
		if (s_Config.XFlag)
		{
			switch (color)
			{
			case whiteColor:
				colorX = whiteXColor;
				break;
			case blackColor:
				colorX = blackXColor;
				break;
			case redColor:
				colorX = redXColor;
				break;
			case greenColor:
				colorX = greenXColor;
				break;
			case blueColor:
				colorX = blueXColor;
				break;
			case cyanColor:
				colorX = cyanXColor;
				break;
			case magentaColor:
				colorX = magentaXColor;
				break;
			case yellowColor:
				colorX = yellowXColor;
				break;
			}
			XSetForeground(thePort->display, thePort->gc,
				s_tb.basicColors[colorX].pixel);
			thePort->foreColor = s_tb.basicColors[colorX].pixel;
		}
		fprintf(s_UNIX.drawingFile,"ForeColor(%ld);\n", color);
	}
}


void BackColor(long color)
{
int colorX;

	if (s_UNIX.drawingFlag)
	{
		if (s_Config.XFlag)
		{
			switch (color)
			{
			case whiteColor:
				colorX = whiteXColor;
				break;
			case blackColor:
				colorX = blackXColor;
				break;
			case redColor:
				colorX = redXColor;
				break;
			case greenColor:
				colorX = greenXColor;
				break;
			case blueColor:
				colorX = blueXColor;
				break;
			case cyanColor:
				colorX = cyanXColor;
				break;
			case magentaColor:
				colorX = magentaXColor;
				break;
			case yellowColor:
				colorX = yellowXColor;
				break;
			}
			XSetBackground(thePort->display, thePort->gc,
				s_tb.basicColors[colorX].pixel);
			thePort->backColor = s_tb.basicColors[colorX].pixel;
		}
		fprintf(s_UNIX.drawingFile,"BackColor(%ld);\n", color);
	}
}



void RGBForeColor(RGBColor *colorP)
{
XColor cellColor;

	if (s_UNIX.drawingFlag)
	{
		if (s_Config.XFlag)
		{
			cellColor.red = colorP->red;
			cellColor.green = colorP->green;
			cellColor.blue = colorP->blue;
			cellColor.flags = DoRed | DoGreen | DoBlue;
			XAllocColor(thePort->display, thePort->colormap,
				&cellColor);
			XSetForeground(thePort->display, thePort->gc,
					cellColor.pixel);
			thePort->foreColor = cellColor.pixel;
		}
		fprintf(s_UNIX.drawingFile,"color.red=%hud\n", colorP->red);
		fprintf(s_UNIX.drawingFile,"color.green=%hud\n", colorP->green);
		fprintf(s_UNIX.drawingFile,"color.blue=%hud\n", colorP->blue);
		fprintf(s_UNIX.drawingFile,"RGBForeColor(&color);\n");
	}
}


void RGBBackColor(RGBColor *colorP)
{
XColor cellColor;

	if (s_UNIX.drawingFlag)
	{
		if (s_Config.XFlag)
		{
			cellColor.red = colorP->red;
			cellColor.green = colorP->green;
			cellColor.blue = colorP->blue;
			cellColor.flags = DoRed | DoGreen | DoBlue;
			XAllocColor(thePort->display, thePort->colormap,
				&cellColor);
			XSetBackground(thePort->display, thePort->gc,
					cellColor.pixel);
			thePort->backColor = cellColor.pixel;
		}
		fprintf(s_UNIX.drawingFile,"color.red=%hud\n", colorP->red);
		fprintf(s_UNIX.drawingFile,"color.green=%hud\n", colorP->green);
		fprintf(s_UNIX.drawingFile,"color.blue=%hud\n", colorP->blue);
		fprintf(s_UNIX.drawingFile,"RGBBackColor(&color);\n");
	}
}



/********** Polygon **********/

PolyHandle OpenPoly(void)
{
int i=1;
PolyHandle retPolyHandle;

	dwf("OpenPoly\n");

	while(s_tb.polyUsed[i])
		i++;
	
	if (i >= MAX_POLY )
	{
		fprintf(stderr, "Error: Too many polygons!\n");
		i = 0;
	}
	
	if (s_UNIX.drawingFlag)
	{
		fprintf(s_UNIX.drawingFile,"poly[%d]=OpenPoly();\n", i);
	}
	
	s_tb.polyUsed[i] = 1;

	if (s_Config.XFlag)
	{
		HidePen();
		if ((thePort->polySave = malloc(sizeof(PolySaveType)))
			&& (thePort->polySave->poly = malloc(sizeof(Polygon *)))
			&& (thePort->polySave->poly[0] = malloc(sizeof(Polygon))))
		{
			thePort->polySave->savePoints[0].x =
				thePort->pnLoc.h;
			thePort->polySave->savePoints[0].y =
				thePort->pnLoc.v;
			thePort->polySave->num = 1;
			thePort->polySave->poly[0]->polyNum = i;
			return thePort->polySave->poly;
		}
		else
			return 0;
	}
	return 0;
}

void ClosePoly(void)
{
int i;

	if (s_UNIX.drawingFlag)
	{
		if (s_Config.XFlag)
		{
			ShowPen();
			if (!(thePort->polySave->poly[0]->points =
				malloc(thePort->polySave->num *
					sizeof(XPoint))))
				return;
			if (!(thePort->polySave->poly[0]->polyPoints =
				malloc(thePort->polySave->num *
					sizeof(Point))))
				return;
			for (i = 0; i < thePort->polySave->num; i++)
			{
				thePort->polySave->poly[0]->points[i] =
					thePort->polySave->savePoints[i];
				thePort->polySave->poly[0]->polyPoints[i].h =
					thePort->polySave->savePoints[i].x;
				thePort->polySave->poly[0]->polyPoints[i].v =
					thePort->polySave->savePoints[i].y;
			}
			thePort->polySave->poly[0]->npoints =
				thePort->polySave->num;
            thePort->polySave->poly[0]->polySize = 
                4 * thePort->polySave->num + 10;
		}
		fprintf(s_UNIX.drawingFile,"ClosePoly();\n");
	}
}

void KillPoly(PolyHandle poly)
{
	if (!poly)
		return;
	
	if (s_UNIX.drawingFlag)
	{
		fprintf(s_UNIX.drawingFile,"KillPoly(poly[%d]);\n", poly[0]->polyNum);
		s_tb.polyUsed[poly[0]->polyNum] = 0;
		if (s_Config.XFlag)
		{
			free(poly[0]->points);
			free(poly[0]->polyPoints);
            free(poly[0]);
			free(poly);
		}
	}
}


void FramePoly(PolyHandle poly)
{
	if (s_UNIX.drawingFlag)
	{
		if (s_Config.XFlag)
		{
			XDrawLines(thePort->display, thePort->win, thePort->gc,
				poly[0]->points, poly[0]->npoints, CoordModeOrigin);
		}
		fprintf(s_UNIX.drawingFile,"FramePoly(poly[%d]);\n", poly[0]->polyNum);
	}
}


void PaintPoly(PolyHandle poly)
{
	if (s_UNIX.drawingFlag)
	{
		fprintf(s_UNIX.drawingFile,"ErasePoly(poly[%d]);\n", poly[0]->polyNum);

		if (s_Config.XFlag)
		{
			XFillPolygon( thePort->display, thePort->win, thePort->gc,
				poly[0]->points, poly[0]->npoints, Nonconvex, CoordModeOrigin);
			XFlush( thePort->display );
		}
	}
}


void ErasePoly(PolyHandle poly)
{
	if (s_UNIX.drawingFlag)
	{
		fprintf(s_UNIX.drawingFile,"ErasePoly(poly[%d]);\n", poly[0]->polyNum);

		if (s_Config.XFlag)
		{
			XSetForeground( thePort->display, thePort->gc, thePort->backColor);
			XFillPolygon( thePort->display, thePort->win, thePort->gc,
				poly[0]->points, poly[0]->npoints, Nonconvex, CoordModeOrigin);
			XSetForeground( thePort->display, thePort->gc, thePort->foreColor);
			XFlush( thePort->display );
		}
	}
}


void FillPoly(PolyHandle poly, Pattern pat)
{
	if (s_UNIX.drawingFlag)
	{
		fprintf(s_UNIX.drawingFile,"GetIndPattern(pat, 0, %d);\n", pat[0].num);
		fprintf(s_UNIX.drawingFile,"FillPoly(poly[%d], pat);\n", poly[0]->polyNum);

		if (s_Config.XFlag)
		{
			XSetStipple(thePort->display, thePort->gc, pat[0].pixmap);
			XSetFillStyle(thePort->display, thePort->gc,
				FillOpaqueStippled);
			XFillPolygon( thePort->display, thePort->win, thePort->gc,
				poly[0]->points, poly[0]->npoints, Nonconvex, CoordModeOrigin);
			XSetFillStyle(thePort->display, thePort->gc,
				FillSolid);
			XFlush( thePort->display );
		}
	}
}


/********** Region **********/

RgnHandle NewRgn(void)
{
int i=1;

	while(s_tb.rgnUsed[i])
		i++;
	
	if (i >= MAX_RGN )
	{
		fprintf(stderr, "Error: Too many regions!\n");
		i = 0;
	}
	
	if (s_UNIX.drawingFlag)
	{
		fprintf(s_UNIX.drawingFile,"rgn[%d]=NewRgn();\n", i);
	}
	
	s_tb.rgnUsed[i] = 1;
	
	return i;
}

void OpenRgn(void)
{
	if (s_UNIX.drawingFlag)
	{
		fprintf(s_UNIX.drawingFile,"OpenRgn();\n");
	}
}

void CloseRgn(RgnHandle rgnNo)
{
	if (s_UNIX.drawingFlag)
	{
		fprintf(s_UNIX.drawingFile,"CloseRgn(rgn[%d]);\n", rgnNo);
	}
}

void PaintRgn(RgnHandle rgnNo)
{
	if (s_UNIX.drawingFlag)
	{
		fprintf(s_UNIX.drawingFile,"PaintRgn(rgn[%d]);\n", rgnNo);
	}
}


void SectRgn(RgnHandle rgn1, RgnHandle rgn2, RgnHandle rgn3)
{
	if (s_UNIX.drawingFlag)
	{
		fprintf(s_UNIX.drawingFile,"SectRgn(rgn[%d],rgn[%d],rgn[%d]);\n",
					rgn1, rgn2, rgn3);
	}
}



/********** Pen **********/

void PenSize(short h, short v)
{
	if (s_UNIX.drawingFlag)
	{
		thePort->pnSize.h = h;
		thePort->pnSize.v = v;
		if (s_Config.XFlag)
		{
			XSetLineAttributes(thePort->display, thePort->gc,
				h, LineSolid, CapButt, JoinMiter);
		}
		fprintf(s_UNIX.drawingFile,"PenSize(%hd,%hd);\n", h, v);
	}
}

void PenPat(Pattern pat)
{
	if (s_UNIX.drawingFlag)
	{
		fprintf(s_UNIX.drawingFile,"GetIndPattern(pat, 0, %d);\n", pat[0].num);
		fprintf(s_UNIX.drawingFile,"PenPat(pat);\n");

		if (s_Config.XFlag)
		{
			XSetStipple(thePort->display, thePort->gc, pat[0].pixmap);
			XSetFillStyle(thePort->display, thePort->gc, FillOpaqueStippled);
			thePort->pnPat[0] = pat[0];
		}
	}
}

void GetPenState(PenState *psP)
{
	psP->pnLoc = thePort->pnLoc;
	psP->pnSize = thePort->pnSize;
	psP->pnMode = thePort->pnMode;
	psP->pnPat[0] = thePort->pnPat[0];
}

void SetPenState(PenState *psP)
{
	thePort->pnLoc = psP->pnLoc;
	thePort->pnSize = psP->pnSize;
	thePort->pnMode = psP->pnMode;
	thePort->pnPat[0] = psP->pnPat[0];
}


void PenMode(int mode)
{
char *macMode;

	if (s_UNIX.drawingFlag)
	{
		if (s_Config.XFlag)
		{
			XSetFunction(thePort->display, thePort->gc, mode);
		}
		switch (mode)
		{
		case patCopy:
			macMode = "patCopy";
			break;
		case notPatCopy:
			macMode = "notPatCopy";
			break;
		case patOr:
			macMode = "patOr";
			break;
		case notPatOr:
			macMode = "notPatOr";
			break;
		case patXor:
			macMode = "patXor";
			break;
		case notPatXor:
			macMode = "notPatXor";
			break;
		case patBic:
			macMode = "patBic";
			break;
		case notPatBic:
			macMode = "notPatBic";
			break;
		}
		fprintf(s_UNIX.drawingFile,"PenMode(%s);\n", macMode);
	}
}

void HidePen(void)
{
	thePort->pnVis--;

	if (s_UNIX.drawingFlag)
	{
		if (s_Config.XFlag)
		{
			if (thePort->pnVis <= 0)
			{
				XSetFunction(thePort->display, thePort->gc,
					GXnoop);
			}
		}
		fprintf(s_UNIX.drawingFile,"HidePen();\n");
	}
}

void ShowPen(void)
{
	thePort->pnVis++;

	if (s_UNIX.drawingFlag)
	{
		if (s_Config.XFlag)
		{
			if (thePort->pnVis > 0)
			{
				XSetFunction(thePort->display, thePort->gc,
					thePort->pnMode);
			}
		}
		fprintf(s_UNIX.drawingFile,"HidePen();\n");
	}
}


/********** Other QuickDraw **********/

void ScrollRect(Rect *scrollRectP, short dh, short dv, RgnHandle updateRgn)
{
	if (s_Config.XFlag)
		XCopyArea(thePort->display, thePort->win, thePort->win,
			thePort->gc,
			scrollRectP->left, scrollRectP->top,
			scrollRectP->right - scrollRectP->left,
			scrollRectP->bottom - scrollRectP->top,
			scrollRectP->left + dh,
			scrollRectP->top + dv);
}


void GetIndPattern(Pattern pat, int patListID, int index)
{
	pat[0].pixmap = s_tbSystemPattern[index];
	pat[0].num = index;
}


void GlobalToLocal(Point *pt)
{
int localX, localY;
Window child;

	XTranslateCoordinates(thePort->display,
			DefaultRootWindow(thePort->display),
			thePort->win, pt->h, pt->v,
			&localX, &localY, &child);
	pt->h = localX;
	pt->v = localY;
}


void GetCPixel(short x, short y, RGBColor *pixelP)
{
XColor xc;
XImage *xi;
Window w;
int wx, wy;
unsigned int width, height, b, depth;

    XGetGeometry(thePort->display, thePort->win, &w, &wx, &wy,
                &width, &height, &b, &depth);
    xi = XGetImage(thePort->display, thePort->win, 0, 0, 
            width, height, -1, XYPixmap);
    xc.pixel = XGetPixel(xi, x, y);
    XQueryColor(thePort->display, thePort->colormap, &xc);
    pixelP->red = xc.red;
    pixelP->green = xc.green;
    pixelP->blue = xc.blue;
}


RGBColor *GetPortForeColor(CGrafPtr port, RGBColor *cp)
{
XColor xc;

    xc.pixel = port->foreColor;
    XQueryColor(thePort->display, thePort->colormap, &xc);
    cp->red = xc.red;
    cp->green = xc.green;
    cp->blue = xc.blue;
    return cp;
}

RGBColor *GetPortBackColor(CGrafPtr port, RGBColor *cp)
{
XColor xc;

    xc.pixel = port->backColor;
    XQueryColor(thePort->display, thePort->colormap, &xc);
    cp->red = xc.red;
    cp->green = xc.green;
    cp->blue = xc.blue;
    return cp;
}

/******************** Font *********************/

void GetFNum(char *fontName, short *fontNumP)
{
int i;

	for (i = 0; i < FONT_NUM; i++)
	{
		if (strcmp(fontName, s_tb.macFont[i]) == 0)
		{
			*fontNumP = i;
			return;
		}
	}
	*fontNumP = thePort->txFont;
}


OSStatus FMGetFontFamilyName(short font, Str255 fontNameP)
{
    if (font < FONT_NUM)
        strcpy(fontNameP, s_tb.macFont[font]);
    return 0;
}


int ChangeXFont(void)
{
char fullFontName[256];
Font fontID;
#if 1
int count;
char **list;
#endif

	//sprintf(fullFontName, "%s%s-%hd", s_tb.XFont[thePort->txFont],
#if 1
	sprintf(fullFontName, "*-%s-%s-normal-*-%hd-%hd-*", s_tb.XFont[thePort->txFont],
		s_tb.fontStyle, thePort->txSize, 10*thePort->txSize);
#else
	sprintf(fullFontName, "%s%sce%hd", s_tb.XFont[thePort->txFont],
		s_tb.fontStyle, thePort->txSize);
#endif
#if 1
	list = XListFonts( thePort->display, fullFontName, 1, &count);
	if (!count)
		return -1;
	fontID = XLoadFont(thePort->display, list[0]);
	XFreeFontNames(list);
#else
	fontID = XLoadFont(thePort->display, fullFontName);
#endif
	XSetFont(thePort->display, thePort->gc, fontID);
}

#if 1
void TBSetStyleWord(void)
{
	if (strcmp(s_tb.XFont[thePort->txFont], "times") == 0)
	{
		switch(thePort->txFace)
		{
		case plain:
			strcpy(s_tb.fontStyle, "medium-r");
			break;
		case italic:
			strcpy(s_tb.fontStyle, "medium-i");
			break;
		case bold:
			strcpy(s_tb.fontStyle, "bold-r");
			break;
		case boldItalic:
			strcpy(s_tb.fontStyle, "bold-i");
			break;
		default:
			strcpy(s_tb.fontStyle, "medium-r");
			break;
		}
	}
	else if (strcmp(s_tb.XFont[thePort->txFont], "symbol") == 0)
	{
		switch(thePort->txFace)
		{
		default:
			strcpy(s_tb.fontStyle, "medium-r");
			break;
		}
	}
	else if (strcmp(s_tb.XFont[thePort->txFont], "gothic medium bbb") == 0)
	{
		switch(thePort->txFace)
		{
		case plain:
		case italic:
			strcpy(s_tb.fontStyle, "medium-r");
			break;
		case bold:
		case boldItalic:
			strcpy(s_tb.fontStyle, "bold-r");
			break;
		default:
			strcpy(s_tb.fontStyle, "medium-r");
			break;
		}
	}
	else
	{
		switch(thePort->txFace)
		{
		case plain:
			strcpy(s_tb.fontStyle, "medium-r");
			break;
		case italic:
			strcpy(s_tb.fontStyle, "medium-o");
			break;
		case bold:
			strcpy(s_tb.fontStyle, "bold-r");
			break;
		case boldItalic:
			strcpy(s_tb.fontStyle, "bold-o");
			break;
		default:
			strcpy(s_tb.fontStyle, "medium-r");
			break;
		}
	}
}
#else
TBSetStyleWord(void)
{
	if (strcmp(s_tb.XFont[thePort->txFont], "Times") == 0)
	{
		switch(thePort->txFace)
		{
		case plain:
			strcpy(s_tb.fontStyle, "-Roman");
			break;
		case italic:
			strcpy(s_tb.fontStyle, "-Italic");
			break;
		default:
			strcpy(s_tb.fontStyle, "-Roman");
			break;
		}
	}
	else if (strcmp(s_tb.XFont[thePort->txFont], "Symbol") == 0)
	{
		switch(thePort->txFace)
		{
		case plain:
			strcpy(s_tb.fontStyle, "");
			break;
		case italic:
			strcpy(s_tb.fontStyle, "");
			break;
		default:
			strcpy(s_tb.fontStyle, "");
			break;
		}
	}
	else
	{
		switch(thePort->txFace)
		{
		case plain:
			strcpy(s_tb.fontStyle, "");
			break;
		case italic:
			strcpy(s_tb.fontStyle, "-Oblique");
			break;
		default:
			strcpy(s_tb.fontStyle, "");
			break;
		}
	}
}
#endif

	
void TextFont(short fontNum)
{
	thePort->txFont = fontNum;
	TBSetStyleWord();
	if (s_Config.XFlag)
	{
		ChangeXFont();
	}
	if (s_Config.XFlag && s_ss.initDrawingFlag)
	{
		fprintf(s_UNIX.drawingFile,"GetFNum(""\\p%s"",&fNum);\n", 
			s_tb.macFont[thePort->txFont]);
		fprintf(s_UNIX.drawingFile,"TextFont(fNum);");
	}
}


void TextSize(short fontSize)
{
	thePort->txSize = fontSize;
	TBSetStyleWord();
	if (s_Config.XFlag)
	{
		ChangeXFont();
	}
	if (s_Config.XFlag && s_ss.initDrawingFlag)
		fprintf(s_UNIX.drawingFile,"TextSize(%hd);", fontSize);
}

void TextFace(Style face)
{
	thePort->txFace = face;
	TBSetStyleWord();
	if (s_Config.XFlag)
	{
		ChangeXFont();
	}
	if (s_Config.XFlag && s_ss.initDrawingFlag)
		fprintf(s_UNIX.drawingFile,"TextFace(%d);",face);
}


/******************** Events ********************/

int GetNextEvent(long eventMask, EventRecord *eventP)
{
int result = 0;
XEvent *xe;
int type;

Window root, child;
int rx, ry, wx, wy;
unsigned int mask;

char buf[20];
int bufSize = 20;
KeySym key;
XComposeStatus cs;

	xe = &(eventP->xEvent);
	if (s_Config.XFlag && s_ss.initDrawingFlag)
	{
		result =  XCheckMaskEvent(thePort->display, eventMask, xe);
	}
	type = xe->type;

	eventP->modifiers = 0;

	if (result)
	{
		switch (type)
		{
		case ButtonPress:
		case ButtonRelease:
			if (type == ButtonPress)
				eventP->what = mouseDown;
			else
				eventP->what = mouseUp;
			eventP->modifiers = xe->xbutton.state;
			break;
		case KeyPress:
		case KeyRelease:
			if (type == KeyPress)
				eventP->what = keyDown;
			else
				eventP->what = keyUp;
			eventP->modifiers = xe->xkey.state;
			XLookupString((XKeyEvent *)xe, buf, bufSize, &key, &cs);
			eventP->message = buf[0] + key<<8;
			break;
		case Expose:
			eventP->what = updateEvt;
			break;
		case FocusIn:
		case FocusOut:
			eventP->what = activateEvt;
			break;
		default:
			eventP->what = osEvt;
			break;
		}
	}
	else
		eventP->what = nullEvent;

	eventP->when = TickCount();
	
	XQueryPointer(thePort->display, thePort->win, &root, &child,
			&rx, &ry, &wx, &wy, &mask);
	eventP->where.h = rx;
	eventP->where.v = ry;

		
	return result;
}

void GetMouse(Point *pen)
{
Window root, child;
int rx, ry, wx, wy;
unsigned int mask;

	XQueryPointer(thePort->display, thePort->win, &root, &child,
			&rx, &ry, &wx, &wy, &mask);
	pen->h = wx;
	pen->v = wy;
	return;
}

/******************** NewWindow ********************/

WindowPtr NewWindow(Ptr wStorage, Rect *boundsRect,
			char *title, Boolean visible,
			int theProc, WindowPtr behind,
			Boolean goAwayFlag, long refcon)
{
Rect portRect;
int i;

/* X */
XSizeHints sizeHints;
XGCValues values;
int count;
unsigned long valuemask;
XSetWindowAttributes attributes;
XColor dbColor;
GrafPtr lastPort;

dp1("NewWindow\n");

	lastPort = thePort;
	thePort=(GrafPtr)NewPtr(sizeof(GrafPort));
	thePort->lastPort = lastPort;
	thePort->portKind = portWindow;
	
	if (s_Config.XFlag)
	{
		thePort->display = XOpenDisplay(NULL);
		thePort->screenNum = DefaultScreen(thePort->display);
		thePort->foreColor = BlackPixel(thePort->display, thePort->screenNum);
		thePort->backColor = WhitePixel(thePort->display, thePort->screenNum);
		thePort->win = XCreateSimpleWindow(thePort->display,
			RootWindow(thePort->display, thePort->screenNum),
			boundsRect->left, boundsRect->top,
			boundsRect->right - boundsRect->left,
			boundsRect->bottom - boundsRect->top,
			thePort->borderWidth,
			thePort->foreColor,
			thePort->backColor);

		sizeHints.flags = PPosition | PSize;
		sizeHints.x = boundsRect->left;
		sizeHints.y = boundsRect->top;
		sizeHints.width = boundsRect->right - boundsRect->left;
		sizeHints.height = boundsRect->bottom - boundsRect->top;

		XSetStandardProperties(thePort->display, thePort->win, "SCL Drawing",
			"", 0, 0, 0, &sizeHints);
		thePort->gc = XCreateGC(thePort->display, thePort->win, 0, &values);
		XSetForeground( thePort->display, thePort->gc, thePort->foreColor);
		XSetBackground( thePort->display, thePort->gc, thePort->backColor);

		XSelectInput( thePort->display, thePort->win,
			      ButtonPressMask | ButtonReleaseMask | KeyPressMask | ExposureMask );

		PenSize(1, 1);

		/* Color Map */
		thePort->colormap = DefaultColormap(
				thePort->display, thePort->screenNum);

		/* Backing Store */
		valuemask = CWBackingStore;
		attributes.backing_store = Always;
		XChangeWindowAttributes(thePort->display, thePort->win, valuemask,
					&attributes);

		if (visible)
			XMapRaised(thePort->display, thePort->win);
		XFlush(thePort->display);

		/* Pattern */
		for (i = 0; i < 39; i++)
		{
			s_tbSystemPattern[i] = XCreateBitmapFromData(
				thePort->display, thePort->win, 
				s_tbPatternData[i], 8, 8);
		}

		/* I don't know why, but setting bitmap
				at other program position fails */
	if (!s_Xt.checkMark)
		s_Xt.checkMark = XCreateBitmapFromData(
							thePort->display, thePort->win,
"\x01\x8\x02\x4\x04\x2\x08\x1\x90\x0\x60\x0\x60\x0\x90\x0\
\x08\x1\x04\x2\x02\x4\x01\x8",
							12, 12);
	if (!s_Xt.upArrow)
		s_Xt.upArrow = XCreateBitmapFromData(
							thePort->display, thePort->win,
							"\x18\x24\x42\xe7\x24\x24\x3c\x00",
							8, 8);
	if (!s_Xt.downArrow)
		s_Xt.downArrow = XCreateBitmapFromData(
							thePort->display, thePort->win,
							"\x00\x3c\x24\x24\xe7\x42\x24\x18",
							8, 8);

		white[0].pixmap = s_tbSystemPattern[20];
		white[0].num = 20;
		black[0].pixmap = s_tbSystemPattern[1];
		black[0].num = 1;
		gray[0].pixmap = s_tbSystemPattern[4];
		gray[0].num = 4;
		ltGray[0].pixmap = s_tbSystemPattern[23];
		ltGray[0].num = 23;
		dkGray[0].pixmap = s_tbSystemPattern[3];
		dkGray[0].num = 3;

		XAllocNamedColor(thePort->display, thePort->colormap,
			"black", &s_tb.basicColors[blackXColor], &dbColor);
		XAllocNamedColor(thePort->display, thePort->colormap,
			"white", &s_tb.basicColors[whiteXColor], &dbColor);
		XAllocNamedColor(thePort->display, thePort->colormap,
			"red", &s_tb.basicColors[redXColor], &dbColor);
		XAllocNamedColor(thePort->display, thePort->colormap,
			"green", &s_tb.basicColors[greenXColor], &dbColor);
		XAllocNamedColor(thePort->display, thePort->colormap,
			"blue", &s_tb.basicColors[blueXColor], &dbColor);
		XAllocNamedColor(thePort->display, thePort->colormap,
			"cyan", &s_tb.basicColors[cyanXColor], &dbColor);
		XAllocNamedColor(thePort->display, thePort->colormap,
			"magenta", &s_tb.basicColors[magentaXColor], &dbColor);
		XAllocNamedColor(thePort->display, thePort->colormap,
			"yellow", &s_tb.basicColors[yellowXColor], &dbColor);
	}


	/* Toolbox */
        portRect.left = 0;
        portRect.top = 0;
        portRect.right = boundsRect->right - boundsRect->left;
        portRect.bottom = boundsRect->bottom - boundsRect->top;
        thePort->portRect = portRect;

	thePort->pnLoc.h = 1;
	thePort->pnLoc.v = 1;
	thePort->pnSize.h = 1;
	thePort->pnSize.v = 1;
	thePort->pnMode = patCopy;
	thePort->pnPat[0] = black[0];
	thePort->pnVis = 1;

	thePort->txFont = 2;
	thePort->txSize = 12;
	thePort->txFace = plain;

	thePort->picSave = 0;
	thePort->rgnSave = 0;
	thePort->polySave = 0;


dp1("End: NewWindow\n");
	return thePort;
}



WindowPtr NewCWindow(Ptr wStorage, Rect *boundsRect,
			char *title, Boolean visible,
			int theProc, WindowPtr behind,
			Boolean goAwayFlag, long refcon)
{
	return NewWindow(wStorage, boundsRect,
			title, visible,
			theProc, behind,
			goAwayFlag, refcon);
}



/******************** Other Toolbox********************/

long TickCount()
{
#ifndef __MWERKS__
struct timeval s_tv;

	gettimeofday(&s_tv,NULL);
	return (long)(s_tv.tv_sec*60+s_tv.tv_usec*6/100000);
#else
	return 0; /* dummy to compile on mac */
#endif
}


void GetDateTime(unsigned long *secP)
{
	time((time_t *)secP);
}

void IUDateString(long sec, DateForm format,
			Str255 dateString)
{
struct tm *tmP;

	tmP = localtime((time_t *)&sec);
	strftime(dateString, 255, "%y.%m.%d", tmP);
}


void IUTimeString(long sec, Boolean secFlag,
			Str255 timeString)
{
struct tm *tmP;

	tmP = localtime((time_t *)&sec);
	strftime(timeString, 255, "%H:%S", tmP);
}


void DateString(long sec, DateForm format,
			Str255 dateString, Handle h)
{
	IUDateString(sec, format, dateString);
}


void TimeString(long sec, Boolean secFlag,
			Str255 timeString, Handle h)
{
	IUTimeString(sec, secFlag, timeString);
}


void SystemTask(void)
{
	if (s_Config.foreground)
	{
		fflush(stdout);
	}
}



/******************************************
	Parameters Dialog
	uses X Toolkit
*******************************************/
	
#define DITLITEM(dialog) \
(((ItemList*)(((DialogPeek)dialog)->items))->DITLItems)

#define DITLCOUNT(dialog) \
(((ItemList*)(((DialogPeek)dialog)->items))->dlgMaxIndex)

DialogPtr GetNewDialog(short resID, Ptr dStorage, WindowPtr behind)
{
DialogPeek newDialog;
int i;
int dRecNum;
int argc = 0;
char *argv[1] = {""};
DITLItem *dItem;


	newDialog = malloc(sizeof(DialogRecord));
	for (dRecNum = 0; SXDialogResource[dRecNum].resID;
			 dRecNum++)
	{
		if (SXDialogResource[dRecNum].resID == resID)
		{
			newDialog->recordNumber = dRecNum;
			SXGetDialogResourceInfo(SXDialogResource[dRecNum],
					newDialog);
			break;
		}
	}
	if (!SXDialogResource[dRecNum].resID)
	{
		PrintError("Dialog Resource not found");
		exit -1;
	}
	newDialog->resID = resID;
	newDialog->window.port.portKind = portDialog;
	newDialog->items = malloc((1 + SXDialogResource[dRecNum].itemCount)
							*sizeof(DITLItem) + sizeof(long));/*padding?*/
	dItem = DITLITEM(newDialog);
	for (i = 0; i <= SXDialogResource[dRecNum].itemCount; i++)
		dItem[i].inited = 0;
	if (!s_Xt.topWidget)	
		s_Xt.topWidget = XtVaAppInitialize(&(s_Xt.appContext),
						"SCL", 0, 0, &argc, argv, 0, 0);
	newDialog->shell = XtVaCreatePopupShell("shell",
						transientShellWidgetClass, s_Xt.topWidget,
						XtNx, newDialog->window.port.portRect.left,
						XtNy, newDialog->window.port.portRect.top,
						XtNwidth, newDialog->window.port.portRect.right
								- newDialog->window.port.portRect.left,
						XtNheight, newDialog->window.port.portRect.bottom
								- newDialog->window.port.portRect.top,
						0);
	newDialog->form = XtVaCreateManagedWidget("form", formWidgetClass,
						newDialog->shell, 0);
						
	SetDialogItems(newDialog, resID);

	return (DialogPtr)newDialog;
}


void SXGetDialogResourceInfo(SXDialogResourceItem dialogRes,
								DialogPeek newDialog)
{
WindowPeek w;
SXResourceDLOG r;
char tmpS[256];

	w = &(newDialog->window);
	r = dialogRes.DLOG;
	w->port.portRect = r.boundsRect;
	w->windowKind = r.procID;
	w->visible = r.visible;
	w->goAwayFlag = r.goAway;
	w->refCon = r.refCon;
	w->titleHandle = (Str255 *)malloc(sizeof(Str255));
	strcpy(w->titleHandle[0], r.title);
	newDialog->itemCount = dialogRes.itemCount;
}

int SetDialogItems(DialogPeek newDialog, short resID)
{
int resN;
SXDItemArgItem *dial; /* shorten the name */
char name[256];
Arg args[32];
Cardinal num_args = 0;
WidgetClass widget_class = 0;
DITLItem *dItem;
DITLItem *dir;
int itemN;

	dir = SXDItemResource;
	dItem = DITLITEM(newDialog);
	for (itemN = 0; dir[itemN].resID; itemN++)
	{
		if (dir[itemN].resID == resID)
		{
			dItem[dir[itemN].itemN] = dir[itemN];
		}
		dItem[dir[itemN].itemN].dialog = newDialog;
	}

	dial = SXDItemArgList;
	for(resN = 0; dial[resN].name != SXDItemArgListEnd;
			resN++)
	{
		if (dial[resN].name == SXDLOGStart
				&& dial[resN].value == resID)
			goto resourceFound;
	}
	return 0; /* resource not found */
	
resourceFound:
	for(resN++; dial[resN].name != SXDLOGEnd
			&& dial[resN].name != SXDItemArgListEnd; resN++)
	{
		switch((int)dial[resN].name)
		{
		case SXItemBegin:
			num_args = 0;
			break;
		case SXItemEnd:
			itemN = (int)dial[resN].value;
			SXDItemToWidget(resID, itemN, newDialog, &dItem[itemN],
							num_args, args);
			break;
		default:
			XtSetArg(args[num_args], dial[resN].name, dial[resN].value);
			num_args++;
			break;
		}
	}

	SXAccelerate(dItem[1].widget, newDialog->form);
	
	return 1; /* resource found */
}


void SXDItemToWidget(int resID, int itemN, DialogPeek dialog, DITLItem *dItem,
						Cardinal num_args, Arg args[])
{
char name[256];
WidgetClass widget_class = 0;
#define SXEditButtonMaxCharNum 11
char s[SXEditButtonMaxCharNum+1];
XtCallbackList editTextCallback;

	XtSetArg(args[num_args], XtNhorizDistance, dItem->itmRect.left);
	num_args++;
	XtSetArg(args[num_args], XtNvertDistance, dItem->itmRect.top);
	num_args++;
	XtSetArg(args[num_args], XtNwidth, dItem->itmRect.right
										- dItem->itmRect.left);
	num_args++;
	XtSetArg(args[num_args], XtNheight, dItem->itmRect.bottom
										- dItem->itmRect.top);
	num_args++;
	switch (dItem->itmType & 0x7f)
	{
	case kButtonDialogItem:
		widget_class = commandWidgetClass;
		XtSetArg(args[num_args], XtNlabel, (XtArgVal)(dItem->itmData.label));
		num_args++;
		XtSetArg(args[num_args], XtNshapeStyle, XmuShapeRoundedRectangle);
		num_args++;
		XtSetArg(args[num_args], XtNcornerRoundPercent, 35);
		num_args++;
		if ( 0 && 4 <= itemN && itemN <= 55)
		{
			XtSetArg(args[num_args], XtNwidth, 30);
			num_args++;
			XtSetArg(args[num_args], XtNheight, 14);
			num_args++;
		}
		break;
	case kStaticTextDialogItem:
		widget_class = labelWidgetClass;
		XtSetArg(args[num_args], XtNlabel, (XtArgVal)(dItem->itmData.label));
		num_args++;
		XtSetArg(args[num_args], XtNborderWidth, 0);
		num_args++;
		switch (dItem->itmData.value)
		{
		case 0:
			XtSetArg(args[num_args], XtNjustify, XtJustifyCenter);
			break;
		case 1:
			XtSetArg(args[num_args], XtNjustify, XtJustifyLeft);
			break;
		case 2:
			XtSetArg(args[num_args], XtNjustify, XtJustifyRight);
			break;
		}
		num_args++;
		XtSetArg(args[num_args], XtNvertDistance, dItem->itmRect.top+1);
		num_args++;
		break;		
	case kEditTextDialogItem:
		dItem->optWidget = calloc(2, sizeof(Widget));
		widget_class = asciiTextWidgetClass;
		XtSetArg(args[num_args], XtNeditType, XawtextEdit);
		num_args++;
		XtSetArg(args[num_args], XtNtopMargin, 1);
		num_args++;
		XtSetArg(args[num_args], XtNbottomMargin, 1);
		num_args++;
		XtSetArg(args[num_args], XtNleftMargin, 1);
		num_args++;
		XtSetArg(args[num_args], XtNrightMargin, 1);
		num_args++;
		XtSetArg(args[num_args], XtNuseStringInPlace, True);
		num_args++;
		XtSetArg(args[num_args], XtNlength, 255);
		num_args++;
		XtSetArg(args[num_args], XtNstring, (XtArgVal)(dItem->itmData.label));
		num_args++;
		editTextCallback = malloc(2*sizeof(XtCallbackRec));
		editTextCallback[0].callback = &SXEditTextCallback;
		editTextCallback[0].closure = dItem;
		editTextCallback[1].callback = 0;
		editTextCallback[1].closure = 0;
		XtSetArg(args[num_args], XtNcallback, editTextCallback);
		num_args++;
		sprintf(name, "dItem%dOpt0", itemN);
		dItem->optWidget[0] = XtCreateWidget(name, widget_class,
					dialog->form, args, num_args);
		widget_class = commandWidgetClass;
		num_args = 0;
		XtSetArg(args[num_args], XtNhorizDistance, dItem->itmRect.left);
		num_args++;
		XtSetArg(args[num_args], XtNvertDistance, dItem->itmRect.top);
		num_args++;
		XtSetArg(args[num_args], XtNwidth, dItem->itmRect.right
											- dItem->itmRect.left);
		num_args++;
		XtSetArg(args[num_args], XtNheight, dItem->itmRect.bottom
											- dItem->itmRect.top);
		num_args++;
		XtSetArg(args[num_args], XtNshapeStyle, XmuShapeRoundedRectangle);
		num_args++;
		XtSetArg(args[num_args], XtNcornerRoundPercent, 35);
		num_args++;
		if (strlen(dItem->itmData.label) > SXEditButtonMaxCharNum)
		{
			sprintf(s, "%.*s...", SXEditButtonMaxCharNum-3, dItem->itmData.label);
			XtSetArg(args[num_args], XtNlabel, s);
		}
		else
			XtSetArg(args[num_args], XtNlabel, dItem->itmData.label);
		num_args++;
		sprintf(name, "dItem%dOpt1", itemN);
		dItem->optWidget[1] = XtCreateWidget(name, widget_class,
					dialog->form, args, num_args);
		if (strlen(dItem->itmData.label) > SXEditButtonMaxCharNum)
			dItem->widget = dItem->optWidget[1];
		else
			dItem->widget = dItem->optWidget[0];
		XtManageChild(dItem->widget);
		break;		
	case kCheckBoxDialogItem:
		widget_class = commandWidgetClass;
		if (dItem->itmData.value)
			XtSetArg(args[num_args], XtNbitmap, s_Xt.checkMark);
		else
			XtSetArg(args[num_args], XtNbitmap, 0);
		num_args++;
		XtSetArg(args[num_args], XtNlabel, "  ");
		num_args++;
		XtSetArg(args[num_args], XtNinternalWidth, 0);
		num_args++;
		XtSetArg(args[num_args], XtNinternalHeight, 0);
		num_args++;
		XtSetArg(args[num_args], XtNwidth, 12);
		num_args++;
		XtSetArg(args[num_args], XtNheight, 12);
		num_args++;
		break;
	case kRadioButtonDialogItem:
		widget_class = toggleWidgetClass;
		XtSetArg(args[num_args], XtNlabel, dItem->itmData.label);
		num_args++;
		switch (dItem->itmData.value)
		{
		case 0:
			XtSetArg(args[num_args], XtNjustify, XtJustifyCenter);
			break;
		case 1:
			XtSetArg(args[num_args], XtNjustify, XtJustifyLeft);
			break;
		case 2:
			XtSetArg(args[num_args], XtNjustify, XtJustifyRight);
			break;
		}
		num_args++;
		if (s_Xt.radioGroup[dItem->itmData.value])
		{
			XtSetArg(args[num_args], XtNradioGroup,
				s_Xt.radioGroup[dItem->itmData.value]);
			num_args++;
		}
		/* else s_Xt.radiGroup will be set later */
		break;		
	case kResourceControlDialogItem:
		switch (dItem->itmData.value)
		{
		case rUpArrowDialogItem:
			widget_class = commandWidgetClass;
			XtSetArg(args[num_args], XtNbitmap, s_Xt.upArrow);
			num_args++;
			XtSetArg(args[num_args], XtNshapeStyle, XmuShapeRoundedRectangle);
			num_args++;
			XtSetArg(args[num_args], XtNcornerRoundPercent, 35);
			num_args++;
			break;
		case rDownArrowDialogItem:
			widget_class = commandWidgetClass;
			XtSetArg(args[num_args], XtNbitmap, s_Xt.downArrow);
			num_args++;
			XtSetArg(args[num_args], XtNshapeStyle, XmuShapeRoundedRectangle);
			num_args++;
			XtSetArg(args[num_args], XtNcornerRoundPercent, 35);
			num_args++;
			break;
		case rSimpleEditTextDialogItem:
			widget_class = asciiTextWidgetClass;
			XtSetArg(args[num_args], XtNeditType, XawtextEdit);
			num_args++;
			XtSetArg(args[num_args], XtNuseStringInPlace, True);
			num_args++;
			XtSetArg(args[num_args], XtNstring, (XtArgVal)(dItem->itmData.label));
			num_args++;
			XtSetArg(args[num_args], XtNlength, 255);
			num_args++;
			XtSetArg(args[num_args], XtNjustify, XtJustifyLeft);
			num_args++;
			XtSetArg(args[num_args], XtNwrap, XawtextWrapLine);
			num_args++;
			break;
		default:
			return;
		}
		break;
	default:
		return;
		break;
	}

	if ((dItem->itmType & 0x7f) != kEditTextDialogItem)
	{				
		sprintf(name, "dItem%d", itemN);
		dItem->widget = XtCreateManagedWidget(name, widget_class, dialog->form,
				args, num_args);
	}
	dItem->inited = 1;
	dItem->itmHndl = dItem;
	
	switch (dItem->itmType & 0x7f)
	{
	case kRadioButtonDialogItem:
		if (dItem->itmData.value)
			s_Xt.radioGroup[dItem->itmData.value] = dItem->widget;
		/*continue*/
	case kButtonDialogItem:
	case kCheckBoxDialogItem:
		XtAddCallback(dItem->widget, XtNcallback, &SXDItemHit,
							(XtPointer)itemN);
		break;
	case kResourceControlDialogItem:
		switch (dItem->itmData.value)
		{
		case rUpArrowDialogItem:
		case rDownArrowDialogItem:
			XtAddCallback(dItem->widget, XtNcallback, &SXDItemHit,
							(XtPointer)itemN);
			break;
		}
		break;
	case kEditTextDialogItem:
		if (!s_Xt.editButtonPopup)
		{
			s_Xt.editButtonPopup = SXMakeEditButtonPopup();
			XtAddCallback(s_Xt.editButtonPopup->button,
				XtNcallback, &SXPopdownText, s_Xt.editButtonPopup);
		}
		XtAddCallback(dItem->optWidget[1], XtNcallback,
						&SXEditButtonCallback, (XtPointer)dItem);
		break;
	default:
		break;
	}
}


void SXDItemHit(Widget widget, XtPointer clientData, XtPointer callerData)
{
	s_Xt.itemHit = (int)clientData;
}


void SXEditTextCallback(Widget widget, XtPointer clientData,
								 XtPointer callerData)
{
int length;

	length = strlen(((DITLItem *)clientData)->itmData.label);
	if (length > SXEditButtonMaxCharNum)
	{
		s_Xt.editButtonPopup->dItem = (DITLItem *)clientData;
		XtVaSetValues(s_Xt.editButtonPopup->text, XtNstring,
				s_Xt.editButtonPopup->dItem->itmData.label,
				XtNuseStringInPlace, True, 0);
		XtPopup(s_Xt.editButtonPopup->shell, XtGrabExclusive);
	}
}


void SXEditButtonCallback(Widget widget, XtPointer clientData,
								 XtPointer callerData)
{
	s_Xt.editButtonPopup->dItem = (DITLItem *)clientData;
	s_Xt.editButtonPopup->appear = 1;
	XtVaSetValues(s_Xt.editButtonPopup->text, XtNstring,
			s_Xt.editButtonPopup->dItem->itmData.label,
			XtNuseStringInPlace, True, 0);
	XtPopup(s_Xt.editButtonPopup->shell, XtGrabExclusive);
}

char sxTmpStr[256] = "abcd efg";

SXEditButtonPopupPtr SXMakeEditButtonPopup(void)
{
SXEditButtonPopupPtr P;

	P = malloc(sizeof(SXEditButtonPopupRec));
	P->shell = XtVaCreatePopupShell("editPopup",
			transientShellWidgetClass, s_Xt.topWidget, 0);
	P->form = XtVaCreateManagedWidget("form", formWidgetClass,
				P->shell, 0);
	P->text = XtVaCreateManagedWidget("text", asciiTextWidgetClass,
				P->form,
				XtNeditType, XawtextEdit, XtNlength, 255,
				XtNwrap, XawtextWrapLine,
				XtNstring, sxTmpStr,
				XtNuseStringInPlace, True,
				XtNwidth, 500, XtNheight, 50,
				XtNtop, XtChainTop, XtNbottom, XtChainBottom,
				XtNleft, XtChainLeft, XtNright, XtChainRight,
				XtNresizable, True,
				0);
	P->button = XtVaCreateManagedWidget("closeButton", commandWidgetClass,
					P->form, XtNlabel, "Close", XtNfromVert, P->text, 0);
	P->dItem = 0;
	P->appear = 0;
	return P;
}


void SXPopdownText(Widget widget, XtPointer clientData,
								 XtPointer callerData)
{
SXEditButtonPopupPtr P;
int length;
static char s[256];

	P = (SXEditButtonPopupPtr)clientData;
	XtUnmanageChild(P->dItem->widget);	
	length = strlen(P->dItem->itmData.label);
	if (length > SXEditButtonMaxCharNum)
	{
		P->dItem->widget = P->dItem->optWidget[1];
		sprintf(s, "%.*s...", SXEditButtonMaxCharNum-3, P->dItem->itmData.label);
		XtVaSetValues(P->dItem->widget, XtNlabel, s, 0);
	}
	else
	{
		P->dItem->widget = P->dItem->optWidget[0];
		XtVaSetValues(P->dItem->widget, XtNstring,
				P->dItem->itmData.label, 0);
	}
	XtManageChild(P->dItem->widget);
	XtPopdown(P->shell);
	P->appear = 0;
}


void SXAccelerate(Widget w, Widget form)
{	
char accTable[] = "#override\n<KeyPress>Return: set() notify()";
XtAccelerators acc;

	acc = XtParseAcceleratorTable(accTable);
	XtVaSetValues(w, XtNaccelerators, acc, 0);
	XtInstallAllAccelerators(form, w);
}

ControlHandle NewControl(WindowPtr theWindow, Rect *ctrlRect,
		Str255 title, Boolean visible, short initialValue,
		short minValue, short maxValue, short ctrlType,
		long refCon)
{
DITLItem **iHandle;

	iHandle = malloc(sizeof(DITLItem *));
	iHandle[0] = malloc(sizeof(DITLItem));

	iHandle[0]->itmRect = *ctrlRect;
	iHandle[0]->itmType = ctrlType|ctrlItem;
	switch(ctrlType)
	{
	case btnCtrl:
		strcpy(iHandle[0]->itmData.label, title);
		break;
	case chkCtrl:
		iHandle[0]->itmData.value = initialValue;
		break;
	default:
		break;
	}
	iHandle[0]->dialog = (DialogPeek)theWindow;
	iHandle[0]->NCFlag = 1;
	
	return iHandle;
}

void GetDialogItem(DialogPtr dialog, short itemN, short *iTypeP,
		Handle *iHandleP, Rect *iRectP)
{
DITLItem *dItem;

	dItem = DITLITEM(dialog);
	*iTypeP = dItem[itemN].itmType;
	*iHandleP = malloc(sizeof(DITLItem **));
	**((DITLItem ***)iHandleP) = &dItem[itemN];
	*iRectP = dItem[itemN].itmRect;
}


void SetDialogItem(DialogPtr dialog, short itemN, short iType,
		Handle iHandle, Rect *iRect)
{
DITLItem *dItem;
Cardinal num_args = 0;
Arg args[64];

	dItem = DITLITEM(dialog);
	dItem[itemN].itmType = iType;
	strcpy(dItem[itemN].itmData.label, ((DITLItem **)iHandle)[0]->itmData.label);
	if (((DITLItem **)iHandle)[0]->NCFlag)
		free(((DITLItem **)iHandle)[0]);
	*((DITLItem **)iHandle) = &dItem[itemN];
	dItem[itemN].itmRect = *iRect;

	if (!dItem[itemN].inited)
		SXDItemToWidget(((DialogPeek)dialog)->resID, itemN, (DialogPeek)dialog, &dItem[itemN],
							num_args, args);
}


void GetDialogItemText(Handle iHandle, Str255 textStr)
{
DITLItem *dItem;

	dItem = *((DITLItem **)iHandle);
	strcpy(textStr, dItem->itmData.label);
}

void SetDialogItemText(Handle iHandle, Str255 textStr)
{
DITLItem *dItem;
Cardinal num_args = 0;
Arg args[32];
static char s[SXEditButtonMaxCharNum+1];

	dItem = *((DITLItem **)iHandle);
	strcpy(dItem->itmData.label, textStr);

	if (!dItem->inited)
		SXDItemToWidget(dItem->resID, dItem->itemN, dItem->dialog, dItem,
							num_args, args);
	else
	{
		switch(dItem->itmType & 63)
		{
		case kStaticTextDialogItem:
			XtVaSetValues(dItem->widget, XtNlabel, dItem->itmData.label, 0);
			break;
		case kEditTextDialogItem:
			XtUnmanageChild(dItem->widget);
			if (strlen(dItem->itmData.label) > SXEditButtonMaxCharNum)
			{
				dItem->widget = dItem->optWidget[1];
				sprintf(s, "%.*s...", SXEditButtonMaxCharNum-3,
											 dItem->itmData.label);
				XtVaSetValues(dItem->widget, XtNlabel, s, 0);
			}
			else
			{
				dItem->widget = dItem->optWidget[0];
				XtVaSetValues(dItem->widget, XtNstring, dItem->itmData.label, 0);
				XtVaSetValues(dItem->widget, XtNlength, 255, 0);
			}
			XtManageChild(dItem->widget);
			break;
		default:
			break;
		}
	}
}


void SelectDialogItemText(DialogPtr dialog, short item, short from, short to)
{
/*Do nothing on UNIX*/
}



short GetControlValue(ControlHandle control)
{
DITLItem *dItem;

	dItem = ((DITLItem **)control)[0];
	switch (dItem->itmType)
	{
	case kButtonDialogItem:
		return 0;
	case kCheckBoxDialogItem:
		return dItem->itmData.value;
	default:
		return 0;
	}
	return 0;
}

void SetControlValue(ControlHandle control, short value)
{
DITLItem *dItem;
Cardinal num_args = 0;
Arg args[32];

	dItem = ((DITLItem **)control)[0];
	switch (dItem->itmType)
	{
	case kButtonDialogItem:
		break;
	case kCheckBoxDialogItem:
		dItem->itmData.value = value;
		if (!dItem->inited)
			SXDItemToWidget(dItem->dialog->resID, dItem->itemN, dItem->dialog,
				dItem, num_args, args);
		if (value)
			XtVaSetValues(dItem->widget, XtNbitmap, s_Xt.checkMark, 0);
		else
			XtVaSetValues(dItem->widget, XtNbitmap, 0, 0);
		break;
	default:
		break;
	}
}


void ModalDialog(ProcPtr filter, short *itemHit)
{
XEvent event;
KeySym key;

	s_Xt.itemHit = 0;
	while (s_Xt.itemHit == 0)
	{
		XtAppNextEvent(s_Xt.appContext, &event);
		if (event.type == KeyPress)
		{
			XLookupString((XKeyEvent *)&event, 0, 0, &key, 0);
			if (key == XK_Return)
			{
				*itemHit = 1;
				return;
			}
		}
		XtDispatchEvent(&event);
	}
	*itemHit = s_Xt.itemHit;
}


void SelectWindow(WindowPtr window)
{
	window->lastPort = thePort;
	thePort = window;
}


void ShowWindow(WindowPtr window)
{
	if (window->portKind == portDialog)
	{
		SXPopUpDialog(window);
	}
}


void HideWindow(WindowPtr window)
{
	if (window->portKind == portDialog)
	{
		SXPopDownDialog(window);
	}
	if (thePort == window)
		thePort = window->lastPort;
}


void SXPopUpDialog(WindowPtr dialog)
{
	XtPopup(((DialogPeek)dialog)->shell, XtGrabExclusive);
}


void SXPopDownDialog(WindowPtr dialog)
{
XEvent event;

	if (s_Xt.editButtonPopup && s_Xt.editButtonPopup->appear)
		XtPopdown(s_Xt.editButtonPopup->shell);
	XtPopdown(((DialogPeek)dialog)->shell);
	while (XtAppPending(s_Xt.appContext))
	{
		XtAppNextEvent(s_Xt.appContext, &event);
		XtDispatchEvent(&event);
	}
}


/******* Memory *******/

void HLock(void *handle)
{
}

void HUnlock(void *handle)
{
}

#endif /*S_UNIX*/
#endif /*pragma once*/
